/**
 * 搭配以下插件使用
 * @param jQuery jquery-1.9.1.min.js
 * @param pinchZoom pinch-zoom.js
 */
(function (factory) {
	/* CommonJS module. */
	if (typeof module === 'object' && typeof module.exports === 'object') {
		module.exports = factory(window);
		/* AMD module. */
	} else if (typeof define === 'function' && define.amd) {
		define(factory(window));
		/* Browser globals. */
	} else {
		factory(window);
	}
})(function (global, undefined) {
	'use strict';

	var veImagePrototype = veImage.prototype;
	//默认参数
	var defaults = {
		canvas: null, //画布
		el: null, //画布父元素
		image: null, //图片
		container: null, // 容器
		// ctWidth: 375, // 容器宽度
		// ctHeight: 370, // 容器高度
	};

	//绑定的事件 目前只适用于webkit内核浏览器
	var events = {
		start: 'touchstart',
		move: 'touchmove',
		end: 'touchend',
		click: 'click',
	};

	/**
	 * 简单的数组合并
	 */
	function extend (source, target) {
		for (var key in source) {
			if (source.hasOwnProperty(key)) target[key] = source[key];
		}
		return target;
	}

	function veImage (opts) {
		if (!opts.canvas || !opts.image) {
			throw new Error('请传入正确的画布或图片');
		}

		this.opts = extend(opts, defaults); //默认参数与传入参数合并
		this.canvas = this.opts.canvas; //画布
		this.container = this.opts.container; // 容器
		this.ctWidth = $(this.container).width(); // 容器宽度
		this.ctHeight = $(this.container).height(); // 容器高度
		this.rate = 1; // 图片缩放比例
		// this.size = this.canvas.getBoundingClientRect(); // 画布的尺寸参数
		this.size = {}; // 画布的尺寸参数
		this.context = this.canvas.getContext('2d'); // 画布上下文
		this.image = opts.image; // 图片地址或base64编码
		this.mode = {}; // 模式
		this.el = null; // 画布父元素
		this.tailor = {}; // 裁图框参数
		this.data = []; // 坐标数据

		// 同步读取图片
		this._readPic().then(() => {
			this._draw(); // 画图
			this._alt(); // 生成截图框
			this._bind(); // 绑定手指事件
		});
	}

	/**
	 * 记录手指的坐标
	 * @param touches
	 * @private
	 */
	function _finger (touches) {
		var coordinate = [],
			len = touches.length;
		for (var i = 0; i < len; i++) {
			//TouchList类型 不能用forEach
			coordinate.push({
				x: touches[i].clientX, // 当前手指页面移动位置 X 轴距离
				y: touches[i].clientY, // 当前手指页面移动位置 Y 轴距离
			});
		}
		return coordinate;
	}

	/**
	 * 绑定开始事件
	 */
	veImagePrototype.startEvt = function (e) {
		this.type = $(e.target).data().type || ''; // 记录当前要移动的模块
		this.type === 'tailor' ? (this.start = _finger(e.touches)) : (this.startImg = _finger(e.touches)); //记录开始的坐标
		this._mode(e.touches); //模式初始化
	};

	/**
	 * 绑定手指移动缩放事件
	 */
	veImagePrototype.moveEvt = function (e) {
		// e.preventDefault(); //禁止滚动

		var fingers = _finger(e.touches); //记录移动中的坐标
		//不能仅仅通过手指数量来判断 因为当缩放后，移除手指的时候，会出现一个手指还停留在页面上，导致位移
		if (this.type === 'tailor') {
			// 移动裁剪框
			if (this.mode.tailor == 1) {
				//位移
				this._translate(fingers, 'tailor');
				//将start的坐标复为移动中的坐标 时时计算偏移值，不然会变得非常大，图片在移动中会飞出画面
				this.start = fingers;
			} else if (this.mode.tailor == 2) {
				//缩放
				// this._zoom(fingers);
			}
		} else {
			// 移动图片
			if (this.mode.image == 1) {
				// 位移
				this._translate(fingers);
				// 将start的坐标复为移动中的坐标 时时计算偏移值，不然会变得非常大，图片在移动中会飞出画面
				this.startImg = fingers;
			} else if (this.mode.image == 2) {
				//缩放
				this._zoom(fingers);
			}
		}
	};

	/**
	 * 模式判断
	 */
	veImagePrototype._mode = function (touches) {
		var mode; // 模式
		if (touches.length == 1) {
			mode = 1; //位移模式
		} else if (touches.length > 1) {
			mode = 2; //缩放模式
		}
		// 区分模式
		if (this.type === 'tailor') this.mode.tailor = mode;
		else this.mode.image = mode;
	};

	/**
	 * 位移
	 */
	veImagePrototype._translate = function (fingers) {
		//计算手指的位移
		if (this.type === 'tailor') {
			this._drawEasel(fingers[0].x - this.start[0].x, fingers[0].y - this.start[0].y);
		} else {
			this._draw(fingers[0].x - this.startImg[0].x, fingers[0].y - this.startImg[0].y);
		}
	};

	/**
	 * 绑定结束事件
	 */
	veImagePrototype.endEvt = function (e) { };

	/**
	 * 裁剪
	 * @param w 裁剪的宽度
	 * @param h 裁剪的高度
	 * @param x 开始裁剪的X轴坐标
	 * @param y 开始裁剪的Y轴坐标
	 * @returns {string}
	 */
	veImagePrototype.crop = function () {
		var canvas = document.createElement('canvas'),
			context = canvas.getContext('2d');
		// rate = this.canvas.width / this.opts.relativeWidth; //当前画布的宽度与设置的相对宽度 计算比率
		// x *= rate;
		// y *= rate;
		canvas.width = this.tailor.w;
		canvas.height = this.tailor.h;
		// canvas.width = this.tailor.w * this.putRate;
		// canvas.height = this.tailor.h * this.putRate;
		context.drawImage(this.canvas, this.tailor.x * this.putRate, this.tailor.y * this.putRate, this.tailor.w * this.putRate, this.tailor.h * this.putRate, 0, 0, this.tailor.w, this.tailor.h);
		// context.drawImage(
		// 	this.canvas,
		// 	this.tailor.x * this.putRate,
		// 	this.tailor.y * this.putRate,
		// 	this.tailor.w * this.putRate,
		// 	this.tailor.h * this.putRate,
		// 	0,
		// 	0,
		// 	this.tailor.w * this.putRate,
		// 	this.tailor.h * this.putRate
		// );
		let base64 = canvas.toDataURL('image/jpg');

		// let imageGzip = new Image();
		// imageGzip.src = base64;
		// imageGzip.onload = function () {
		// 	console.log(this.width);
		// 	console.log(this.height);
		// };

		// 查看图片大小
		let stringLength = base64.replace(/^data:image\/\w+;base64,/, '').length;
		let sizeInBytes = 4 * Math.ceil(stringLength / 3) * 0.5624896334383812;
		let sizeInKb = sizeInBytes / 1024;
		console.log('画布裁剪大小', sizeInKb + 'kb');
		// context.drawImage(this.canvas, x, y, canvas.width, canvas.height, 0, 0, canvas.width, canvas.height);
		return { base64Data: base64, ...this.tailor, cw: this.fixedWidth, ch: this.fixedHeight, rate: this.putRate };
	};

	/**
	 * 画图
	 * @param dx X轴偏移量
	 * @param dy Y轴偏移量
	 * @param zoomWidth 宽度缩放比
	 * @param zoomHeight 高度缩放比
	 * @private
	 */
	veImagePrototype._draw = function (dx, dy, zoomWidth, zoomHeight) {
		//清空画布 擦除之前绘制的所有内容 不清空的话会显示各个步骤的画布
		this.context.clearRect(0, 0, this.imgData.width, this.imgData.height);
		console.log(this.imgData.width, this.imgData.height);
		this.canvas.width = this.imgData.width;
		this.canvas.height = this.imgData.height;
		// this.canvas.width = this.fixedWidth;
		// this.canvas.height = this.fixedHeight;

		// this.context.save(); //保存状态

		//画图
		// this.context.drawImage(this.imgData, 0, 0, this.imgData.width, this.imgData.height, 0, 0, this.dWidth, this.dHeight); //目标画布的中心点
		this.context.drawImage(this.imgData, 0, 0, this.imgData.width, this.imgData.height); // 目标画布的中心点
		this.canvas.style.width = this.fixedWidth + 'px';
		this.canvas.style.height = this.fixedHeight + 'px';

		//还原到上一个状态 也就是空画布的时候
		// this.context.restore();
	};

	/**
	 * 移动裁剪框
	 * @param dx X轴偏移量
	 * @param dy Y轴偏移量
	 */
	veImagePrototype._drawEasel = function (dx, dy) {
		// 计算裁剪框需要移动到的位置
		var shiftLocation = {
			x: this.tailor.x + dx,
			y: this.tailor.y + dy,
		};

		// 移动 X 轴
		if (shiftLocation.x > 0 && shiftLocation.x + this.tailor.w < this.dWidth) {
			// 偏移裁图框位置
			$('.tailor-case').css({ left: shiftLocation.x });
			// 偏移图片位置
			$('.tailor-img').css({ left: -shiftLocation.x });
			this.tailor.x = shiftLocation.x;
		}

		// 移动 Y 轴
		if (shiftLocation.y > 0 && shiftLocation.y + this.tailor.h < this.dHeight) {
			// 偏移裁图框位置
			$('.tailor-case').css({ top: shiftLocation.y });
			// 偏移图片位置
			$('.tailor-img').css({ top: -shiftLocation.y });
			this.tailor.y = shiftLocation.y;
		}
	};

	/**
	 * 生成截图框
	 */
	veImagePrototype._alt = function () {
		// 添加父盒子
		$(this.canvas).wrap('<div class="canvas-box"></div>');
		// 生成裁剪区域
		var tailorHtml = `
				<div class="tailor-mask"></div>
				<div class="tailor-case">
					<img src="${this.image}" alt="" class="tailor-img"  data-type="tailor">
					<div class="top-pull" data-type="top-pull"></div>
					<div class="bottom-pull" data-type="bottom-pull"></div>
					<div class="left-pull" data-type="left-pull"></div>
					<div class="right-pull" data-type="right-pull"></div>
					<div class="left-top-pull" data-type="left-top-pull"></div>
					<div class="left-bottom-pull" data-type="left-bottom-pull"></div>
					<div class="right-top-pull" data-type="right-top-pull"></div>
					<div class="right-bottom-pull" data-type="right-bottom-pull"></div>
				</div>`;
		$('.canvas-box').append(tailorHtml); // 生成裁图区域
		this.tailor.el = $('.tailor-case').get(0); // 获取到裁图元素

		// 设置裁图框大小位置
		$('.tailor-case').css({
			width: this.tailor.w,
			height: this.tailor.h,
			left: this.tailor.x,
			top: this.tailor.y,
		});
		// 设置图片大小及位置
		$('.tailor-img').css({
			width: this.dWidth,
			height: this.dHeight,
			left: -this.tailor.x,
			top: -this.tailor.y,
		});
	};

	/**
	 * 读取图片设置画布宽高和图片宽高
	 */
	veImagePrototype._readPic = function () {
		// 同步读取文件
		return new Promise((resolve, reject) => {
			var img = new Image();
			img.src = this.image;
			img.onload = () => {
				this.imgData = img; // 保存读取后的图片数据
				var rate, imgWidth, imgHeight, imgZoomWidth, imgZoomHeight;
				imgWidth = img.width;
				imgHeight = img.height;
				rate = this.ctWidth / imgWidth;
				imgZoomWidth = this.ctWidth;
				imgZoomHeight = rate * imgHeight;
				if (this.ctHeight < imgZoomHeight) {
					imgZoomHeight = this.ctHeight;
					rate = imgZoomHeight / imgHeight;
					imgZoomWidth = imgWidth * rate;
				}
				// 原图缩小比例
				this.shrinkRate = imgZoomWidth / img.width;
				// 图片放大比例
				this.putRate = img.width / imgZoomWidth;
				// 记录固定宽高
				this.fixedWidth = imgZoomWidth;
				this.fixedHeight = imgZoomHeight;
				//设置目标宽高 也就是缩放后的宽高
				this.dWidth = imgZoomWidth;
				this.dHeight = imgZoomHeight;
				// 设置画布宽高
				this.canvas.width = this.fixedWidth;
				this.canvas.height = this.fixedHeight;
				// 获取画布尺寸产数
				this.size = this.canvas.getBoundingClientRect();
				//将画布的中心点 作为起始坐标 缩放的时候会从中间开始放大，效果更佳，图片也能画在正中间
				this.origin = {
					x: this.dWidth / 2,
					y: this.dHeight / 2,
				};

				// 裁图区域初始位置及宽高
				this.tailor.w = this.dWidth; // 起始宽
				this.tailor.h = this.dHeight; // 起始高
				this.tailor.x = 0; // 起始位置
				this.tailor.y = 0; // 起始位置

				resolve(true);
			};
		});
	};

	/**
	 * 裁图框边框开始拉动
	 */
	veImagePrototype.tailorStartEvt = function (e) {
		this.tailorStart = _finger(e.touches);
	};

	/**
	 * 裁图框边框拉动中
	 */
	veImagePrototype.tailorMoveEvt = function (e) {
		// 静止滚动
		// e.preventDefault();

		// 获取自定义属性
		var type = $(e.target).data().type;
		// 双指操作不能缩放,单指操作移动
		if (this.tailorStart.length === 1) {
			// 计算偏移量
			var fingers = _finger(e.touches),
				dx = fingers[0].x - this.tailorStart[0].x,
				dy = fingers[0].y - this.tailorStart[0].y,
				bd = 24 * 3 - 3; // 根据边框设置最小宽度
			this.tailorStart = fingers;

			// 根据自定义属性值判断当前要拉动的位置
			switch (type) {
				case 'top-pull': // 上边框
					if (this.tailor.y + dy >= 0 && this.tailor.h - dy > bd) {
						this.tailor.h -= dy;
						this.tailor.y += dy;
					}
					break;
				case 'bottom-pull': // 下边框
					if (this.tailor.h + dy <= this.canvas.height / this.putRate - this.tailor.y && this.tailor.h + dy > bd) {
						this.tailor.h += dy;
					}
					break;
				case 'left-pull': // 左边框
					if (this.tailor.x + dx >= 0 && this.tailor.w - dx > bd) {
						this.tailor.w -= dx;
						this.tailor.x += dx;
					}
					break;
				case 'right-pull': // 右边框
					if (this.tailor.w + dx <= this.canvas.width / this.putRate - this.tailor.x && this.tailor.w + dx > bd) {
						this.tailor.w += dx;
					}
					break;
				case 'left-top-pull': // 左上边框
					if (this.tailor.x + dx >= 0 && this.tailor.w - dx > bd) {
						this.tailor.w -= dx;
						this.tailor.x += dx;
					}
					if (this.tailor.y + dy >= 0 && this.tailor.h - dy > bd) {
						this.tailor.h -= dy;
						this.tailor.y += dy;
					}
					break;
				case 'right-top-pull': // 右上边框
					if (this.tailor.w + dx <= this.canvas.width / this.putRate - this.tailor.x && this.tailor.w + dx > bd) {
						this.tailor.w += dx;
					}
					if (this.tailor.y + dy >= 0 && this.tailor.h - dy > bd) {
						this.tailor.h -= dy;
						this.tailor.y += dy;
					}
					break;
				case 'left-bottom-pull': // 左下边框
					if (this.tailor.x + dx >= 0 && this.tailor.w - dx > bd) {
						this.tailor.w -= dx;
						this.tailor.x += dx;
					}
					if (this.tailor.h + dy <= this.canvas.height / this.putRate - this.tailor.y && this.tailor.h + dy > bd) {
						this.tailor.h += dy;
					}
					break;
				case 'right-bottom-pull': // 右下边框
					if (this.tailor.w + dx <= this.canvas.width / this.putRate - this.tailor.x && this.tailor.w + dx > bd) {
						this.tailor.w += dx;
					}
					if (this.tailor.h + dy <= this.canvas.height / this.putRate - this.tailor.y && this.tailor.h + dy > bd) {
						this.tailor.h += dy;
					}
					break;
			}
			// 偏移裁图框位置
			$('.tailor-case').css({ left: this.tailor.x, top: this.tailor.y, width: this.tailor.w, height: this.tailor.h });
			$('.tailor-img').css({ left: -this.tailor.x, top: -this.tailor.y });
		}
	};

	/**
	 * 绑定事件 添加事件
	 */
	veImagePrototype._bind = function (e) {
		this.tailor.el.addEventListener(events.start, this);
		this.tailor.el.addEventListener(events.move, this);
		this.tailor.el.addEventListener(events.end, this);
		this.canvas.addEventListener(events.start, this);
		this.canvas.addEventListener(events.move, this);
		this.canvas.addEventListener(events.end, this);
		$('.tailor-case div').get(0).addEventListener(events.start, this);
		$('.tailor-case div').get(0).addEventListener(events.move, this);
		// this.canvas.addEventListener(events.click, this);
	};

	/**
	 * 高级的绑定方法 触发事件
	 */
	veImagePrototype.handleEvent = function (e) {
		switch (e.type) {
			case events.start: // 开始触屏
				if ($(e.target).data().type.indexOf('pull') > -1) {
					this.tailorStartEvt(e);
				} else {
					this.startEvt(e);
				}
				break;
			case events.move: // 触屏移动
				if ($(e.target).data().type.indexOf('pull') > -1) {
					this.tailorMoveEvt(e);
				} else {
					this.moveEvt(e);
				}
				break;
			case events.end: // 触屏结束
				this.endEvt(e);
				break;
		}
	};

	global.veImage = veImage;
	return veImage;
});
