/**
 * 此处主要处理ws的响应事件和请求事件。
 * 响应码和请求码也在这。
 * @constructor
 */


var SiteConHandler = function () {
    var self = this;
    this.ws = null;

    // 连接状态，在app端使用。 用途是判断 手机端 是否连接了socket，如果没有连接则改用post请求来侦听web端在线状态。
    this.conn_state = true;

    this.onopen = function (headler, revent, ws) {
        self.ws = ws;
        console.log('连接成功');
    };

    /**
     * 收到服务器发来的消息
     * @param {*} headler 当前headler对象
     * @param {*} event 返回数据
     * @param {*} ws websocket对象
     */
    this.onmessage = function (headler, event, ws) {
        var data = JSON.parse(event.data);
        self.respCodeHandler(data);
    };

    this.onclose = function (headler, event, ws) {
        if (self.conn_state !== false) {
            self.conn_state = false;
            self.callback.connState(self.conn_state);
        }
        console.log('关闭事件');
    };

    this.onerror = function (headler, event, ws) {
        console.log('错误事件');
        self.connState = false;
    };

    /**
     * 发送心跳，本框架会自动定时调用该方法，请在该方法中发送心跳
     * @param {*} ws
     */
    this.ping = function (ws) {
        ws.send(JSON.stringify({ status: 0,company_id: fun.getItem('company_info')['company_id']}));
    };

    /**
     * 处理各种响应的中转
     * @param data
     */
    this.respCodeHandler = function (data) {
        switch (data.status) {
            case ResponseCode.connect_success.status:
                self.conn_state = true;
                self.callback.connState(true);
                break;
            case ResponseCode.operation_success.status:
            case ResponseCode.request_error.status:
            case ResponseCode.account_offline.status:
            case ResponseCode.web_offline.status: // web端不在线
                var tipsMsg = data['msg'];
                outlin({
                    content: tipsMsg || '提示语...'
                })
                .then(function(){
                    fun.ajax({
                        showLoad: true,
                        hideLoad: true,
                        url: AJAX_URL.logout,
                        data: {},
                        success: function (res) {
                        }
                    });
                    self.ws.setAutoReconnect();
                })
                break;
            case ResponseCode.max_count.status: // 站点满员
                var tipsMsg = data['num'];
                outlin({
                    content: '当前公司可用端口 ' + tipsMsg + ' 个，现已满员。<br />如需增加端口，请与我司联系，谢谢！' || '提示语...'
                })
                .then(function(){
                    fun.ajax({
                        showLoad: true,
                        hideLoad: true,
                        url: AJAX_URL.logout,
                        data: {},
                        success: function (res) {
                        }
                    });
                    self.ws.setAutoReconnect();
                })
                break;
            case ResponseCode.param_error.status: // 参数错误
            case ResponseCode.mobile_forced_offline.status: // 被强制下线
            case ResponseCode.company_forced_offline.status:
                var tipsMsg = data['msg'];
                outlin({
                    content: tipsMsg
                })
                .then(function(){
                    fun.ajax({
                        showLoad: true,
                        hideLoad: true,
                        url: AJAX_URL.logout,
                        data: {},
                        success: function (res) {
                        }
                    });
                    self.ws.setAutoReconnect();
                })
                break;
            case ResponseCode.elsewhere_login.status: // 在其他地方登录;
            case ResponseCode.cloud_close_file.status://云安全-数据归档
                // var tipsMsg = data['msg'];  
                var tipsMsg = data.status == ResponseCode.cloud_close_file.status ? ResponseCode.cloud_close_file.msg : '当前账号已退出！\<br\>*可能出现网络波动，或者重复登录等原因';
                outlin({
                    content: tipsMsg
                })
                .then(function(){
                    fun.ajax({
                        showLoad: true,
                        hideLoad: true,
                        url: AJAX_URL.logout,
                        data: {},
                        success: function (res) {
                        }
                    });
                    self.ws.setAutoReconnect();
                })
                break;

            case ResponseCode.request_show_company_success.status:
                self.callback.showCompanyList(data);
                break;
            case ResponseCode.web_online.status: // 用户上线 在线 下线
                self.callback.webOnline(data);
                self.callback.webIsOnl(true);
                break;
            case ResponseCode.web_upline.status:
                self.callback.webUpline(data);
                self.callback.webIsOnl(true);
                break;
            case ResponseCode.web_downline.status:
                self.callback.webDownline(data);
                self.callback.webIsOnl(false);
                break;
            case ResponseCode.web_refresh_state.status:
                self.callback.refreshWebState(data);
                break;
            case ResponseCode.cloud_close_offline.status:
                fun.showLoad('关闭中...');
                fun.delAllItem();//清除缓存
                //退出软件
                setTimeout(function(){
                    fun.hideLoad();
                    api.closeWidget({
                        silent: true,
                        id: 'A6092659618839',
                    });
                },1500)
                break;
        }
    };

    /**
     * 处理内容的回调函数
     */
    this.callback = {
        /**
         * 会接收到公司的在线列表数据
         * @param data
         */
        showCompanyList: function (data) { console.warn(data); }, // 接收公司的在线列表数据

        webOnline: function (data) { console.warn(data); }, // web端用户在线
        webUpline: function (data) { console.warn(data); }, // web端上线
        webDownline: function (data) { console.warn(data); },// web端下线
        refreshWebState: function (data) { console.warn(data); }, // 更新webd端

        connState: function (state) { }, // 用于通知手机端socket是否建立连接
        webIsOnl: function (state) { },// 用于通知wbe上下线的消息
        cloudServices: function (state) { }//关闭云服务
    };

    
};

RequestCode = {
    forced_offline_web: 5000, // 请求将自己的web端下线
    forced_offline_company: 5001, // 请求将公司内的某个用户的某端下线 【参数：用户id， 登录类型】
    show_company_online: 5010, // 请求查看公司当前的在线列表 【参数：登录类型】
    refresh_web_state: 5011, // 刷新电脑端在线状态
};

//打开提示弹窗,退出登录 (用途：下线之类)
function outlin(params){
    return new Promise(function(resolve, reject){
        api.sendEvent({
            name: 'site',
            extra: {
                msg: params,
            }
        });
        return resolve()
    })
}

// 响应码
ResponseCode = {
    connect_success: {
        'status': 6000,
        'msg': '连接成功'
    },
    operation_success: {
        'status': 6001,
        'msg': '操作成功'
    },
    request_success: {
        'status': 6002,
        'msg': '数据请求成功'
    },
    request_show_company_success: {
        'status': 6003,
        'msg': '公司在线列表'
    },

    param_error: {
        'status': 6010,
        'msg': '参数有误'
    },
    request_error: {
        'status': 6011,
        'msg': '请求参数有误'
    },
    account_offline: {
        'status': 6012,
        'msg': '找不到此用户或此用户未在线'
    },

    max_count: {
        'status': 6020,
        'msg': '此站点已满员'
    },

    elsewhere_login: {
        'status': 6030,
        'msg': '此账号已被登录'
    },

    web_upline: {
        'status': 6050,
        'msg': '您的账号在电脑端登录'
    },
    web_online: {
        'status': 6051,
        'msg': '电脑端已登录'
    },
    web_downline: {
        'status': 6052,
        'msg': '您的账号在电脑端下线'
    },
    web_offline: {
        'status': 6053,
        'msg': '您的账号未在电脑端登录'
    },
    web_refresh_state: {
        'status': 6054,
        'msg': '当前电脑端状态'
    },

    mobile_forced_offline: {
        'status': 6060,
        'msg': '您已被手机端强制下线'
    },
    company_forced_offline: {
        'status': 6061,
        'msg': '您已被管理员强制下线'
    },
    cloud_close_offline: {
        'status': 6062,
        'msg': '云服务关闭强制所有设备下线'
    },
    cloud_close_file: {//云安全数据归档强制下线
        'status': 6074,
        'msg': '小云已搭乘808航班去追寻诗和远方了'
    },
};
