/**
 * 图片处理工具类
 * @param options
 * @constructor
 */
function ImageHelper(options) {

    var that = this;
    var canvasContext = [];                                           // 图片上下文内容
    var imageResources = [];

    var defaultOptions = {
        imageSrc: options.imageSrc,                             // 图片链接，string或者array
        width: options.width || 560,                            // 目标图片宽度
        height: options.height || 310                           // 目标图片高度
    };

    var Fun = {
        initContext: function () {
            var imageCanvas = document.createElement('canvas');
            imageCanvas.width = defaultOptions.width;
            imageCanvas.height = defaultOptions.height;

            return {
                canvasDom: imageCanvas,
                context: imageCanvas.getContext("2d")
            }
        },

        initImage: function (imageSrc, callback) {
            var image = new Image();
            image.src = imageSrc;
            image.onload = function () {
                callback(image);
            };
        }
    };

    /**
     * 初始化canvas
     */
    var init = function () {
        // 初始化canvas
        if (typeof defaultOptions.imageSrc === 'string') {
            imageResources.push(defaultOptions.imageSrc);
        } else {
            imageResources = defaultOptions.imageSrc;
        }

        // 循环创建canvas
        for(var i = 0; i < imageResources.length; i++) {
            (function(i) {
                var canvas = Fun.initContext();
                Fun.initImage(imageResources[i], function (image) {
                    // 在画布中心绘制图片
                    canvas.context.fillStyle = '#fff';
                    canvas.context.drawImage(image, defaultOptions.width / 2 - image.width / 2, defaultOptions.height / 2 - image.height / 2);
                    canvas.image = image;
                    canvasContext[i] = canvas;
                });
            })(i);
        }
    };


    // 对外开发方法 ----------------------------------------------------------------------------------------------------
    /**
     * 图片缩放,固定宽高
     */
    that.imageZoom = function () {
        // 1. 图片放大处理， 取图片跟容器宽比，高比最小的一个来处理
        // 2. 图片缩小处理， 取图片跟容器宽比，高比最大的一个来处理
        if (canvasContext.length === imageResources.length) {
            var containerWidth = defaultOptions.width,
                containerHeight = defaultOptions.height;

            var imageBase64Arr = [];
            for (var i = 0; i < canvasContext.length; i++) {
                var image = canvasContext[i].image;
                var canvas = canvasContext[i].context;
                var canvasDom = canvasContext[i].canvasDom;
                var imgWidth = image.width;
                var imgHeight = image.height;

                var ratio = 1;
                var widthRatio = containerWidth / imgWidth;
                var heightRatio = containerHeight / imgHeight;
                if (widthRatio > heightRatio) {
                    ratio = heightRatio;
                } else {
                    ratio = widthRatio;
                }

                // 清除画布内容
                canvas.clearRect(0, 0, containerWidth, containerHeight);
                canvas.scale(ratio, ratio);
                // canvas.translate((containerWidth / 2 - imgWidth / 2) / 2, 0);
                // 绘制矩形区域
                canvas.fillRect(0, 0, containerWidth, containerHeight);
                // 绘制图片
                canvas.drawImage(image, 0, 0);
                // 与矩形区域合成
                canvas.globalCompositeOperation = 'source-over';
                canvas.restore();

                imageBase64Arr.push(canvasDom.toDataURL());
            }

            if (typeof that.ready === 'function') {
                that.ready(imageBase64Arr);
            }
        } else {
            // 因为加载画布时用了闭包，这里需要延时后重新调用，
            setTimeout(function() {
                that.imageZoom();
            }, 200)
        }
    };

    init();
}
