/**
 * Painter -- 报表设计器
 * v1.0.1 2017-11-01 15:35:27 jx 创建
 * v1.0.2 2017-11-02 22:24:52 jx
 *   1)增加api: export 导出对象配置、页面配置信息
 *   2)增加变量: 总页数、当前页、日期、时间
 *   3)强制栏对象宽度为页面宽度
 * v1.0.3 2017-11-03 19:46:41 jx
 *   1)固定栏位置
 *   2)增加api: crateID 生成对象名
 *   3)增加 子元素跟随栏移动
 * v1.0.4 2017-11-07 10:47:56 jx
 *   1)增加 拖动改变元素大小功能
 * v1.0.5 2017-11-09 20:55:06 jx
 *   1)增加 模式mode: edit(编辑)、print(打印)、preview(预览)
 * v1.0.6 2017-11-10 11:09:05 jx
 *   1)增加 打印方向：横向、纵向, 此设置只应用于模式为print
 *   2)增加api: ready 类创建就绪后触发的方法
 *   3)增加api: setPageConfig
 *   4)调整 边距参数 改为 cm 单位
 * V1.0.7 2017-11-12 15:44:05
 *   1)增加 printConfig属性，设置 打印区域的宽高
 *
 *
 * V1.0.8 2019-02-28 17:31:44 sc
 * 增加 绘制参考线                            （元素移动和缩放时元素之间的对齐参考线，修复js浮点数运算时的精度问题）
 * 增加 撤销(ctrl+z) 和 恢复(ctrl+y) 功能
 * 增加 字体选择 ：宋体、微软雅黑、黑体
 * 增加 选中元素的蓝色虚线
 * 增加 多选功能                             （Shift + 鼠标左键 选中多个后可同时移动、缩放、设置选中元素）
 * 增加 用鼠标移动元素的同时按住 Shift键，则元素只能上下方向移动，按住 Alt键，则元素只能左右方向移动
 * 增加 Shift + 方向键 移动当前选中元素
 * 增加 二维码、条形码、图片类型的元素          （增加图片类型的元素）
 * 增加 是否自动高度 属性                     （在顶部工具栏设置单个元素的自动高度属性，元素内容超出高度时重新计算位置和大小）
 * 增加 是否打印空白行 设置                   （分页后用空白格填充无数据的主数据区域）
 * 增加 根据主数据(结构)做分页                （根据传入主数据的数据集和页面减去页头、头、尾、页尾后剩下的高度进行分页）
 * 增加 导出蓝牙打印指令所需的图片数据         （__this.blueToothPrintData）
 * 文本内容超出宽度时，只显示宽度范围内的文字
 *
 * 2019-05-29 14:08:48 sc
 * 增加 无限页高 功能
 *
 * 2019-07-10 16:41:35 sc
 * 修改主数据体中各个元素纵轴位置为各自相对主数据体的纵轴位置
 * 修改主数据体中的元素自动高度功能开启的前提为主数据体的自动高度已开启
 *
 * 2019-07-15 08:48:31 sc
 * 增加公式：加、减、乘、除、小括号（示例：[<单据.数量> * <单据.单价> / (<单据.前欠> + <单据.累欠>)]）
 * 增加内置函数：sum、min、max、avg、count、abs、Changedx、StringReplace（示例：[sum(<记录.数量>)][记录.单位]）
 *
 * 2019-07-23 10:58:34 sc
 * 增加自动宽度功能
 *
 * 2019-09-28 19:02:30 zsg
 * 增加隐藏数值0功能
 *
 * 2019-09-28 19:12:00 zsg
 * 增加控件可打印、可预览选项
 *
 * 2019-10-17 11:37:00 zsg
 * 增加标签控件，数值格式化方式  -1：默认、 0：取整、 1：保留1位小数、 2：保留2位小数
 *
 * 2020-01-09 11:34:00 zsg
 * 增加iif函数，作用于文本输入框 (不支持sum等聚合函数)
 *
 * 数据部分
 * head     页头
 * footer   页尾
 * data_header  头
 * data_footer  尾
 * data_main    主数据集
 * 待完成功能：
 * 动态码数                                  （留最后）
 *
 */
function Painter(options) {
    var VERSION = '1.0.8';
    // 内部维护了一个displayList，保存了canvas绘图时，每个元素放置的位置
    //            /iif\(((((?!\,).)*)(\,)+){2}(((?!\,|\)).)*)+\)/g
    var IIF_REG = /iif\(((((?!\,).)*)(\,)+){2}(((?!\,).)*)+\)/g; // iif 正则表达式，提取最小iif单元

    //默认配置
    var objDefaultOptions = {
        'id': null,
        'type': 'text', //类型, rect、 text、
        'text': '', // 内容, 类型为text有效
        'font_size': 16, // 文字大小(px)
        'x': 0, // x坐标(px)
        'y': 0, // y坐标(px)
        'top': 0, // 相对于容器的上边距(cm)
        'left': 0, // 相对于容器的左边距(cm)
        'width': 0, // 宽度(cm)
        'height': 0, // 高度(cm)
        'isDrag': true, //拖动
        'isParent': false,// 是否为父元素
        'parentID': '',
        'childs': [],      // 子元素列表
        'align': 'centen', // 内容水平方向: left,right,centen
        'valign': 'centen',// 内容垂直方向：top,bottom,centen
        'color': '#000', // 文字颜色
        'bold': false,   // 文字加粗
        'oblique': false, // 文字倾斜
        'textWidth': 0, // 文字宽度
        'simple': false,
        'font_family': '宋体',
        'borderStyle': {
            'top': 0,    // 上框线
            'bottom': 0, // 下框线
            'left': 0,   // 左框线
            'right': 0,  // 右框线
            'width': 120,  // 宽度
            'height': 28   // 高度
        }
    };

    // 基础方法
    var funs = {
        /**
         * 控件排列处理方式
         * @param obj
         * @return {int} 移动的距离
         */
        objLayoutHandler: function (obj, type) {

            // 固定栏对象不处理
            var fixedColumns = ['head', 'data_header', 'data_main', 'data_footer', 'footer'];
            var tmp_type = ['qrcode', 'text', 'icon', 'barcode'];
            var tmpID = [];
            if (fixedColumns.indexOf(obj.id) === -1) {
                if (typeof obj.layout === 'undefined' || obj.layout === 'auto') {

                } else if (obj.layout === 'auto_left') {
                    // 往左方向，检测有没有控件，只要一包含，就坐标位置
                    var moveX = 0; // 移动0
                    var tmpLeft = []; //储存左边元素对齐的位置坐标
                    var leftObj = funs.findObj('id', 'margin_left', __this.DisplayList);
                    //控件超出设计页面宽度
                    if (obj.x < leftObj.rect[1].x) obj.x = leftObj.rect[1].x;
                    //控件在设计页面内
                    for (var i = obj.x; i > 0; i--) {
                        for (var j = 0; j < __this.DisplayList.length; j++) {
                            //根据该元素的父级元素向左对齐
                            if (__this.DisplayList[j].id === obj.parentID) {
                                // moveX = obj.x - __this.DisplayList[j].x; // 移动的
                                obj.x = __this.DisplayList[j].x;
                            }
                            //获取该高度到最左边范围内的其他控件
                            for (var n = obj.rect[0].y; n < obj.rect[2].y; n++) {
                                if (__this.DisplayList[j].id !== obj.id && funs.contains(i, n, __this.DisplayList[j].rect)) {
                                    //排除左边的缩放点和边距
                                    if (tmp_type.indexOf(__this.DisplayList[j].type) > -1) {
                                        tmpID.push(__this.DisplayList[j]['id']);
                                        tmpLeft.push(__this.DisplayList[j].rect[1].x);
                                    } else {
                                        obj.x = leftObj.rect[1].x;
                                    }
                                }
                                n = n + 15;
                            }
                        }
                    }
                    //如果该元素的左边有其他元素，距离该元素最近的坐标进行左对齐
                    if (JSON.stringify(tmpLeft) != "[]") obj.x = Math.max.apply(null, tmpLeft);

                } else if (obj.layout === 'auto_right') {
                    var tmpRight = [];//储存右边元素对齐的位置坐标
                    //rightWidth 最右边宽度
                    var rightWidth = funs.findObj('id', 'margin_right', __this.DisplayList).x;

                    //控件超出设计页面宽度
                    if (obj.rect[1].x > rightWidth) {
                        obj.x = rightWidth - (obj.rect[1].x - obj.rect[0].x);
                    }
                    //控件在设计页面内
                    for (var i = obj.rect[1].x; i < rightWidth; i++) {
                        for (var j = 0; j < __this.DisplayList.length; j++) {
                            //根据该元素的父级元素向右对齐
                            if (__this.DisplayList[j].id == obj.parentID) {
                                obj.x = __this.DisplayList[j].rect[1].x - (obj.rect[1].x - obj.rect[0].x);
                            } else {
                                if (__this.DisplayList[j].id == 'margin_right') {
                                    obj.x = __this.DisplayList[j].rect[0].x - (obj.rect[1].x - obj.rect[0].x);
                                }
                            }
                            // 获取该高度到最右边范围内的其他控件
                            for (var n = obj.rect[0].y; n < obj.rect[2].y; n++) {
                                if (__this.DisplayList[j].id !== obj.id && funs.contains(i, n, __this.DisplayList[j].rect)) {
                                    //排除右边的缩放点和边距
                                    if (tmp_type.indexOf(__this.DisplayList[j].type) > -1) {
                                        tmpID.push(__this.DisplayList[j]['id']);
                                        tmpRight.push(__this.DisplayList[j].rect[0].x - (obj.rect[1].x - obj.rect[0].x))
                                    }
                                }
                                n = n + 15;
                            }
                        }
                    }
                    //如果该元素的右边有其他元素，距离该元素最近的坐标进行右对齐
                    if (JSON.stringify(tmpRight) != "[]") obj.x = Math.min.apply(null, tmpRight);

                } else if (obj.layout === 'auto_middle') {
                    var leftWidth = funs.findObj('id', 'margin_left', __this.DisplayList).rect[1].x;
                    var rightWidth = funs.findObj('id', 'margin_right', __this.DisplayList).x;
                    obj.x = parseFloat(parseFloat((leftWidth + (rightWidth - leftWidth) / 2) - ((obj.rect[1].x - obj.rect[0].x) / 2)).toFixed(0))
                }
            }
            if (type === 'delete') {
                var idArr = [];
                for (var i = 0; i < tmpID.length; i++) {
                    if (!idArr.includes(tmpID[i])) {//includes 检测数组是否有某个值
                        idArr.push(tmpID[i]);
                    }
                }
                return idArr;
            } else {
                return obj;
            }


        },
        /**
         * 控件发生改变控件自动排列
         * */
        AutoLayout: function (obj) {
            var autoLeft = [], autoRight = [];
            for (var i = 0; i < __this.DisplayList.length; i++) {
                if (!!obj && JSON.stringify(obj) != '{}' && __this.DisplayList[i].id == obj.id) {
                    __this.DisplayList[i].id = obj.id
                }
                if (__this.DisplayList[i].layout == 'auto_left') {
                    autoLeft.push(__this.DisplayList[i]);
                } else if (__this.DisplayList[i].layout == 'auto_right') {
                    autoRight.push(__this.DisplayList[i]);
                }
            }
            if (JSON.stringify(autoLeft) != '[]') {
                autoLeft = autoLeft.sort(function (obj1, obj2) {
                    return obj1.x - obj2.x;
                })
            }
            if (JSON.stringify(autoRight) != '[]') {
                autoRight = autoRight.sort(function (obj1, obj2) {
                    return obj2.rect[1].x - obj1.rect[1].x;
                })
            }
            if (JSON.stringify(autoLeft) != '[]') {
                //先左对齐，需要右对齐控件的判断右边是否右左对齐的控件并排除
                var existLeft = [];
                for (var i = 0; i < autoLeft.length; i++) {
                    existLeft = existLeft.concat(funs.objLayoutHandler(autoLeft[i], 'delete'))
                    autoLeft[i].rect = funs.getRect(autoLeft[i].x, autoLeft[i].y, autoLeft[i].borderStyle.width, autoLeft[i].borderStyle.height);
                }
                for (var i = 0; i < autoRight.length; i++) {
                    // if(existLeft.indexOf(autoRight[i].id) < 0) {
                    funs.objLayoutHandler(autoRight[i]);
                    autoRight[i].rect = funs.getRect(autoRight[i].x, autoRight[i].y, autoRight[i].borderStyle.width, autoRight[i].borderStyle.height);
                    // }
                }

            } else if (JSON.stringify(autoRight) != '[]') {
                for (var i = 0; i < autoRight.length; i++) {
                    funs.objLayoutHandler(autoRight[i]);
                    autoRight[i].rect = funs.getRect(autoRight[i].x, autoRight[i].y, autoRight[i].borderStyle.width, autoRight[i].borderStyle.height);
                }
            }
        },
        /**
         * 查找区域内选中的元素，该方法只在一次多选中计算一次
         * @param startX int
         * @param startY int
         * @param endX int
         * @param endY int
         */
        findObjsInArea: function (startX, startY, endX, endY) {
            if (startX > endX) {
                var tmp = startX;
                startX = endX;
                endX = tmp;
            }
            if (startY > endY) {
                var tmp = startY;
                startY = endY;
                endY = tmp;
            }
            // 遍历DisplayList，只要元素有个点在区域内，就选中
            if (!__this.ctrlAndMousedown.is_keyup) return;
            if (__this.ctrlAndMousedown.is_keyup) __this.ctrlAndMousedown.is_keyup = false;

            // console.log('startX', startX, 'startY', startY, 'endX', endX, 'endY', endY);
            var isFind = false;
            for (var i = 0; i < __this.DisplayList.length; i++) {
                // 根据元素的rect的位置坐标，判断是否选中
                // rect——四个顶点信息 [{x: '', y: ''}, {x: '', y: ''}, {x: '', y: ''}, {x: '', y: ''}]
                // [ 左上、右上、右下、左下 ];
                var obj = __this.DisplayList[i];
                if (obj.type === 'text') { // 只对文本框有效
                    // 这时候，问题转换成判断两个矩形是否相交，obj的矩形A，选中的矩形为B
                    // 判断矩形是否相交 https://www.jianshu.com/p/a2d881847a31
                    var xc1 = Math.max(obj.rect[0].x, startX);
                    var yc1 = Math.max(obj.rect[0].y, startY);
                    var xc2 = Math.min(obj.rect[2].x, endX);
                    var yc2 = Math.min(obj.rect[2].y, endY);
                    if (xc1 <= xc2 && yc1 <= yc2) {
                        isFind = true;
                        // 设置元素选中  需要判断是否在选中的列表中
                        var hasSign = __this.dragInfo.isInObjList(obj);
                        if (!hasSign) __this.dragInfo.objList.push(funs.extend({}, obj));
                    }
                }
            }

            if (isFind) {
                // 重绘画布
                // 去除绘制的蓝色边框
                __this.ctrlAndMousedown.startX = 0;
                __this.ctrlAndMousedown.startY = 0;
                __this.ctrlAndMousedown.endX = 0;
                __this.ctrlAndMousedown.endY = 0;
                draw.drawAll();
                // 选中多个元素回调
                // if (__this.dragInfo.objList.length > 1 && typeof __opt.onSelectMultipleObj === 'function') {
                //     __opt.onSelectMultipleObj();
                // }
            }
        },
        /**
         * 判断数值结果是否需要处理
         * @param result
         * @return {*}
         */
        getResult: function (result) {
            if (__opt.pageConfig.hideZero && result == 0) {
                return '';
            } else {
                return result;
            }
        },
        /**
         *  根据isForPreview、isForPrint 判断是否需要显示
         *  isForPreview、isForPrint为undefined时，也需要显示
         * @param obj 绘制对象
         */
        isCanDraw: function (obj) {
            if (__opt.mode === 'preview') {
                if (typeof obj.isForPreview === 'undefined') return true;
                return obj.isForPreview;
            } else if (__opt.mode === 'print') {
                if (typeof obj.isForPrint === 'undefined') return true;
                return obj.isForPrint;
            } else {
                return true;
            }

        },

        /**
         * 格式化处理
         * -1： 默认  0：取整  1：保留1位小数  2： 保留2位小数
         * @param obj 绘制对象
         * @param text 绘制对象值
         */
        moneyFormat: function (obj, text) {
            if(typeof text === 'undefined' || text === null) text = '';
            text = String(text).replace(/null|undefined/g, '');
            //文本样式、格式
            if (obj.text_type == 'data') {
                var reg = /a|N/g;
                if (obj.text_format != 'default' && !reg.test(funs.formatDate(obj.text_format, text))) {
                    text = funs.formatDate(obj.text_format, text);
                }
            } else if (obj.text_type == 'Integer') {
                switch (obj.text_format) {
                    case '0':
                        text = IntegerType('slice', '0')
                        break;
                    case '1':
                        text = IntegerType('slice', '1')
                        break;
                    case '2':
                        text = IntegerType('slice', '2')
                        break;
                    case '3':
                        text = IntegerType('slice', '3')
                        break;
                    case '4':
                        text = IntegerType('slice', '4')
                        break;
                    case 'round0':
                        text = IntegerType('round', '0')
                        break;
                    case 'round1':
                        text = IntegerType('round', '1')
                        break;
                    case 'round2':
                        text = IntegerType('round', '2')
                        break;
                    case 'round3':
                        text = IntegerType('round', '3')
                        break;
                    case 'round4':
                        text = IntegerType('round', '4')
                        break;
                }
                //添加分格符
                text = IntegerType('', '', function (value) {
                    if (!obj.divide || obj.divide === '0' || !value || value === '0') return value;
                    if (!!String(value).match(/\./g) && String(value).match(/\./g).length == 1 || String(value).match(/\./g) === null) {
                        //添加分隔符
                        // value = String(value).replace(/^\d*/g,function(replace){
                        //     if(replace > 0){
                        //         var tmp_value = replace.split("").reverse().join("");
                        //         // tmp_value = tmp_value.match(/\d{3}/g).join(",");
                        //         tmp_value = String(tmp_value).replace(/\d{3}/g,function(replace){
                        //             return replace+','
                        //         })
                        //         return tmp_value.split("").reverse().join("");
                        //     }else{
                        //         return replace
                        //     }
                        // })
                        value = Number(value).toLocaleString();
                    }
                    return value;
                })
                //补充0
                text = IntegerType('', '', function (value) {
                    if (!obj.supplementZero || obj.supplementZero === '0' || !value && value !== '0' && value !== 0) return value;
                    if (!!String(value).match(/\./g) && String(value).match(/\./g).length == 1) {
                        var digit = String(value).length - (String(value).indexOf('.') + 1);
                        for (var i = 0; i < obj.supplementZero - digit; i++) {
                            value += '0';
                        }

                    } else if (String(value).match(/\./g) === null) {
                        value = value + '.';
                        for (var i = 0; i < obj.supplementZero; i++) {
                            value += '0';
                        }
                    }
                    return value;
                })

            }
            if (obj.hideZero || typeof obj.hideZero === 'undefined') {
                if (Number(text) === 0) text = '';
            }


            if (obj.type === 'text') {
                // 金额格式化处理，现在只会显示字符串的文本
                var value = Number(text);
                if (!isNaN(value)) {
                    if (typeof obj.money_style !== 'undefined' && text !== '' && obj.money_style !== -1) {
                        if (obj.money_style === 0) {
                            text = Math.round(value).toString();
                        } else {
                            text = value.toFixed(obj.money_style).toString();
                        }
                    }
                }
            }

            function IntegerType(type, Several, callback) {
                var value = 0;
                if (!!String(text).match(/[^\d.-]/g) && String(text).match(/[^\d.-]/g).length > 0) return text;
                if (!!String(text).match(/\-/g) && String(text).match(/\-/g).length > 1) return text;
                if (!!String(text).match(/\./g) && String(text).match(/\./g).length == 1) {
                    if (!!Several) value = Number(Several) + 1;
                    if (type == 'slice') return String(text).slice(0, Number(String(text).indexOf('.')) + value);
                    else if (type == 'round') return parseFloat(text).toFixed(Number(Several));
                }
                if (typeof callback === 'function') return callback(text);

                return text;
            }

            return text;
        },

        // jquery的$.trim()
        trim: function (str) {
            return str.replace(/(^\s*)|(\s*$)/g, "");
        },
        getDate: function () {
            return funs.formatDate(__this.dateFormat.date);
        },
        getTime: function () {
            return funs.formatDate(__this.dateFormat.time);
        },
        formatDate: function (formatStr, newDate) {
            var nowDate = new Date();
            // 兼容苹果系统
            if (newDate) {
                nowDate = new Date(newDate.replace(/-/ig, '/'));
            }

            var str = '';
            if (formatStr) {
                str = formatStr;
            }
            str = str.replace(/Y|D|H/g, function (matching) {
                return matching.toLocaleLowerCase();
            });

            var Week = ['日', '一', '二', '三', '四', '五', '六'];
            str = str.replace(/yyyy/, nowDate.getFullYear());
            str = str.replace(/yy/, funs.toDigit((nowDate.getYear() % 100)));

            str = str.replace(/MM/, funs.toDigit(nowDate.getMonth() + 1));
            str = str.replace(/M/g, nowDate.getMonth() + 1);

            str = str.replace(/w/g, Week[nowDate.getDay()]);

            str = str.replace(/dd/, funs.toDigit(nowDate.getDate()));
            str = str.replace(/d/g, nowDate.getDate());

            str = str.replace(/hh/, funs.toDigit(nowDate.getHours()));
            str = str.replace(/h/g, nowDate.getHours());

            str = str.replace(/mm/, funs.toDigit(nowDate.getMinutes()));
            str = str.replace(/m/g, nowDate.getMinutes());

            str = str.replace(/ss/, funs.toDigit(nowDate.getSeconds()));
            str = str.replace(/s/g, nowDate.getSeconds());

            return str;
        },

        toDigit: function (num) {
            return num < 10 ? '0' + num : '' + num;
        },
        // 生成对象id
        createID: function (name) {
            var i = 1;
            while (funs.findObj('id', name + i)) {
                i++;
            }
            return (name + i);
        },
        // 对象合并方法, 移植JQuery.extend方法
        extend: function () {
            var options, name, src, copy, copyIsArray, clone,
                target = arguments[0] || {},
                i = 1,
                length = arguments.length,
                deep = false;
            if (typeof target === "boolean") {
                deep = target;
                target = arguments[i] || {};
                i++;
            }
            if (typeof target !== "object" && typeof target !== "function") {
                target = {};
            }
            if (i === length) {
                target = this;
                i--;
            }
            for (; i < length; i++) {
                if ((options = arguments[i]) != null) {
                    for (name in options) {
                        src = target[name];
                        copy = options[name];
                        if (target === copy) {
                            continue;
                        }
                        if (deep && copy && (isJSONObject(copy) || (copyIsArray = Array.isArray(copy)))) {
                            if (copyIsArray) {
                                copyIsArray = false;
                                clone = (src && Array.isArray(src)) ? src : [];
                            } else {
                                clone = (src && isJSONObject(src)) ? src : {};
                            }
                            target[name] = arguments.callee(deep, clone, copy);
                        } else if (copy !== undefined) {
                            target[name] = copy;
                        }
                    }
                }
            }
            //判断是否为json对象
            function isJSONObject(obj) {
                return ((typeof obj === "object") && (Object.prototype.toString.call(obj).toLowerCase() === "[object object]")) ? 1 : 0;
            }

            return target;
        },
        // 数组是否包含某个元素, 移植JQuery.inArray
        inArray: function (elem, arr, i) {
            var len;
            if (arr) {
                // if (core_indexOf) {
                //     return core_indexOf.call(arr, elem, i);
                // }
                len = arr.length;
                i = i ? i < 0 ? Math.max(0, len + i) : i : 0;
                for (; i < len; i++) {
                    // Skip accessing in sparse arrays
                    if (i in arr && arr[i] === elem) {
                        return i;
                    }
                }
            }
            return -1;
        },
        /**
         * 判断 给定路径 是否包含 点
         * 根据奇-偶规则 https://en.wikipedia.org/wiki/Even–odd_rule
         **/
        contains: function (x, y, poly) {
            var c = false;
            if (poly) {
                for (var i = 0, j = poly.length - 1; i < poly.length; j = i++) {
                    if ((poly[i].y > y) != (poly[j].y > y) &&
                        (x < (poly[j].x - poly[i].x) * (y - poly[i].y) / (poly[j].y - poly[i].y) + poly[i].x)) {
                        c = !c;
                    }
                }
            }
            return c;
        },
        // 查找指定对象属性
        findObj: function (key, value, list, callback) {
            list = list || __this.DisplayList;
            for (var i = 0; i < list.length; i++) {
                if (list[i] && list[i][key] != undefined && list[i][key] == value) {
                    if (typeof callback === 'function') callback(list[i]);
                    return list[i];
                }
            }
            return null;
        },
        // 以id名获取对象
        get: function (id, index) {
            var list = __this.DisplayList.concat();
            for (var i = 0; i < list.length; i++) {
                if (list[i] && list[i]['id'] != undefined && list[i]['id'] == id) {
                    if (!index) return list[i];
                    return i;
                }
            }
            return null;
        },
        // 获取点击对象属性
        findClickObj: function (x, y, callback) {
            var list = __this.DisplayList.concat();
            list.sort(function (a, b) {
                return b.zIndex - a.zIndex;
            });

            for (var i = 0; i < list.length; i++) {
                if (list[i] !== undefined && funs.contains(x, y, list[i].rect)) {
                    if (typeof callback === 'function') callback(list[i]);
                    return funs.extend(true, {}, list[i]);
                }
            }
            return null;
        },
        setObjWidthHeight: function (obj, opt) {
            if (typeof opt.width === 'undefined' && opt.borderStyle && (typeof opt.borderStyle.width !== 'undefined')) {
                // obj.width = parseFloat((obj.borderStyle.width / __opt.cmToDots).toFixed(2));
                obj.width = parseFloat((funs.decimalDiv(obj.borderStyle.width, __opt.cmToDots)).toFixed(2));
            }

            if (typeof opt.height === 'undefined' && opt.borderStyle && (typeof opt.borderStyle.height !== 'undefined')) {
                // obj.height = parseFloat((obj.borderStyle.height / __opt.cmToDots).toFixed(2));
                obj.height = parseFloat((funs.decimalDiv(obj.borderStyle.height, __opt.cmToDots)).toFixed(2));
            }

            if (typeof opt.width !== 'undefined' && (!opt.borderStyle || (typeof opt.borderStyle.width !== 'undefined'))) {
                // obj.borderStyle.width = obj.width * __opt.cmToDots;
                obj.borderStyle.width = Math.round(funs.decimalMul(obj.width, __opt.cmToDots));
            }

            if (typeof opt.height !== 'undefined' && (!opt.borderStyle || (typeof opt.borderStyle.height !== 'undefined'))) {
                // obj.borderStyle.height = obj.height * __opt.cmToDots;
                obj.borderStyle.height = Math.round(funs.decimalMul(obj.height, __opt.cmToDots));
            }
        },
        /**
         * 初始化父控件位置
         * @param obj
         * @param opt
         * @param list
         */
        setObjPlace: function (obj, opt, list) {
            // if (!obj || !obj.parentID) return;
            if (!obj) return;
            if (!obj.parentID) {
                var tmp_parentID = 'margin_left';
            } else {
                var tmp_parentID = obj.parentID;
            }

            var parentObj = funs.findObj('id', tmp_parentID, list);
            if (!parentObj) return;
            if (typeof opt.top === 'undefined' && typeof opt.y !== 'undefined') {
                obj.top = parseFloat(((obj.y - parentObj.y - 20) / __opt.cmToDots).toFixed(2));
            }

            if (typeof opt.left === 'undefined' && typeof opt.x !== 'undefined') {
                if (tmp_parentID === 'margin_left') {
                    obj.left = parseFloat(((obj.x - parentObj.rect[1].x) / __opt.cmToDots).toFixed(2));
                } else {
                    obj.left = parseFloat(((obj.x - parentObj.x) / __opt.cmToDots).toFixed(2));
                }
            }

            if (typeof opt.x === 'undefined' && typeof opt.left !== 'undefined') {
                obj.x = ((obj.left * __opt.cmToDots) + parentObj.x);
            }

            if (typeof opt.y === 'undefined' && typeof opt.top !== 'undefined') {
                obj.y = ((obj.top * __opt.cmToDots) + (parentObj.y + 20));
            }

        },
        setCanvasHeight: function (height, pageMaxHeight) {
            __opt.pageConfig.height = height; //单位：cm
            if (__opt['mode'] == 'edit' && pageMaxHeight) {
                height = pageMaxHeight * __opt.cmToDots; //单位：px
            } else {
                height = height * __opt.cmToDots; //单位：px
            }
            __this.canvas.height = height;
            // 页面高度
            __opt.pageHeight = height;
            // 中央高度
            __opt.centerHeight = (height - __opt.margin.top - __opt.margin.bottom);
        },
        setCanvasWidth: function (width) {
            __opt.pageConfig.width = width; //单位：cm
            width = width * __opt.cmToDots; //单位：px
            __this.canvas.width = width;
            // 页面宽度
            __opt.pageWidth = width;
            // 中央宽度
            __opt.centerWidth = (width - __opt.margin.left - __opt.margin.right);
        },
        setMargin: function () {
            var margin = __opt.margin;
            var top = {
                type: 'rect',
                borderStyle: {
                    top: 0, left: 0, right: 0, bottom: 0, background: '#FFF', height: margin.top,
                    width: __this.canvas.width
                },
                id: 'margin_top',
                isDrag: false,
                zIndex: -9999999998,
                x: -1,
                y: -1
            };
            top.rect = funs.getRect(0, 0, 0, 0);
            var bottom = {
                type: 'rect',
                borderStyle: {
                    top: 0, left: 0, right: 0, bottom: 0, background: '#FFF', height: margin.bottom,
                    width: __this.canvas.width
                },
                id: 'margin_bottom',
                isDrag: false,
                zIndex: -9999999998,
                x: -1,
                y: __this.canvas.height - margin.bottom + 1
            };
            bottom.rect = funs.getRect(0, 0, 0, 0);
            var left = {
                type: 'rect',
                borderStyle: {
                    top: 0, left: 0, right: 0, bottom: 0, background: '#FFF', height: __this.canvas.height,
                    width: margin.left
                },
                id: 'margin_left',
                isDrag: false,
                zIndex: -9999999998,
                x: -1,
                y: -1
            };
            left.rect = funs.getRect(0, 0, 0, 0);
            var right = {
                type: 'rect',
                borderStyle: {
                    top: 0, left: 0, right: 0, bottom: 0, background: '#FFF', height: __this.canvas.height,
                    width: margin.right + 1
                },
                isDrag: false,
                id: 'margin_right',
                zIndex: -9999999998,
                x: __this.canvas.width - margin.right,
                y: 0
            };
            right.rect = funs.getRect(0, 0, 0, 0);
            var centen = {
                type: 'rect',
                borderStyle: {
                    top: 1, left: 1, right: 1, bottom: 1,
                    height: __this.canvas.height - margin.top - margin.bottom + 1,
                    width: __this.canvas.width - margin.left - margin.right
                },
                id: 'margin_centen',
                isDrag: false,
                zIndex: -9999999999,
                x: margin.left,
                y: margin.top
            };
            centen.rect = funs.getRect(0, 0, 0, 0);
            if (!funs.findObj('id', 'margin_top', null, function (obj) {
                funs.setObjOpt(obj.dataIndex, top, false, false)
            })) draw.addToList(top);
            if (!funs.findObj('id', 'margin_bottom', null, function (obj) {
                funs.setObjOpt(obj.dataIndex, bottom, false, false)
            })) draw.addToList(bottom);
            if (!funs.findObj('id', 'margin_left', null, function (obj) {
                funs.setObjOpt(obj.dataIndex, left, false, false)
            })) draw.addToList(left);
            if (!funs.findObj('id', 'margin_right', null, function (obj) {
                funs.setObjOpt(obj.dataIndex, right, false, false)
            })) draw.addToList(right);
            if (!funs.findObj('id', 'margin_centen', null, function (obj) {
                funs.setObjOpt(obj.dataIndex, centen, false, false)
            })) draw.addToList(centen);
        },
        /**
         * // TODO 方法传参修改为对象传参
         * 设置对象属性
         * @param index {int} 对象序号
         * @param opt {object} 配置信息
         * @param Refresh {boolean} 重绘(刷新画布)
         * @param auto_layout {boolean} 是否自动布局
         */
        setObjOpt: function (index, opt, Refresh, auto_layout) {
            if (typeof opt !== 'object') return;
            var displayable = funs.extend(true, {}, __this.DisplayList[index], { selected: false }, opt);

            funs.setObjWidthHeight(displayable, opt);
            // 如果属于栏系列固定属性，无法使用自动排列功能，跳过
            if (displayable.isParent) {
                displayable.x = __opt.margin.left;
                displayable.borderStyle.width = __opt.centerWidth;
                delete displayable['width'];
                delete displayable['height'];

                var _top = __opt.margin.top;

                if (funs.findObj('id', 'head')) {
                    var obj = funs.get('head');
                    if (displayable.id === obj.id) displayable.y = _top;
                    obj.y = _top;
                    obj.rect = funs.getRect(obj.x, obj.y, obj.borderStyle.width, obj.borderStyle.height);
                    _top += obj.borderStyle.height + 5;
                }

                if (funs.findObj('id', 'data_header')) {
                    var obj = funs.get('data_header');
                    if (displayable.id === obj.id) displayable.y = _top;
                    obj.y = _top;
                    obj.rect = funs.getRect(obj.x, obj.y, obj.borderStyle.width, obj.borderStyle.height);
                    _top += obj.borderStyle.height + 5;
                }

                if (funs.findObj('id', 'data_main')) {
                    var obj = funs.get('data_main');
                    if (displayable.id === obj.id) displayable.y = _top;
                    obj.y = _top;
                    obj.rect = funs.getRect(obj.x, obj.y, obj.borderStyle.width, obj.borderStyle.height);
                    _top += obj.borderStyle.height + 5;
                }

                if (funs.findObj('id', 'data_footer')) {
                    var obj = funs.get('data_footer');
                    if (displayable.id === obj.id) displayable.y = _top;
                    obj.y = _top;
                    obj.rect = funs.getRect(obj.x, obj.y, obj.borderStyle.width, obj.borderStyle.height);
                    _top += obj.borderStyle.height + 5;
                }

                if (displayable.id === 'footer') {
                    // console.log(2,__opt,displayable)
                    var height;
                    if (__opt.pageConfig.pageMaxHeight) {
                        height = __opt.pageConfig.height * __opt.cmToDots - __opt.margin.top - __opt.margin.bottom;
                    } else {
                        height = __opt.centerHeight;
                    }
                    displayable.y = height - displayable.borderStyle.height + __opt.margin.bottom + 1;
                }

                funs.setObjWidthHeight(displayable, displayable);

                var y = displayable.y - __this.DisplayList[index].y;
                var x = displayable.x - __this.DisplayList[index].x;

                // 如果移动到栏对象，子类也一起移动
                for (var i = 0; i < displayable.childs.length; i++) {
                    var obj = funs.findObj('id', displayable.childs[i]);
                    funs.setObjOpt(obj.dataIndex, {
                        x: obj.x + x,
                        y: obj.y + y,
                    }, false, false);
                }
            } else {
                // 没有多选时
                if (!__this.dragInfo.startDrag) funs.setObjPlace(displayable, opt);
                var parentObj = funs.findObj('id', displayable.parentID);
                if (parentObj) {
                    if (__this.DisplayList[index].id !== displayable.id) {
                        for (var i = 0; i < parentObj.childs.length; i++) {
                            if (parentObj.childs[i] === __this.DisplayList[index].id) {
                                parentObj.childs[i] = displayable.id;
                                break;
                            }
                        }
                    }
                }
            }

            // 控件自动对齐处理

            if (auto_layout) {
                funs.objLayoutHandler(displayable);
            }
            displayable.rect = funs.getRect(displayable.x, displayable.y, displayable.borderStyle.width, displayable.borderStyle.height);

            // if(displayable.type == 'text'){
            //     displayable.width = self.measureText(displayable.text);
            // }

            __this.DisplayList[index] = displayable;
            if (typeof Refresh === 'undefined' || Refresh !== false) {
                // parentAndChild.judge(self.DisplayList[index]);
                draw.drawAll();
            }
            // if (opt.selected) {
            // draw.Selected(displayable, false);
            // }
        },

        /**
         * 获取文本输入框的值
         * @param obj
         * @param value
         * @param index 如果是主数据集时，控件在数据集对应的下标
         * @param pageIndex 分页的下标
         * @return {string|*}
         */
        getText: function (obj, value, index, pageIndex) {
            // console.log(obj, value, index, pageIndex)
            var result = value;
            if (!/\[(.*?)\]/.test(result)) {
                return funs.moneyFormat(obj, result);
            }

            result = String(value).replace(/\[(.*?)\]/g, function (oldValue, MatchValue) {
                var aggregationFn = false;
                var match_nameFn = '';

                var textFnList = funs.getTextFn();
                var textFnName = Object.keys(textFnList);
                for (var i = 0; i < textFnName.length; i++) {
                    var nameFn = textFnName[i]; // sum、min、max
                    if (typeof textFnList[nameFn] !== 'function') continue;
                    var reg = nameFn + '\\((.*)\\)'; // 匹配到最后一个 \((.*)\) )    sum\((.*)\)
                    var regFn = new RegExp(reg, 'i');
                    // 聚合函数处理
                    if (regFn.test(MatchValue)) {
                        MatchValue = MatchValue.toLowerCase();  // 聚合函数，统一转小写
                        // 参考 https://developer.mozilla.org/zh-CN/docs/Web/JavaScript/Reference/Global_Objects/String/replace#%E6%8C%87%E5%AE%9A%E4%B8%80%E4%B8%AA%E5%87%BD%E6%95%B0%E4%BD%9C%E4%B8%BA%E5%8F%82%E6%95%B0
                        MatchValue = String(MatchValue).replace(RegExp(regFn.source), function (a, b) {
                            // 匹配聚合函数（数据源为当前页的主数据集）
                            if (/min|max|avg|count|sum|iif/i.test(nameFn)) {
                                // debugger;
                                aggregationFn = true;
                                // 记录匹配的聚合函数名称
                                match_nameFn = nameFn;
                                var value = textFnList[nameFn](b, a, index, pageIndex);   // 调用min(b, a, pageIndex)
                                // console.log(b, a, pageIndex, value);
                                return funs.getResult(value);
                            }

                            if (/\<(.*?)\>/.test(b)) {
                                b = String(b).replace(/\<(.*?)\>/g, function (a, b) {
                                    return funs.getPrintText(a, b, index, true);
                                });
                            }

                            if (/\+|\-|\*|\//m.test(b)) {
                                b = funs.getTextFromOperator(b);
                            }
                            // console.log(a, b, textFnList[nameFn](b, a));
                            return textFnList[nameFn](b, a, index);
                        });
                    }
                }
                // console.log('result1', result);

                if (!!aggregationFn && typeof pageIndex === 'undefined') return oldValue;

                // 正则匹配[....]，没有使用聚合函数的表达式
                // 匹配<>
                var isMatched = false;
                if (/\<(.*?)\>/.test(MatchValue)) {
                    MatchValue = String(MatchValue).replace(/\<(.*?)\>/g, function (a, b) {
                        return funs.getPrintText(a, b, index, true);
                    });
                    isMatched = true;
                }

                // 匹配加减运算
                if (/\+|\-|\*|\//m.test(MatchValue)) {
                    if (!(match_nameFn === 'iif' && PaintParse.parseResult.operator[funs.md5(String(MatchValue))])) {
                        MatchValue = funs.getTextFromOperator(MatchValue);
                    }
                }

                // 如果已经匹配<>，说明替换已经完成，不需要再次进行替换
                if (isMatched) {
                    // 金额格式化
                    return funs.moneyFormat(obj, MatchValue);
                }
                var printText = funs.getPrintText(MatchValue, MatchValue, index, false);

                // 通过暴露的外部方法获取小程序码地址
                if (obj.type === 'miniCode') {
                    return printText;
                }
                // 金额格式化
                return funs.moneyFormat(obj, printText);
            })

            // 隐藏0，显示空字符串
            // console.log('result2', result);
            return result;
        },
        //
        /**
         * 转换打印文本
         * @param a
         * @param b
         * @param index
         * @param isOperator 是否进行四则运算
         * @return {*}
         */
        getPrintText: function (a, b, index, isOperator) {
            var tmp = b.split('.');
            // if (tmp.length < 2) return '("")';
            if (tmp.length < 2) return b;
            var empty = (tmp[0] === '变量' ? a : '');
            var result = '';
            if (typeof index === 'undefined') {
                // result = __opt['data'][tmp[0]] ? __opt['data'][tmp[0]][tmp[1]] || empty : empty;
                if (!!__opt['data'][tmp[0]]) {
                    if (Array.isArray(__opt['data'][tmp[0]])) {
                        if (typeof __opt['data'][tmp[0]][0][tmp[1]] !== 'undefined') {
                            result = __opt['data'][tmp[0]][0][tmp[1]];
                        } else {
                            result = empty;
                        }
                    } else {
                        if (typeof __opt['data'][tmp[0]][tmp[1]] !== 'undefined') {
                            result = __opt['data'][tmp[0]][tmp[1]];
                        } else {
                            result = empty;
                        }
                    }
                } else {
                    result = b;
                }
            } else {
                // result = __opt['data'][tmp[0]][index] ? __opt['data'][tmp[0]][index][tmp[1]] || empty : empty;
                if (!!__opt['data'][tmp[0]] && !!__opt['data'][tmp[0]][index]) {
                    if (typeof __opt['data'][tmp[0]][index][tmp[1]] !== 'undefined') {
                        result = __opt['data'][tmp[0]][index][tmp[1]];
                    } else {
                        result = empty;
                    }
                } else {
                    if (!!__opt['data'][tmp[0]] && !Array.isArray(__opt['data'][tmp[0]]) && typeof __opt['data'][tmp[0]][tmp[1]] !== 'undefined') {
                        result = __opt['data'][tmp[0]][tmp[1]];
                    } else {
                        result = b;
                    }
                }
            }
            // 判断，隐藏0
            return result;
            // return isOperator ? result : funs.getResult(result);
        },
        //获取内置函数
        getTextFn: function () {
            return {
                // str：文本内容, m：原始字符串, index：控件对应明细横的数据下标, pageIndex
                //                   start: 渲染数据源开始下标  end: 渲染数据源结束下标   data：渲染数据源
                // execute: function(start, end, data) {
                /**
                     * 获取合计数据
                     * @param str
                     * @param m
                     * @param index
                     * @param pageIndex
                 * @return {*}
                 */
                sum: function (str, m, index, pageIndex) {
                    return PaintParse.textFnExecutor({
                        str: str,
                        m: m,
                        pageIndex: pageIndex,
                        execute: function (start, end, data, text) {
                            // 解析数据
                            return PaintParse.parseDataMain({
                                text: text,
                                data_arr: data,
                                index: index,
                                defaultValue: 0,
                                start: start,
                                end: end,
                                parseFun: function (res, obj, value) {
                                    // 判断解析的数据存不存在
                                    if (typeof obj !== 'undefined' && typeof value !== 'undefined' && !Number.isNaN(parseFloat(value))) {
                                        return funs.decimalAdd(res, parseFloat(value));
                                    }
                                }
                            });
                        }
                    });
                },
                min: function (str, m, index, pageIndex) {
                    return PaintParse.textFnExecutor({
                        str: str,
                        m: m,
                        pageIndex: pageIndex,
                        execute: function (start, end, data, text) {
                            return PaintParse.parseDataMain({
                                text: text,
                                data_arr: data,
                                index: index,
                                defaultValue: [],
                                start: start,
                                end: end,
                                parseFun: function (res, obj, value) {
                                    // 判断解析的数据存不存在
                                    if (typeof obj !== 'undefined' && typeof value !== 'undefined' && !Number.isNaN(parseFloat(value))) {
                                        res.push(parseFloat(value));
                                        return res;
                                    }
                                },
                                returnValFun: function (res) {
                                    if (res.length < 1) return '';
                                    return Math.min.apply(null, (res));
                                }
                            });
                        }
                    });
                },
                max: function (str, m, index, pageIndex) {
                    return PaintParse.textFnExecutor({
                        str: str,
                        m: m,
                        pageIndex: pageIndex,
                        execute: function (start, end, data, text) {
                            return PaintParse.parseDataMain({
                                text: text,
                                data_arr: data,
                                index: index,
                                defaultValue: [],
                                start: start,
                                end: end,
                                parseFun: function (res, obj, value) {
                                    // 判断解析的数据存不存在
                                    if (typeof obj !== 'undefined' && typeof value !== 'undefined' && !Number.isNaN(parseFloat(value))) {
                                        res.push(parseFloat(value));
                                        return res;
                                    }
                                },
                                returnValFun: function (res) {
                                    if (res.length < 1) return '';
                                    return Math.max.apply(null, res);
                                }
                            });
                        }
                    });
                },
                avg: function (str, m, index, pageIndex) {
                    return PaintParse.textFnExecutor({
                        str: str,
                        m: m,
                        pageIndex: pageIndex,
                        execute: function (start, end, data, text) {
                            return PaintParse.parseDataMain({
                                text: text,
                                data_arr: data,
                                index: index,
                                defaultValue: [],
                                start: start,
                                end: end,
                                parseFun: function (res, obj, value) {
                                    // 判断解析的数据存不存在
                                    if (typeof obj !== 'undefined' && typeof value !== 'undefined' && !Number.isNaN(parseFloat(value))) {
                                        res.push(parseFloat(value));
                                        return res;
                                    } else {
                                        return res.push(0);
                                    }
                                },
                                returnValFun: function (res) {
                                    if (typeof res === "undefined" || res.length === 0) res = [0];
                                    return parseFloat(funs.decimalDiv(res.reduce(function (accumulator, currentValue) {
                                        return funs.decimalAdd(accumulator, currentValue);
                                    }), res.length).toFixed(4));
                                }
                            });
                        }
                    });
                },
                count: function (str, m, index, pageIndex) {
                    return PaintParse.textFnExecutor({
                        str: str,
                        m: m,
                        pageIndex: pageIndex,
                        execute: function (start, end, data, text) {
                            return String(str).replace(/\<(.*?)\>/g, function (a, b) {
                                return funs.decimalSub(end, start);
                            });
                        }
                    });
                },
                abs: function (str) {
                    var num = parseFloat(str);
                    if (Number.isNaN(num)) {
                        return str;
                    } else {
                        return String(Math.abs(num));
                    }
                },
                changedx: function (str) {
                    var num = parseFloat(str);
                    if (isNaN(num)) return "";
                    var strPrefix = "";
                    if (num < 0) strPrefix = "(负)";
                    num = Math.abs(num);
                    if (num >= 999999999) return "无效数值！";
                    var strOutput = "";
                    var strUnit = '仟佰拾亿仟佰拾万仟佰拾元角分厘毫';
                    var strCapDgt = '零壹贰叁肆伍陆柒捌玖';
                    num += "0000";
                    var intPos = num.indexOf('.');
                    if (intPos >= 0) {
                        num = num.substring(0, intPos) + num.substr(intPos + 1, 4);
                    }
                    strUnit = strUnit.substr(strUnit.length - num.length);
                    for (var i = 0; i < num.length; i++) {
                        strOutput += strCapDgt.substr(num.substr(i, 1), 1) + strUnit.substr(i, 1);
                    }
                    return strPrefix + strOutput.replace(/零角零分零厘零毫$/, '整').replace(/零分零厘零毫$/, '').replace(/零厘零毫$/, '').replace(/零毫$/, '').replace(/零[仟佰拾]/g, '零').replace(/零{2,}/g, '零').replace(/零([亿|万])/g, '$1').replace(/零+元/, '元').replace(/亿零{0,3}万/, '亿').replace(/^元/, "零元");
                },
                stringreplace: function (arg) {
                    var args = arg.split(',');
                    if (args.length !== 3) return arg;
                    var str = args[0], substr = args[1], newSubStr = args[2];
                    var replaceReg = new RegExp(substr, 'ig');
                    return str.replace(replaceReg, newSubStr);
                },
                /**
                 * iif 函数解析
                 * 计算过程 —— iif(1<>2,iif((4-3)*2>0,f2,f),f1)
                 * 1. 计算iif((4-3)*2>0,f2,f)，得到 f2
                 * 2. 计算iif(1<>2,f2,f1)，得到f2
                 * @param str 文本内容
                 * @param originStr 原始字符串
                 * @param index 控件对应明细横的数据下标
                 * @param pageIndex
                 */
                iif: function (str, originStr, index, pageIndex) {
                    return PaintParse.textFnExecutor({
                        str: str,
                        m: originStr,
                        pageIndex: pageIndex,
                        execute: function (start, end, data) {
                            return funs.iifParser(originStr, start, end, data, index);
                        }
                    });
                }
            }
        },

        /**
         * 解析值 —— <记录.数量> 值，返回对应的表达式
         * @param str 字符串内容
         * @param start 数据源开始下标
         * @param end 数据源结束下标
         * @param data 数据源
         * @param data_index {Integer} 数据源对应的下标
         * @param callback 回调函数，计算具体的值
         * @return {string}
         */
        parseTextValue: function (str, start, end, data, data_index, callback) {
            // 解析 <> 中的值
            return str.replace(/\<(.*?)\>/g, function (a, b) {
                // 没有匹配到值，返回原始值
                if (b === '') {
                    return a;
                }
                // 处理 iif(<明细横.空差><0,'',<明细横.空差>*<明细横.匹数>) 的情况 匹配到 <0,'',<明细横.空差>
                // b = 0,'',<明细横.空差

                // 处理 iif(<单据.本单金额><(<单据.账户_金额>+<单据.抵订金>),欠款,结清) 匹配到 <(<单据.账户_金额> 的情况
                // b = (<单据.账户_金额
                var b_value = b;
                var last_index = b.lastIndexOf('<');
                if (last_index > -1) { // 修正正则匹配问题
                    b_value = b.slice(last_index + 1);
                }

                // 正确的表达式，解析为数值
                b_value = '<' + b_value + '>';
                var res = PaintParse.parseDataMain({
                    text: b_value,
                    data_arr: data,
                    index: data_index,
                    defaultValue: 0,
                    start: start,
                    end: end,
                    parseFun: function (res, obj, value) {
                        if (typeof obj !== 'undefined' && typeof value !== 'undefined') {
                            return callback(res, value);
                        }
                    }
                });

                // 判断是否包含四则运算
                var regexOperator = /\+|\-|\*|\//m;
                if (regexOperator.test(res)) {
                    // 判断解析出来的值，有没有包含四则运算符合，key: value形式，true: 表示有
                    PaintParse.parseResult['operator'][funs.md5(String(res))] = true;
                }

                // 替换异常表达式中的值
                if (last_index > -1) {
                    return a.replace(b_value, res);
                } else {
                    return res;
                }
            });
        },
        /**
         * iif 函数解析，使用递归解析
         * @param str 解析表达式
         * @param start 分页数据源开始下标
         * @param end 分页数据源结束下标
         * @param data 数据源
         * @param index 记录行对应的下标
         */
        iifParser: function (str, start, end, data, index) {
            // debugger;
            // str 例如 iif(<记录.金额> = 0,'2','1')
            str = str.replace(/\s/g, "");
            var matchResult = str.match(IIF_REG);
            if (!!matchResult) {
                //匹配出成对的括号（）
                //例如iif(2>3,2,iif(1>3,1,12-(2+2)+(2+2))) 正则会匹配成 iif(1>3,1,12-(2+2)+(2+2))) 多出个括号
                var value = matchResult[0];
                var tmp1 = [], tmp2 = [];
                for (var i = 0; i < value.length; i++) {
                    var bracket1 = value.indexOf("(", i);
                    var bracket2 = value.indexOf(")", i);
                    if (bracket1 > -1) {
                        if (tmp1.indexOf(bracket1) < 0) tmp1.push(bracket1);
                    } else if (bracket1 === -1 && tmp1.length === tmp2.length) break;
                    if (bracket2 > -1) {
                        if (tmp2.indexOf(bracket2) < 0) tmp2.push(bracket2);
                    }
                }
                matchResult[0] = value.substr(0, Number(tmp2[tmp2.length - 1]) + 1);

                // 替换字段的具体内容  返回
                // 解析值 <记录.数量> 的值
                var result = funs.parseTextValue(matchResult[0], start, end, data, index, function (res, value) {
                    return value;
                });
                // console.log('result', result);
                // 获取iif() 的函数内容 /iif\((.*?)\)/g
                var regFn = new RegExp('iif\\((.*)\\)', 'ig');
                result = result.replace(regFn, function (origin, value) {
                    // debugger;
                    // console.log('origin', origin, 'value', value);
                    // 处理嵌套iif情况   1='',,iif(1='',,，)
                    var parts = value.split(','); // 分割成数组 [0=0,2,1]
                    parts[2] = value.replace(String(parts[0])+',', '').replace(String(parts[1]) + ',', '');
                    // console.log(parts);
                    // 根据正则获取
                    var compareRegex = /(\<\>)|(\<\=)|(\>\=)|[\<|\>|\=]/g; // 获取表达式
                    var compare_match_result = parts[0].match(compareRegex); // 计算表达式处理
                    if (!!compare_match_result) {
                        var compare_parts = parts[0].split(compare_match_result[0]);
                        // 如果表达式成立，然后左边值
                        // 如果表达式不成立，然后右边值
                        // 比如 [0=0,2,1] 0=0, 成立，返回2，不成立，返回1
                        // 支持四则运算
                        var regexOperator = /\+|\-|\*|\//m;
                        // debugger;
                        if(regexOperator.test(compare_parts[0]) && !PaintParse.parseResult['operator'][funs.md5(compare_parts[0])])  compare_parts[0] = funs.evalValueFromOperator(compare_parts[0]);
                        if(regexOperator.test(compare_parts[1]) && !PaintParse.parseResult['operator'][funs.md5(compare_parts[1])]) compare_parts[1] = funs.evalValueFromOperator(compare_parts[1]);
                        if(regexOperator.test(parts[1]) && !PaintParse.parseResult['operator'][funs.md5(parts[1])]) parts[1] = funs.evalValueFromOperator(parts[1]);
                        if(regexOperator.test(parts[2]) && !PaintParse.parseResult['operator'][funs.md5(parts[2])]) parts[2] = funs.evalValueFromOperator(parts[2]);

                        // compare_parts[0]、compare_parts[1]如果是数字，进行转换
                        // 匹配空字符串
                        if (compare_parts[0] === "''" || compare_parts[0] === '""') compare_parts[0] = '';
                        if (compare_parts[1] === "''" || compare_parts[1] === '""') compare_parts[1] = '';

                        var numberCompareParts0 = Number(compare_parts[0]);
                        var numberCompareParts1 = Number(compare_parts[1]);
                        if (!isNaN(numberCompareParts0)) compare_parts[0] = numberCompareParts0;
                        if (!isNaN(numberCompareParts1)) compare_parts[1] = numberCompareParts1;

                        // console.log('compare_parts', compare_parts, 'parts', parts);
                        switch (compare_match_result[0]) {
                            case '>':   // 大于
                                if (compare_parts[0] > compare_parts[1]) return parts[1];
                                return parts[2];
                            case '>=':  // 大于等于
                                if (compare_parts[0] >= compare_parts[1]) return parts[1];
                                return parts[2];
                            case '<':   // 小于
                                if (compare_parts[0] < compare_parts[1]) return parts[1];
                                return parts[2];
                            case '<=':  // 小于等于
                                if (compare_parts[0] <= compare_parts[1]) return parts[1];
                                return parts[2];
                            case '<>':  // 不等于
                                if (compare_parts[0] != compare_parts[1]) return parts[1];
                                return parts[2];
                            case '=':  // 等于
                                if (compare_parts[0] == compare_parts[1]) return parts[1];
                                return parts[2];
                            default:   // 表达式不成立，然后原始值
                                return value;
                        }
                    } else {
                        return value;
                    }
                });

                // 结果替换
                str = str.replace(matchResult[0], result);
                return funs.iifParser(str, start, end, data, index);
            } else {
                return str;
            }
        },
        //中缀表达式运算
        getTextFromOperator: function (str) {
            var result = str;
            try {
                if (/-\s*?-/.test(String(result))) {
                    result = result.replace(/-\s*?-/g, '+');
                }

                // 判断 + - * / 分割后有没有0，没有的话补0
                var arr = result.split(/\+|\-|\*|\//);
                if (funs.trim(arr[0]).length === 0) {
                    result = '0 ' + result;
                }
                if (funs.trim(arr[1]).length === 0) {
                    result = result + ' 0';
                }

                result = String(parseFloat(Number(eval(result)).toFixed(4)));
            } catch (err) {
                console.log('getTextFromOperator ' + result, err)
            } finally {
                return funs.getResult(result);
            }
        },

        /**
         * 使用eval计算四则运算
         * @param str
         * @return {string}
         */
        evalValueFromOperator: function (str) {
            var result = str;
            try {
                if (/-\s*?-/.test(String(result))) {
                    result = result.replace(/-\s*?-/g, '+');
                }
                result = String(parseFloat(Number(eval(result)).toFixed(4)));
            } catch (err) {
                console.log('getTextFromOperator ' + result, err)
            } finally {
                return result;
            }
        },
        // 获取文本高度
        getTextHeight: function (obj) {
            var text_arr = funs.subsection(obj.text, obj.font_size, obj.borderStyle.width);
            if (text_arr.length > 0) {
                return obj.borderStyle.height + (text_arr.length - 1) * obj.font_size;
            } else {
                return obj.borderStyle.height;
            }
        },
        // 获取文本宽度
        getTextWidth: function (text, font_size) {
            // var tmp = __this.ctx.font;
            // console.log(text, JSON.stringify(__this.ctx.font));
            var tmp = __this.textFont;
            // __this.ctx.font = font_size + 'px 宋体';
            __this.ctx.font = font_size + 'px ' + objDefaultOptions["font_family"];
            // .replace(/微软雅黑|宋体/ig,function(word){
            //     if(word==='微软雅黑'){
            //         return 'myYaHei'
            //     }else if(word==='宋体'){
            //      return 'mySongTi'
            //     }
            //     return '黑体'
            // });
            var res = __this.ctx.measureText(text).width;
            __this.ctx.font = tmp;
            // (tmp || '').replace(/微软雅黑|宋体/ig,function(word){
            //     if(word==='微软雅黑'){
            //         return 'myYaHei'
            //     }else if(word==='宋体'){
            //      return 'mySongTi'
            //     }
            //     return '黑体'
            // });
            return res;
        },
        // 根据参数 获取对应矩形路径数据
        getRect: function (x, y, width, height) {
            y -= 1;
            return [
                { x: x, y: y },
                { x: x + width, y: y },
                { x: x + width, y: y + height },
                { x: x, y: y + height }
            ];
        },
        //截取文本
        substr: function (text, font_size, len, start) {
            var res = '';
            start = start ? start : 0;
            for (var i = start; i < text.length; i++) {
                res += text[i];
                if (funs.getTextWidth(res, font_size) > len) {
                    res = res.substr(0, res.length - 1);
                    return res;
                }
            }
            return res;
        },
        //分段
        subsection: function (text, font_size, max_len) {
            var res = [];
            var tmp = '';
            while (tmp.length < text.length) {
                var tmp2 = funs.substr(text, font_size, max_len, tmp.length);
                if (tmp2 == '') tmp2 = text[tmp.length];
                tmp += tmp2;
                res.push(tmp2);
            }
            return res;
        },
        //通过处理xy坐标实现一个像素宽
        xyHandlers: function (xy) {
            if (__this.ctx.lineWidth >= 1.5) return;
            xy.x = parseInt(xy.x) + 0.5;
            xy.y = parseInt(xy.y) + 0.5;
        },

        parentAndChild: function () {
            // var list = _VariableStorage.parentAndChildList = {};
            this.judge = function (displayable) {
                var parent_displayable = findHover(displayable.x, displayable.y, function (tmp) {
                    if (!tmp || !tmp.isParent || tmp.dataIndex == displayable.dataIndex) return false;
                    return true;
                });
                if (displayable.dataIndex > 12 && parent_displayable && parent_displayable.isParent) {
                    var list = self.DisplayList[parent_displayable.dataIndex].childs;
                    // if (displayable.parentDataIndex == parent_displayable.dataIndex) {
                    //     return;
                    // }
                    // if (displayable.parentDataIndex != undefined) {
                    //     delete list[displayable.dataIndex];
                    // }
                    // displayable.parentDataIndex = parent_displayable.dataIndex;
                    // list[displayable.dataIndex] = displayable.id;

                    if (displayable.parentDataIndex != parent_displayable.dataIndex) {
                        if (displayable.parentDataIndex != undefined) {
                            delete self.DisplayList[displayable.parentDataIndex].childs[displayable.dataIndex];
                        }
                        displayable.parentDataIndex = parent_displayable.dataIndex;
                        list[displayable.dataIndex] = displayable.id;
                    }
                } else {
                    if (displayable.parentDataIndex != undefined) {
                        delete self.DisplayList[displayable.parentDataIndex].childs[displayable.dataIndex];
                        displayable.parentDataIndex = undefined;
                    }
                }
            };
            this.move = function (dataIndex, x, y) {
                for (var i in self.DisplayList[dataIndex].childs) {
                    self.set(i, {
                        x: self.findById(self.DisplayList[dataIndex].childs[i]).x + x,
                        y: self.findById(self.DisplayList[dataIndex].childs[i]).y + y
                    }, false);
                }
            };
        },
        //绘制元素的8个缩放点
        drawScalePoint: function (param) {
            if (!param) {
                // __this.delete('scale_' + i);
                for (var i = 1; i <= 8; i++) {
                    var obj = funs.findObj('id', 'scale_' + i);
                    if (!obj) continue;
                    funs.setObjOpt(obj.dataIndex, {
                        x: -99,
                        y: -99,
                        isPrint: true
                    }, false, false);
                }
                return;
            }
            if (funs.findObj('id', 'scale_1') || funs.findObj('id', 'scale_2') || funs.findObj('id', 'scale_3') || funs.findObj('id', 'scale_4') || funs.findObj('id', 'scale_5') || funs.findObj('id', 'scale_6') || funs.findObj('id', 'scale_7') || funs.findObj('id', 'scale_8')) return;
            for (var i = 1; i <= 8; i++) {
                draw.addToList({
                    type: 'rect',
                    id: 'scale_' + i,
                    borderStyle: {
                        top: 1, bottom: 1, left: 1, right: 1, background: '#353535', width: __opt['scale_size'],
                        height: __opt['scale_size']
                    },
                    rect: funs.getRect(0, 0, __opt['scale_size'], __opt['scale_size']),
                    zIndex: 999999999999,
                    isDrag: true,
                    x: 0,
                    y: 0
                });
            }
        },

        //十进制加法（修复浮点数精度问题）
        decimalAdd: function (arg1, arg2) {
            var r1, r2, m;
            try {
                r1 = arg1.toString().split('.')[1].length;
            } catch (e) {
                r1 = 0;
            }
            try {
                r2 = arg2.toString().split('.')[1].length;
            } catch (e) {
                r2 = 0;
            }
            m = Math.pow(10, Math.max(r1, r2));
            return (funs.decimalMul(arg1, m) + funs.decimalMul(arg2, m)) / m;
        },

        //十进制减法（修复浮点数精度问题）
        decimalSub: function (arg1, arg2) {
            var r1, r2, m, n;
            try {
                r1 = arg1.toString().split('.')[1].length;
            }
            catch (e) {
                r1 = 0;
            }
            try {
                r2 = arg2.toString().split('.')[1].length;
            }
            catch (e) {
                r2 = 0;
            }
            m = Math.pow(10, Math.max(r1, r2));
            n = (r1 >= r2) ? r1 : r2;
            return Number(((funs.decimalMul(arg1, m) - funs.decimalMul(arg2, m)) / m).toFixed(n));
        },

        //十进制乘法（修复浮点数精度问题）
        decimalMul: function (arg1, arg2) {
            var m = 0,
                s1 = arg1.toString(),
                s2 = arg2.toString();
            try {
                m += s1.split('.')[1].length;
            } catch (e) {
            }
            try {
                m += s2.split('.')[1].length;
            } catch (e) {
            }
            return Number(s1.replace('.', '')) * Number(s2.replace('.', '')) / Math.pow(10, m);
        },

        //十进制除法（修复浮点数精度问题）
        decimalDiv: function (arg1, arg2) {
            var t1 = 0, t2 = 0, r1, r2;
            try {
                t1 = arg1.toString().split('.')[1].length;
            }
            catch (e) {
            }
            try {
                t2 = arg2.toString().split('.')[1].length;
            }
            catch (e) {
            }
            with (Math) {
                r1 = Number(arg1.toString().replace('.', ''));
                r2 = Number(arg2.toString().replace('.', ''));
                return funs.decimalMul((r1 / r2), pow(10, t2 - t1));
            }
        },

        //文本宽度超出边框宽度时，重新计算元素宽度和防止同一行内水平重叠
        overlapMove: function (param) {
            var obj = param;
            var objDataId = funs.get(obj.id, true);
            var text_width = funs.getTextWidth(obj.text, obj.font_size);
            var exceed = text_width - obj.borderStyle.width;
            //当文本内容超出边框宽度时，增加边框宽度
            if (exceed > 0) funs.setObjOpt(objDataId, { borderStyle: { width: obj.borderStyle.width += exceed } }, false, false);
            for (var i = 0; i < __this.DisplayList.length; i++) {
                var tempBlock = funs.extend(true, {}, __this.DisplayList[i]);
                if (tempBlock.id === obj.id || tempBlock.type !== 'text') continue;
                if (obj.y + obj.borderStyle.height > tempBlock.y + tempBlock.borderStyle.height) continue;
                //当元素重叠到同一行其他元素上时则后移
                if (obj.x >= tempBlock.x && obj.x <= tempBlock.x + tempBlock.borderStyle.width && obj.y >= tempBlock.y && obj.y <= tempBlock.y + tempBlock.borderStyle.height) {
                    funs.setObjOpt(objDataId, {
                        x: obj.x += tempBlock.x + tempBlock.borderStyle.width - obj.x,
                    }, false, false);
                }
            }
        },

        //文本高度超出边框高度时，重新计算元素高度和同一行内所有开启自动高度的元素的高度
        autoRowHeight: function (obj, row) {
            // var obj = param;
            // var objDataId = funs.get(obj.id,true);
            // //当文本内容高度超出边框高度需要换行时，增加边框高度
            // if(row > 0){
            //     funs.setObjOpt(objDataId, { borderStyle : {height: obj.borderStyle.height += funs.decimalMul(row , obj.font_size)}}, false);
            //     return;
            // }
            // //同一行内如果有开启自动高度元素并高度大于自身时，则增加自己的边框高度
            // for(var i = 0; i < __this.DisplayList.length; i++){
            //     var tempBlock = funs.extend(true,{},__this.DisplayList[i]);
            //     if(tempBlock.id === obj.id || tempBlock.type !== 'text') continue;
            //     if(obj.y + tempBlock.borderStyle.height > tempBlock.y + tempBlock.borderStyle.height) continue;
            //     if(tempBlock.y <= obj.y && tempBlock.borderStyle.height > obj.borderStyle.height && tempBlock.y + tempBlock.borderStyle.height > obj.y + obj.borderStyle.height ){
            //         funs.setObjOpt(objDataId, {
            //             borderStyle:{
            //                 height:obj.borderStyle.height += tempBlock.borderStyle.height - obj.borderStyle.height
            //             }
            //         }, false);
            //     }
            // }
            //funs.subsection换行
            var text_arr = funs.subsection(obj.text, obj.font_size, obj.borderStyle.width);
            if (text_arr <= 1) return;
            //增加边框高
            obj.borderStyle.height += funs.decimalMul((text_arr.length - 1), obj.font_size);
            for (var i = 0; i < __this.DisplayList.length; i++) {
                var tempBlock = __this.DisplayList[i];
                if (tempBlock.id === obj.id || tempBlock.type !== 'text') continue;
                if (obj.y !== tempBlock.y || obj.y + obj.borderStyle.height < tempBlock.y + tempBlock.borderStyle.height) continue;
                if (funs.subsection(tempBlock.text, tempBlock.font_size, tempBlock.borderStyle.width).length > 1) continue;
                tempBlock.borderStyle.height += obj.borderStyle.height - tempBlock.borderStyle.height;
            }
        },

        //文本宽度超出边框宽度时，增加边框宽度
        autoRowWidth: function (obj, row) {
            var textWidth = funs.getTextWidth(obj.text, obj.font_size);
            if (!!obj.autoWidth && !obj.autoHeight && textWidth > obj.borderStyle.width) {
                obj.borderStyle.width = textWidth + (obj.borderStyle.top || obj.borderStyle.bottom || obj.borderStyle.left || obj.borderStyle.right || 0) * 2;
            }
        },

        //文本高度超出边框高度时，增加边框高度
        maxRowHeight: function (obj) {
            var text_arr = funs.subsection(obj.text, obj.font_size, obj.borderStyle.width);
            if (text_arr <= 1) return;
            obj.borderStyle.height = obj.parentHeight;
        },

        //图片比例缩放
        imgScale: function (obj) {
            if ((obj.img_scale || typeof obj.img_scale === 'undefined') && obj.borderStyle.orig_height) {
                var tmp_obj = obj.borderStyle;
                if ((tmp_obj.orig_width < tmp_obj.orig_height || tmp_obj.orig_width > tmp_obj.orig_height)
                    && (tmp_obj.width > tmp_obj.height || tmp_obj.width == tmp_obj.height || tmp_obj.orig_width == tmp_obj.orig_height)) {
                    var oldwidth = JSON.parse(JSON.stringify(tmp_obj.width));
                    tmp_obj.width = parseFloat(parseFloat(tmp_obj.orig_width * (tmp_obj.height / tmp_obj.orig_height)).toFixed(0));
                    obj.x = parseFloat(parseFloat(obj.x + (oldwidth - tmp_obj.width) / 2)).toFixed(0);
                } else if ((tmp_obj.orig_width < tmp_obj.orig_height || tmp_obj.orig_width > tmp_obj.orig_height || tmp_obj.orig_width == tmp_obj.orig_height)
                    && tmp_obj.width < tmp_obj.height) {
                    var oldheight = JSON.parse(JSON.stringify(tmp_obj.height));
                    tmp_obj.height = parseFloat(parseFloat(tmp_obj.orig_height * (tmp_obj.width / tmp_obj.orig_width)).toFixed(0));
                    obj.y = parseFloat(parseFloat(obj.y + (oldheight - tmp_obj.height) / 2)).toFixed(0);
                }
            }
        },


        //数字转字节
        numberToByte: function (param) {
            if (isNaN(param) || param > 255) return 0;
            if (param <= 127) return param;
            return -(256 - param);
        },

        //将二值化数据转为打印所需数据
        imageDataToBlueToothPrint: function (data) {
            var result = [];
            var dataLen = data.length;
            for (var i = 0; i < dataLen; i += 24) {
                var dataSubLen = data[i].length;
                for (var j = 0; j < dataSubLen; j++) {
                    var c1 = (data[i] === void 0 ? 0 : (data[i][j] === void 0 ? 0 : (data[i][j] == 0 ? 128 : 0)));
                    var c2 = (data[i + 1] === void 0 ? 0 : (data[i + 1][j] === void 0 ? 0 : (data[i + 1][j] == 0 ? 64 : 0)));
                    var c3 = (data[i + 2] === void 0 ? 0 : (data[i + 2][j] === void 0 ? 0 : (data[i + 2][j] == 0 ? 32 : 0)));
                    var c4 = (data[i + 3] === void 0 ? 0 : (data[i + 3][j] === void 0 ? 0 : (data[i + 3][j] == 0 ? 16 : 0)));
                    var c5 = (data[i + 4] === void 0 ? 0 : (data[i + 4][j] === void 0 ? 0 : (data[i + 4][j] == 0 ? 8 : 0)));
                    var c6 = (data[i + 5] === void 0 ? 0 : (data[i + 5][j] === void 0 ? 0 : (data[i + 5][j] == 0 ? 4 : 0)));
                    var c7 = (data[i + 6] === void 0 ? 0 : (data[i + 6][j] === void 0 ? 0 : (data[i + 6][j] == 0 ? 2 : 0)));
                    var c8 = (data[i + 7] === void 0 ? 0 : (data[i + 7][j] === void 0 ? 0 : (data[i + 7][j] == 0 ? 1 : 0)));

                    var c9 = (data[i + 8] === void 0 ? 0 : (data[i + 8][j] === void 0 ? 0 : (data[i + 8][j] == 0 ? 128 : 0)));
                    var c10 = (data[i + 9] === void 0 ? 0 : (data[i + 9][j] === void 0 ? 0 : (data[i + 9][j] == 0 ? 64 : 0)));
                    var c11 = (data[i + 10] === void 0 ? 0 : (data[i + 10][j] === void 0 ? 0 : (data[i + 10][j] == 0 ? 32 : 0)));
                    var c12 = (data[i + 11] === void 0 ? 0 : (data[i + 11][j] === void 0 ? 0 : (data[i + 11][j] == 0 ? 16 : 0)));
                    var c13 = (data[i + 12] === void 0 ? 0 : (data[i + 12][j] === void 0 ? 0 : (data[i + 12][j] == 0 ? 8 : 0)));
                    var c14 = (data[i + 13] === void 0 ? 0 : (data[i + 13][j] === void 0 ? 0 : (data[i + 13][j] == 0 ? 4 : 0)));
                    var c15 = (data[i + 14] === void 0 ? 0 : (data[i + 14][j] === void 0 ? 0 : (data[i + 14][j] == 0 ? 2 : 0)));
                    var c16 = (data[i + 15] === void 0 ? 0 : (data[i + 15][j] === void 0 ? 0 : (data[i + 15][j] == 0 ? 1 : 0)));

                    var c17 = (data[i + 16] === void 0 ? 0 : (data[i + 16][j] === void 0 ? 0 : (data[i + 16][j] == 0 ? 128 : 0)));
                    var c18 = (data[i + 17] === void 0 ? 0 : (data[i + 17][j] === void 0 ? 0 : (data[i + 17][j] == 0 ? 64 : 0)));
                    var c19 = (data[i + 18] === void 0 ? 0 : (data[i + 18][j] === void 0 ? 0 : (data[i + 18][j] == 0 ? 32 : 0)));
                    var c20 = (data[i + 19] === void 0 ? 0 : (data[i + 19][j] === void 0 ? 0 : (data[i + 19][j] == 0 ? 16 : 0)));
                    var c21 = (data[i + 20] === void 0 ? 0 : (data[i + 20][j] === void 0 ? 0 : (data[i + 20][j] == 0 ? 8 : 0)));
                    var c22 = (data[i + 21] === void 0 ? 0 : (data[i + 21][j] === void 0 ? 0 : (data[i + 21][j] == 0 ? 4 : 0)));
                    var c23 = (data[i + 22] === void 0 ? 0 : (data[i + 22][j] === void 0 ? 0 : (data[i + 22][j] == 0 ? 2 : 0)));
                    var c24 = (data[i + 23] === void 0 ? 0 : (data[i + 23][j] === void 0 ? 0 : (data[i + 23][j] == 0 ? 1 : 0)));

                    var index = Math.floor(i / 24);
                    if (result[index] === void 0) result[index] = [];
                    result[index].push(funs.numberToByte(c1 + c2 + c3 + c4 + c5 + c6 + c7 + c8));
                    result[index].push(funs.numberToByte(c9 + c10 + c11 + c12 + c13 + c14 + c15 + c16));
                    result[index].push(funs.numberToByte(c17 + c18 + c19 + c20 + c21 + c22 + c23 + c24));
                }
            }

            return result;
        },

        //图像二值化（一维OTSU图像处理算法）
        OTSUAlgorithm: function (param) {
            var m_pFstdHistogram = new Array();//表示灰度值的分布点概率
            var m_pFGrayAccu = new Array();//其中每一个值等于m_pFstdHistogram中从0到当前下标值的和
            var m_pFGrayAve = new Array();//其中每一值等于m_pFstdHistogram中从0到当前指定下标值*对应的下标之和
            var m_pAverage = 0;//值为m_pFstdHistogram【256】中每一点的分布概率*当前下标之和
            var m_pHistogram = new Array();//灰度直方图
            var i, j;
            var temp = 0, fMax = 0;//定义一个临时变量和一个最大类间方差的值
            var nThresh = 0;//最优阀值

            //初始化各项参数
            for (i = 0; i < 256; i++) {
                m_pFstdHistogram[i] = 0;
                m_pFGrayAccu[i] = 0;
                m_pFGrayAve[i] = 0;
                m_pHistogram[i] = 0;
            }
            //获取图像信息
            var canvasData = param;
            //获取图像的像素
            var pixels = canvasData.data;
            //下面统计图像的灰度分布信息
            for (i = 0; i < pixels.length; i += 4) {
                //获取r的像素值，因为灰度图像，r=g=b，所以取第一个即可
                var r = pixels[i];
                m_pHistogram[r]++;
            }
            //下面计算每一个灰度点在图像中出现的概率
            var size = canvasData.width * canvasData.height;
            for (i = 0; i < 256; i++) {
                m_pFstdHistogram[i] = m_pHistogram[i] / size;
            }
            //下面开始计算m_pFGrayAccu和m_pFGrayAve和m_pAverage的值
            for (i = 0; i < 256; i++) {
                for (j = 0; j <= i; j++) {
                    //计算m_pFGaryAccu[256]
                    m_pFGrayAccu[i] += m_pFstdHistogram[j];
                    //计算m_pFGrayAve[256]
                    m_pFGrayAve[i] += j * m_pFstdHistogram[j];
                }
                //计算平均值
                m_pAverage += i * m_pFstdHistogram[i];
            }
            //下面开始就算OSTU的值，从0-255个值中分别计算ostu并寻找出最大值作为分割阀值
            for (i = 0; i < 256; i++) {
                temp = (m_pAverage * m_pFGrayAccu[i] - m_pFGrayAve[i])
                    * (m_pAverage * m_pFGrayAccu[i] - m_pFGrayAve[i])
                    / (m_pFGrayAccu[i] * (1 - m_pFGrayAccu[i]));
                if (temp > fMax) {
                    fMax = temp;
                    nThresh = i;
                }
            }
            var byteArr = [];
            //下面执行二值化过程
            for (i = 0; i < canvasData.width; i++) {
                for (j = 0; j < canvasData.height; j++) {
                    //取得每一点的位置
                    var ids = (i + j * canvasData.width) * 4;
                    //取得像素的R分量的值
                    var r = canvasData.data[ids];
                    //与阀值进行比较，如果小于阀值，那么将改点置为0，否则置为255
                    var gray = r > nThresh ? 255 : 0;
                    canvasData.data[ids + 0] = gray;
                    canvasData.data[ids + 1] = gray;
                    canvasData.data[ids + 2] = gray;
                    if (byteArr[j] === undefined) byteArr[j] = [];
                    byteArr[j].push(gray);
                }
            }
            //显示二值化图像
            // var canvas = document.getElementById('canvas');
            // if(canvas){
            //     var newImage = canvas.getContext('2d');
            //     canvas.height = canvasData.height;
            //     canvas.width = canvasData.width;
            //     newImage.putImageData(canvasData, 0, 0);
            // }

            return byteArr;
        },

        /**
         * 计算需要填充的空行
         * 根据 空白行数 - 数据行数  计算出需生成的空白记录行。
         * @param {*} options 必传参数 { data_main: '', dataMaxHeight: '', rowHeight: '', pageData: '', top: '' }
         */
        calcFillBlankLine: function (options) {
            var _dataMaxHeight = options.dataMaxHeight;
            var rowHeight = options.rowHeight;
            var data_main = options.data_main;
            var pageData = options.pageData;
            var top = options.top;

            var blankLines = '';
            if (__opt.pageConfig.blankLines !== '' && typeof __opt.pageConfig.blankLines !== 'undefined') {
                blankLines = __opt.pageConfig.blankLines - options.printLines;   // 可绘制的空行行数
                // 如果值不为空，根据当前填写的值，生成报表时，先计算：
                // 数据记录行数：即有包含打印数据行数
                // 根据 空白行数 - 数据行数  计算出需生成的空白记录行。
                blankLines = blankLines >= 0 ? blankLines : 0;
            }

            // 判断是否可以绘制空白行
            var canAppendBlankLine = function () {
                if (blankLines !== '') {
                    // 每循环一次，可绘制的空白行减1，直到空白行为0
                    if (blankLines <= 0) {
                        return false;
                    }
                    blankLines--;
                }
                return _dataMaxHeight > rowHeight;
            };

            // 开启绘制默认行
            if (__opt.pageConfig.drawBlank) {
                while (canAppendBlankLine()) {
                    if (!data_main.childs || data_main.childs.length === 0) break;
                    var childs = [];
                    data_main.childs.forEach(function (value, index, data) {
                        var obj = funs.extend(true, {}, funs.findObj('id', value, __opt.pageStyle));
                        if (!obj) return;

                        //防止出现不重合边框
                        obj = funs.extend(true, obj, { borderStyle: { top: 0 } });
                        obj.text = funs.getText(obj, '');
                        obj.y = top;
                        if (rowHeight < funs.getTextHeight(obj)) {
                            rowHeight = funs.getTextHeight(obj)
                        }
                        childs.push(obj);
                    });
                    childs.forEach(function (value, index, data) { value.borderStyle.height = rowHeight; });
                    top = top + rowHeight;
                    _dataMaxHeight -= rowHeight;

                    pageData = pageData.concat(childs);
                }
            }

            return { pageData: pageData, top: top };
        },

        // md5加密算法
        md5: (function () {
            function safeAdd(x, y) {
                var lsw = (x & 0xffff) + (y & 0xffff)
                var msw = (x >> 16) + (y >> 16) + (lsw >> 16)
                return (msw << 16) | (lsw & 0xffff)
            }

            function bitRotateLeft(num, cnt) {
                return (num << cnt) | (num >>> (32 - cnt))
            }

            function md5cmn(q, a, b, x, s, t) {
                return safeAdd(bitRotateLeft(safeAdd(safeAdd(a, q), safeAdd(x, t)), s), b)
            }

            function md5ff(a, b, c, d, x, s, t) {
                return md5cmn((b & c) | (~b & d), a, b, x, s, t)
            }

            function md5gg(a, b, c, d, x, s, t) {
                return md5cmn((b & d) | (c & ~d), a, b, x, s, t)
            }

            function md5hh(a, b, c, d, x, s, t) {
                return md5cmn(b ^ c ^ d, a, b, x, s, t)
            }

            function md5ii(a, b, c, d, x, s, t) {
                return md5cmn(c ^ (b | ~d), a, b, x, s, t)
            }

            function binlMD5(x, len) {
                x[len >> 5] |= 0x80 << (len % 32)
                x[((len + 64) >>> 9 << 4) + 14] = len
                var i
                var olda
                var oldb
                var oldc
                var oldd
                var a = 1732584193
                var b = -271733879
                var c = -1732584194
                var d = 271733878
                for (i = 0; i < x.length; i += 16) {
                    olda = a
                    oldb = b
                    oldc = c
                    oldd = d
                    a = md5ff(a, b, c, d, x[i], 7, -680876936)
                    d = md5ff(d, a, b, c, x[i + 1], 12, -389564586)
                    c = md5ff(c, d, a, b, x[i + 2], 17, 606105819)
                    b = md5ff(b, c, d, a, x[i + 3], 22, -1044525330)
                    a = md5ff(a, b, c, d, x[i + 4], 7, -176418897)
                    d = md5ff(d, a, b, c, x[i + 5], 12, 1200080426)
                    c = md5ff(c, d, a, b, x[i + 6], 17, -1473231341)
                    b = md5ff(b, c, d, a, x[i + 7], 22, -45705983)
                    a = md5ff(a, b, c, d, x[i + 8], 7, 1770035416)
                    d = md5ff(d, a, b, c, x[i + 9], 12, -1958414417)
                    c = md5ff(c, d, a, b, x[i + 10], 17, -42063)
                    b = md5ff(b, c, d, a, x[i + 11], 22, -1990404162)
                    a = md5ff(a, b, c, d, x[i + 12], 7, 1804603682)
                    d = md5ff(d, a, b, c, x[i + 13], 12, -40341101)
                    c = md5ff(c, d, a, b, x[i + 14], 17, -1502002290)
                    b = md5ff(b, c, d, a, x[i + 15], 22, 1236535329)
                    a = md5gg(a, b, c, d, x[i + 1], 5, -165796510)
                    d = md5gg(d, a, b, c, x[i + 6], 9, -1069501632)
                    c = md5gg(c, d, a, b, x[i + 11], 14, 643717713)
                    b = md5gg(b, c, d, a, x[i], 20, -373897302)
                    a = md5gg(a, b, c, d, x[i + 5], 5, -701558691)
                    d = md5gg(d, a, b, c, x[i + 10], 9, 38016083)
                    c = md5gg(c, d, a, b, x[i + 15], 14, -660478335)
                    b = md5gg(b, c, d, a, x[i + 4], 20, -405537848)
                    a = md5gg(a, b, c, d, x[i + 9], 5, 568446438)
                    d = md5gg(d, a, b, c, x[i + 14], 9, -1019803690)
                    c = md5gg(c, d, a, b, x[i + 3], 14, -187363961)
                    b = md5gg(b, c, d, a, x[i + 8], 20, 1163531501)
                    a = md5gg(a, b, c, d, x[i + 13], 5, -1444681467)
                    d = md5gg(d, a, b, c, x[i + 2], 9, -51403784)
                    c = md5gg(c, d, a, b, x[i + 7], 14, 1735328473)
                    b = md5gg(b, c, d, a, x[i + 12], 20, -1926607734)
                    a = md5hh(a, b, c, d, x[i + 5], 4, -378558)
                    d = md5hh(d, a, b, c, x[i + 8], 11, -2022574463)
                    c = md5hh(c, d, a, b, x[i + 11], 16, 1839030562)
                    b = md5hh(b, c, d, a, x[i + 14], 23, -35309556)
                    a = md5hh(a, b, c, d, x[i + 1], 4, -1530992060)
                    d = md5hh(d, a, b, c, x[i + 4], 11, 1272893353)
                    c = md5hh(c, d, a, b, x[i + 7], 16, -155497632)
                    b = md5hh(b, c, d, a, x[i + 10], 23, -1094730640)
                    a = md5hh(a, b, c, d, x[i + 13], 4, 681279174)
                    d = md5hh(d, a, b, c, x[i], 11, -358537222)
                    c = md5hh(c, d, a, b, x[i + 3], 16, -722521979)
                    b = md5hh(b, c, d, a, x[i + 6], 23, 76029189)
                    a = md5hh(a, b, c, d, x[i + 9], 4, -640364487)
                    d = md5hh(d, a, b, c, x[i + 12], 11, -421815835)
                    c = md5hh(c, d, a, b, x[i + 15], 16, 530742520)
                    b = md5hh(b, c, d, a, x[i + 2], 23, -995338651)
                    a = md5ii(a, b, c, d, x[i], 6, -198630844)
                    d = md5ii(d, a, b, c, x[i + 7], 10, 1126891415)
                    c = md5ii(c, d, a, b, x[i + 14], 15, -1416354905)
                    b = md5ii(b, c, d, a, x[i + 5], 21, -57434055)
                    a = md5ii(a, b, c, d, x[i + 12], 6, 1700485571)
                    d = md5ii(d, a, b, c, x[i + 3], 10, -1894986606)
                    c = md5ii(c, d, a, b, x[i + 10], 15, -1051523)
                    b = md5ii(b, c, d, a, x[i + 1], 21, -2054922799)
                    a = md5ii(a, b, c, d, x[i + 8], 6, 1873313359)
                    d = md5ii(d, a, b, c, x[i + 15], 10, -30611744)
                    c = md5ii(c, d, a, b, x[i + 6], 15, -1560198380)
                    b = md5ii(b, c, d, a, x[i + 13], 21, 1309151649)
                    a = md5ii(a, b, c, d, x[i + 4], 6, -145523070)
                    d = md5ii(d, a, b, c, x[i + 11], 10, -1120210379)
                    c = md5ii(c, d, a, b, x[i + 2], 15, 718787259)
                    b = md5ii(b, c, d, a, x[i + 9], 21, -343485551)
                    a = safeAdd(a, olda)
                    b = safeAdd(b, oldb)
                    c = safeAdd(c, oldc)
                    d = safeAdd(d, oldd)
                }
                return [a, b, c, d]
            }

            function binl2rstr(input) {
                var i
                var output = ''
                var length32 = input.length * 32
                for (i = 0; i < length32; i += 8) {
                    output += String.fromCharCode((input[i >> 5] >>> (i % 32)) & 0xff)
                }
                return output
            }

            function rstr2binl(input) {
                var i
                var output = []
                output[(input.length >> 2) - 1] = undefined
                for (i = 0; i < output.length; i += 1) {
                    output[i] = 0
                }
                var length8 = input.length * 8
                for (i = 0; i < length8; i += 8) {
                    output[i >> 5] |= (input.charCodeAt(i / 8) & 0xff) << (i % 32)
                }
                return output
            }

            function rstrMD5(s) {
                return binl2rstr(binlMD5(rstr2binl(s), s.length * 8))
            }

            function rstrHMACMD5(key, data) {
                var i
                var bkey = rstr2binl(key)
                var ipad = []
                var opad = []
                var hash
                ipad[15] = opad[15] = undefined
                if (bkey.length > 16) {
                    bkey = binlMD5(bkey, key.length * 8)
                }
                for (i = 0; i < 16; i += 1) {
                    ipad[i] = bkey[i] ^ 0x36363636
                    opad[i] = bkey[i] ^ 0x5c5c5c5c
                }
                hash = binlMD5(ipad.concat(rstr2binl(data)), 512 + data.length * 8)
                return binl2rstr(binlMD5(opad.concat(hash), 512 + 128))
            }

            function rstr2hex(input) {
                var hexTab = '0123456789abcdef'
                var output = ''
                var x
                var i
                for (i = 0; i < input.length; i += 1) {
                    x = input.charCodeAt(i)
                    output += hexTab.charAt((x >>> 4) & 0x0f) + hexTab.charAt(x & 0x0f)
                }
                return output
            }

            function str2rstrUTF8(input) {
                return unescape(encodeURIComponent(input))
            }

            function rawMD5(s) {
                return rstrMD5(str2rstrUTF8(s))
            }

            function hexMD5(s) {
                return rstr2hex(rawMD5(s))
            }

            function rawHMACMD5(k, d) {
                return rstrHMACMD5(str2rstrUTF8(k), str2rstrUTF8(d))
            }

            function hexHMACMD5(k, d) {
                return rstr2hex(rawHMACMD5(k, d))
        }

            function md5(string, key, raw) {
                if (!key) {
                    if (!raw) {
                        return hexMD5(string)
                    }
                    return rawMD5(string)
                }
                if (!raw) {
                    return hexHMACMD5(key, string)
                }
                return rawHMACMD5(key, string)
            }

            return md5;
        })(),
    };

    // 绘制方法
    var draw = {
        addToList: function (obj) {
            obj.id = obj.id ? obj.id : 'id_' + (new Date().getTime().toString().substr(3)) + (parseInt(Math.random() * 10000));
            obj.dataIndex = __this.DisplayList.length;
            obj.zIndex = obj.zIndex != undefined ? obj.zIndex : obj.dataIndex;
            obj.isParent = obj.isParent ? true : false;
            obj.parentDataIndex = undefined;
            obj.childs = [];
            obj._handlers = {};
            __this.DisplayList.push(obj);
            funs.setObjOpt(obj.dataIndex, obj, true, false);
            return obj.dataIndex;
        },
        // 选择
        Selected: function (obj, isDraw) {
            if (obj.id && /scale_[1-8]/.test(obj.id)) return;
            if (/margin_top|margin_bottom|margin_left|margin_right|margin_centen/.test(obj.id)) return;
            funs.drawScalePoint(true);
            for (var i = 1; i <= 8; i++) {
                var objScale = funs.findObj('id', 'scale_' + i);
                var x = obj.x - __opt.scale_size / 2;
                var y = obj.y - __opt.scale_size / 2;
                var width = obj.borderStyle.width + __opt.scale_size;
                var height = obj.borderStyle.height + __opt.scale_size;
                if (i < 4) {
                    var offset_x = width / 2 - __opt.scale_size / 2;
                    x += (i - 1) * offset_x;
                } else if (i == 4 || i == 5) {
                    x += (i - 4) * obj.borderStyle.width;
                    var offset_y = height / 2 - __opt.scale_size / 2;
                    y += offset_y;
                } else {
                    var offset_x = width / 2 - __opt.scale_size / 2;
                    x += (i - 6) * offset_x;
                    y += obj.borderStyle.height;
                }
                funs.setObjOpt(objScale.dataIndex, {
                    x: x,
                    y: y,
                    isPrint: true
                }, false, false);
            }
            if (typeof isDraw === 'undefined' || isDraw === true) {
                draw.drawAll();
            }
        },
        // 绘画 蓝色 虚线
        drawDash: function (dataIndex) {
            var obj = funs.extend(true, {}, __this.DisplayList[dataIndex]);
            if (JSON.stringify(obj) === '{}') return;
            var width = obj.borderStyle.width;
            var height = obj.borderStyle.height;
            var x = obj.x;
            var y = obj.y;
            var style = {
                lineDesh: true,
                color: '#3c77d0',
                lineWidth: 1.3
            };
            draw.Line({ x: x, y: y }, { x: x + width, y: y }, style);
            draw.Line({ x: x + width, y: y }, { x: x + width, y: y + height }, style);
            draw.Line({ x: x + width, y: y + height }, { x: x, y: y + height }, style);
            draw.Line({ x: x, y: y + height }, { x: x, y: y }, style);
        },

        /**
         * 绘制-所有，如果操作没有绘制数据，返回 false
         * @param callback
         * @return {boolean}
         */
        drawAll: function (callback) {
            var drawList = { len: 0 };      //图片加载队列
            __this.ctx.clearRect(0, 0, __this.canvas.width, __this.canvas.height);
            __this.ctx.fillStyle = '#ffffff';
            __this.ctx.fillRect(0, 0, __this.canvas.width, __this.canvas.height);

            var list = __this.DisplayList.concat();
            if (list.length === 0) {
                return false;
            }

            list.sort(function (a, b) {
                return a.zIndex - b.zIndex;
            });
            for (var i = 0; i < list.length; i++) {
                var obj = list[i];
                if (obj == undefined || obj.isPrint === false || !funs.isCanDraw(obj)) continue;
                switch (obj.type) {
                    case 'text':
                        draw.Text(obj);
                        if (__opt['mode'] !== 'edit') {
                            drawList.len += 1;
                            setTimeout(function () {
                                completionCallback();
                            }, 0);
                        }
                        break;
                    case 'rect':
                        draw.Rect(obj);
                        if (__opt['mode'] !== 'edit') {
                            drawList.len += 1;
                            setTimeout(function () {
                                completionCallback();
                            }, 0);
                        }
                        break;
                    case 'icon':
                        if (__opt['mode'] === 'edit') {
                            draw.Rect(obj);
                        } else {
                            drawList.len += 1;
                            draw.Image(obj, completionCallback);
                        }
                        break;
                    case 'barcode':
                        if (__opt['mode'] === 'edit') {
                            draw.Rect(obj)
                        } else {
                            drawList.len += 1;
                            draw.BarCode(obj, completionCallback);
                        };
                        break;
                    case 'qrcode':
                        // (__opt['mode'] === 'edit') ? draw.Rect(obj) : draw.QRBar(obj);

                        if (__opt['mode'] === 'edit') {
                            draw.Rect(obj)
                        } else {
                            drawList.len += 1;
                            draw.QRBar(obj, completionCallback);
                        }
                        break;
                    case 'miniCode':
                        if (__opt['mode'] === 'edit') {
                            draw.Rect(obj);
                        } else {
                            drawList.len += 1;
                            draw.miniCode(obj, completionCallback);
                        }
                        break;
                }
                // if (__opt['mode'] === 'edit' && obj.selected) {
                // draw.Selected(obj,false);
                // }
            }
            // if(!!__this.selectObj){
            // 绘制选中的元素边框
            if (!!__this.dragInfo.objList && !!__this.dragInfo.objList.length) {
                // draw.drawDash(__this.selectObj.dataIndex);
                for (var x = 0; x < __this.dragInfo.objList.length; x++) {
                    draw.drawDash(__this.dragInfo.objList[x].dataIndex);
                }
            }

            // 绘制鼠标多选的蓝色区域
            if (__this.ctrlAndMousedown.is_ctrl && __this.ctrlAndMousedown.is_mousedown) {
                // console.log('__this.ctrlAndMousedown', JSON.parse(JSON.stringify(__this.ctrlAndMousedown)));
                var rectStartX = __this.ctrlAndMousedown.startX; // 起点坐标x
                var rectStartY = __this.ctrlAndMousedown.startY; // 起点坐标y
                var rectEndX = __this.ctrlAndMousedown.endX;
                var rectEndY = __this.ctrlAndMousedown.endY;

                // 绘制蓝色虚线
                var style = { lineDesh: true, color: '#3c77d0', lineWidth: 0.3 };

                draw.Line({ x: rectStartX, y: rectStartY }, { x: rectEndX, y: rectStartY }, style); // 左上至右上
                draw.Line({ x: rectStartX, y: rectStartY }, { x: rectStartX, y: rectEndY }, style); // 左上至左下
                draw.Line({ x: rectStartX, y: rectEndY }, { x: rectEndX, y: rectEndY }, style); // 左下至右下
                draw.Line({ x: rectEndX, y: rectStartY }, { x: rectEndX, y: rectEndY }, style); // 右上至右下
            }

            // 绘制对齐线
            for (var i = 0; i < _VariableStorage.Snaplines.length; i++) {
                var _obj = _VariableStorage.Snaplines[i];
                switch (_obj.type) {
                    case 'line':
                        draw.Line(_obj);
                        break;
                }
            }

            //图片加载完成回调
            function completionCallback() {
                if (drawList.len > 0) drawList.len -= 1;
                if (drawList.len === 0) {
                    if (typeof callback === 'function') callback();
                }
            }

            return true;
        },
        // 绘制-线条
        Line: function (origin, end, style) {
            if (typeof origin === 'object' && end == undefined) {
                style = origin.style;
                end = origin.end;
                origin = origin.origin;
            }
            style = style ? style : {};
            __this.ctx.lineWidth = style.lineWidth ? parseFloat(style.lineWidth) : 1;
            __this.ctx.strokeStyle = style.color ? style.color : '#000';
            funs.xyHandlers(origin);
            funs.xyHandlers(end);
            if (style.lineDesh) {
                __this.ctx.setLineDash([3, 2]);
            } else {
                __this.ctx.setLineDash([1, 0]);
            }
            __this.ctx.beginPath();
            if (style.type == 'left') {
                __this.ctx.moveTo(origin.x, origin.y + (style.lineHeight || 0));
                __this.ctx.lineTo(end.x, end.y - (style.lineHeight || 0));
            } else if (style.type == 'right') {
                __this.ctx.moveTo(origin.x, origin.y - (style.lineHeight || 0));
                __this.ctx.lineTo(end.x, end.y + (style.lineHeight || 0));
            } else {
                __this.ctx.moveTo(origin.x, origin.y);
                __this.ctx.lineTo(end.x, end.y);
            }

            __this.ctx.stroke();
            __this.ctx.closePath();
        },
        // 绘制-边框
        Border: function (x, y, style) {
            if (typeof x == 'object') {
                style = x.borderStyle;
                y = x.y;
                x = x.x;
            }
            if (style.selected) {
                return;
            }
            var width = style.width;
            var height = style.height;

            style.top && draw.Line({ x: x, y: y }, { x: x + width, y: y }, { type: 'top', lineHeight: style.right / 2, lineWidth: style.top, color: style.color });
            style.right && draw.Line({ x: x + width, y: y }, { x: x + width, y: y + height }, { type: 'right', lineHeight: style.right / 2, lineWidth: style.right, color: style.color });
            style.bottom && draw.Line({ x: x + width, y: y + height }, { x: x, y: y + height }, { type: 'bottom', lineHeight: style.right / 2, lineWidth: style.bottom, color: style.color });
            style.left && draw.Line({ x: x, y: y + height }, { x: x, y: y }, { type: 'left', lineHeight: style.left / 2, lineWidth: style.left, color: style.color });
        },
        //绘制-矩形(色块)
        Rect: function (x, y, style) {
            // if(!!obj && obj.barcodeDirection) {
            //     __this.ctx.rotate(10);
            //     // __this.ctx.transform(1, Math.PI / 6, -Math.PI / 6, 1, 0, 0);
            // }else {
            //     __this.ctx.rotate(0);
            //     // __this.ctx.transform(1,0,0,1,0,0);
            // }
            var border = false;
            var obj = {};
            if (typeof x === 'object') {
                obj = x;
                style = x.borderStyle;
                y = x.y;
                x = x.x;
                border = true;
            }
            var xy = { x: x, y: y };
            funs.xyHandlers(xy);
            __this.ctx.fillStyle = style.background ? style.background : 'rgba(255,255,255,0)';
            var width = style.width;
            var height = style.height;
            __this.ctx.fillRect(xy.x, xy.y, width, height);
            if (typeof obj.title === 'object') {
                draw.Rect(xy.x, xy.y, {
                    background: 'rgba(227,227,227,0.6)',
                    width: style.width,
                    height: 20
                });
                __this.ctx.font = 'normal bold 16px 宋体';
                __this.textFont = 'normal bold 16px 宋体';
                __this.ctx.fillStyle = '#000';
                __this.ctx.fillText(obj.title.text, xy.x, xy.y + 16);
                if (obj.dataSet) {
                    __this.ctx.fillText(obj.dataSet, width - funs.getTextWidth(obj.dataSet, 16), xy.y + 16);
                }
            }
            if (obj.text) {
                var textY;
                if (height - 20 < 16) {
                    textY = xy.y + 36;
                } else {
                    textY = xy.y + ((height - 20 - 16) / 2) + 36;
                }
                __this.ctx.fillText(obj.text, xy.x, textY, width);
            }
            border && draw.Border(x, y, style);
        },
        // 绘制-文本
        Text: function (obj) {
            // obj.textWidth = funs.getTextWidth(obj.text, obj.font_size);
            obj.rect = funs.getRect(obj.x, obj.y, obj.borderStyle.width, obj.borderStyle.height);

            // obj = funs.extend(true, {}, obj);
            var font = '';
            var lineWidth = (obj.borderStyle.top || obj.borderStyle.bottom || obj.borderStyle.left || obj.borderStyle.right || 0) * 2;
            if (obj.bold) font += 'bold ';
            if (obj.oblique) font += 'oblique ';
            // __this.ctx.font = font + obj.font_size + 'px 宋体';
            __this.ctx.font = font + obj.font_size + 'px ' + (!obj['font_family'] ? objDefaultOptions.font_family : obj.font_family);
            __this.textFont = font + obj.font_size + 'px ' + (!obj['font_family'] ? objDefaultOptions.font_family : obj.font_family);

            if (__opt['mode'] !== 'edit') {
                if (obj.autoHeight) {
                    if (obj.parentID === "data_main" && !obj.data_main_auto_height) {
                        var text_arr = funs.subsection(obj.text, obj.font_size, obj.borderStyle.width - lineWidth).slice(0, 1);
                    } else {
                        var text_arr = funs.subsection(obj.text, obj.font_size, obj.borderStyle.width - lineWidth);
                    }
                } else if (obj.maxHeight || obj.auto_br) {
                    // if(obj.parentID === "data_main" && !obj.data_main_auto_height) {
                    //     var text_arr = funs.subsection(obj.text, obj.font_size, obj.borderStyle.width).slice(0, 1);
                    // }else{
                    //最大高度
                    var tmpText_arr = funs.subsection(obj.text, obj.font_size, obj.borderStyle.width - lineWidth).length;
                    // funs.findObj('id', 'scale_1')
                    function getText_arr(textArr) {
                        var parentHeight;

                        if (obj.maxHeight) {
                            var maxHeight = obj.parentHeight;
                        } else if (obj.auto_br) {
                            var maxHeight = obj.borderStyle.height;
                        }
                        if (textArr <= 1) {
                            parentHeight = funs.subsection(obj.text, obj.font_size, obj.borderStyle.width - lineWidth).slice(0, 1);
                        } else if (maxHeight > funs.decimalMul(textArr, obj.font_size)) {
                            parentHeight = funs.subsection(obj.text, obj.font_size, obj.borderStyle.width - lineWidth).slice(0, textArr);
                        } else {
                            parentHeight = getText_arr(textArr - 1)
                        }
                        /*for (var i = 0; i < parentHeight.length; i++) {
                            if (!!obj.paragraph && i !== 0) {
                                paragraph += Number(obj.paragraph / 72 * 96);
                                paragraph = String(paragraph)
                                if(paragraph.indexOf('.') > -1) {
                                    paragraph = paragraph.substr(0, paragraph.indexOf('.'));
                                }
                                paragraph = Number(paragraph);
                            console.log(paragraph)
                                if( maxHeight < funs.decimalMul(textArr, obj.font_size) + paragraph) {
                                    parentHeight = parentHeight.splice(0,i-1)
                                    break;
                                }

                            }
                        }*/

                        return parentHeight;
                    }
                    var text_arr = getText_arr(tmpText_arr);
                } else {
                    // var text_arr = [obj.text];
                    var text_arr = funs.subsection(obj.text, obj.font_size, obj.borderStyle.width).slice(0, 1);
                }
                // var text_arr = funs.subsection(obj.text, obj.font_size, obj.borderStyle.width);
                // if (text_arr.length > 0)
                //     obj.borderStyle.height += (text_arr.length - 1) * obj.font_size;
            }
            //绘制背景
            draw.Rect(obj.x, obj.y, obj.borderStyle);
            //绘制边框
            draw.Border(obj.x, obj.y, obj.borderStyle);

            // 绘制文本 坐标是左下角
            var font_size = obj.font_size;
            // __this.ctx.font = font + obj.font_size + 'px 宋体';
            // __this.ctx.font = font + obj.font_size + 'px ' + (!obj['font_family'] ? objDefaultOptions.font_family : obj.font_family);
            __this.ctx.fillStyle = obj.color;
            if (__opt['mode'] !== 'edit') {
                for (var i = 0; i < text_arr.length; i++) {
                    //段落
                    /* var paragraph = 0;
                     obj.font_size = font_size * text_arr.length;
                     obj.textWidth = funs.getTextWidth(text_arr[i], font_size);
                     if(!!obj.paragraph && i !== 0){
                         paragraph = String(obj.paragraph / 72 * 96);
                         paragraph = Number(paragraph.substr(0,paragraph.indexOf('.'))) * i;
                          __this.ctx.fillText(text_arr[i], align(obj), (valign(obj) + font_size * (i + 1)));
                     }*/
                    obj.font_size = font_size * text_arr.length;
                    obj.textWidth = funs.getTextWidth(text_arr[i], font_size);
                    __this.ctx.fillText(text_arr[i], align(obj), valign(obj) + font_size * (i + 1));

                }
            } else {
                var text = obj.text;
                var textWidth = funs.getTextWidth(text, obj.font_size);
                if (textWidth > obj.borderStyle.width) {
                    text = funs.substr(text, obj.font_size, obj.borderStyle.width - funs.getTextWidth('..', obj.font_size)) + '..';
                }
                obj.textWidth = textWidth;
                __this.ctx.fillText(text, align(obj), valign(obj) + obj.font_size);
            }
            if (obj.simple || __opt['mode'] !== 'edit') {
                return;
            }
            var x = obj.x;
            var y = obj.y;
            var borderWidth = obj.borderStyle.width;
            var borderheight = obj.borderStyle.height;
            var size = 2;
            //绘制文本框 四个角
            draw.Line({ x: x, y: y }, { x: x + size, y: y });
            draw.Line({ x: x + borderWidth - size, y: y }, { x: x + borderWidth, y: y });
            draw.Line({ x: x + borderWidth, y: y }, { x: x + borderWidth, y: y + size });
            draw.Line({ x: x + borderWidth, y: y + borderheight - size }, { x: x + borderWidth, y: y + borderheight });
            draw.Line({ x: x + borderWidth, y: y + borderheight }, { x: x + borderWidth - size, y: y + borderheight });
            draw.Line({ x: x + size, y: y + borderheight }, { x: x, y: y + borderheight });
            draw.Line({ x: x, y: y + borderheight }, { x: x, y: y + borderheight - size });
            draw.Line({ x: x, y: y + size }, { x: x, y: y });
            //水平对齐方式
            function align(obj) {
                switch (obj.align) {
                    case 'left':
                        return obj.x;
                    case 'centen':
                        if (obj.borderStyle.width - obj.textWidth > 0) {
                            return obj.x + (obj.borderStyle.width - obj.textWidth) / 2;
                        } else {
                            return obj.x
                        }
                    case 'right':
                        return obj.x + (obj.borderStyle.width - obj.textWidth);
                    default:
                        return obj.x;
                }
            }

            function valign(obj) {
                switch (obj.valign) {
                    case 'top':
                        return obj.y - 3;
                    case 'centen':
                        return obj.y + (obj.borderStyle.height - obj.font_size) / 2 - 3;
                    case 'bottom':
                        return obj.y + (obj.borderStyle.height - obj.font_size) - 3;
                    default:
                        return obj.y - 3;
                }
            }
        },
        //绘制对齐线
        Snaplines: function (displayable) {
            _VariableStorage.Snaplines.length = 0;
            _VariableStorage.snaplines_x = false;
            _VariableStorage.snaplines_y = false;
            var list = __this.DisplayList;
            // var tmpx_1 = [displayable.x, displayable.x + displayable.borderStyle.width];
            // var tmpy_1 = [displayable.y, displayable.y + displayable.borderStyle.height];
            var tmpx_1 = [displayable.x, funs.decimalAdd(displayable.x, displayable.borderStyle.width)];
            var tmpy_1 = [displayable.y, funs.decimalAdd(displayable.y, displayable.borderStyle.height)];
            for (var i = 5; i < list.length; i++) {
                if (displayable.dataIndex == list[i].dataIndex || /scale_[1-8]/.test(list[i].id)) continue;

                // var tmpx_2 = [list[i].x, list[i].x + list[i].borderStyle.width];
                var tmpx_2 = [list[i].x, funs.decimalAdd(list[i].x, list[i].borderStyle.width)];
                tmpx_1.forEach(function (value1) {
                    tmpx_2.forEach(function (value2) {
                        if (value1 == value2) {
                            _VariableStorage.snaplines_x = true;
                            addDrawList(value1, displayable.y, value2, list[i].y);
                        }
                    });
                });

                // var tmpy_2 = [list[i].y, list[i].y + list[i].borderStyle.height];
                var tmpy_2 = [list[i].y, funs.decimalAdd(list[i].y, list[i].borderStyle.height)];
                tmpy_1.forEach(function (value1) {
                    tmpy_2.forEach(function (value2) {
                        if (value1 == value2) {
                            _VariableStorage.snaplines_y = true;
                            addDrawList(displayable.x, value1, list[i].x, value2);
                        }
                    });
                });
            }

            function addDrawList(x1, y1, x2, y2) {
                _VariableStorage.Snaplines.push({
                    type: 'line',
                    style: {
                        color: '#109cea',
                        lineWidth: 1
                    },
                    origin: {
                        x: x1,
                        y: y1,
                    },
                    end: {
                        x: x2,
                        y: y2
                    }
                });
            }
            draw.drawAll();
        },
        // 绘制-图片
        Image: function (obj, callback) {
            // if (!obj.borderStyle.img && __opt['mode'] === 'edit') {
            if (!obj.borderStyle.img) {
                var img = new Image();
                img.src = obj.dataUrl;
                img.onload = function () {
                    obj.borderStyle.img = img;
                    __this.ctx.drawImage(img, obj.x, obj.y, obj.borderStyle.width, obj.borderStyle.height);
                    if (typeof callback === 'function') callback();
                };
            } else {
                __this.ctx.drawImage(obj.borderStyle.img, obj.x, obj.y, obj.borderStyle.width, obj.borderStyle.height);
            }

            if (__opt['mode'] === 'edit')
                draw.Border(obj.x, obj.y, obj.borderStyle);
        },
        // 绘制-二维码'PT380/UB144BL'
        QRBar: function (obj, callback) {
            // __this.ctx.drawImage(obj.borderStyle.img, obj.x, obj.y, obj.borderStyle.width, obj.borderStyle.height);

            var div = document.createElement('div');
            var qrcode = new QRCode(div, {
                text: obj.text,
                width: obj.borderStyle.width,
                height: obj.borderStyle.height,
                correctLevel: QRCode.CorrectLevel.L
            });
            var img = new Image();
            img.src = div.children[0].toDataURL();
            img.onload = function () {
                __this.ctx.drawImage(img, obj.x, obj.y, obj.borderStyle.width, obj.borderStyle.height);
                if (typeof callback === 'function') callback();
            }
        },
        // 绘制条形码
        BarCode: function (obj, callback) {
            if (typeof obj.text === 'undefined' || obj.text == '') return typeof callback === 'function' ? callback() : '';
            var img = new Image();
            JsBarcode(img)
                .CODE128(obj.text, {
                    fontSize: obj.font_size,
                    height: obj.borderStyle.height,
                    textMargin: 0,
                    margin: 0
                }).render();
            img.onload = function () {
                if (obj.barcodeDirection) {
                    __this.ctx.rotate(1 * 90 * Math.PI / 180);
                    var y = obj.rect[2].y - obj.rect[0].y;
                    var x = obj.rect[1].x - obj.rect[0].x;
                    var imgX = parseFloat((obj.rect[0].y + y / 2) - x / 2).toFixed(0)
                    __this.ctx.drawImage(img, imgX, -obj.rect[1].x + y, obj.borderStyle.width, obj.borderStyle.height);
                } else {
                    __this.ctx.drawImage(img, obj.x, obj.y, obj.borderStyle.width, obj.borderStyle.height);
                }
                if (typeof callback === 'function') callback();
            }
        },
        // 绘制小程序码
        miniCode: function (obj, callback) {
            if (typeof obj.text === 'undefined' || obj.text === '') {
                if (typeof callback === 'function') callback();
                return '';
            }
            if (typeof __this.opt.onMiniCodeLoad === 'function') {
                __this.opt.onMiniCodeLoad(obj, function (src) { // 利用回调函数，生成小程序码
                    var img = new Image();
                    img.crossOrigin = "Anonymous";
                    img.onload = function () {
                        __this.ctx.drawImage(img, obj.x, obj.y, obj.borderStyle.width, obj.borderStyle.height);
                        if (typeof callback === 'function') callback();
                    };
                    img.onerror = function () {
                        if (typeof callback === 'function') callback();
                    };
                    img.src = src;
                })
            } else {
                if (typeof callback === 'function') callback();
                return '';
            }
        }
    };

    // 解析打印数据方法
    var PaintParse = {
        /**
         * @param param
         * @param param.str {String} 正则匹配到的字符串
         * @param param.m  {String} 正则匹配到起始位置的字符串
         * @param param.pageIndex {Integer} 数据源对应的下标
         * @param param.execute {function(*)} 执行自定义方法
         * @return {string|(function(*): number)|m|string|void|*}
         */
        textFnExecutor: function (param) {
            // 检查
            if (typeof param.pageIndex === 'undefined') return param.m;

            // 数据源，默认主数据集
            var data = __this.opt.data['主数据集']; // 执行 getText() 前已经设置
            //  每一页，数据源的开始和结束
            var start = 0, end = 0;
            var pagination = __opt['data']['分页'] || [0];
            if (param.pageIndex !== 0) start = Number(pagination[param.pageIndex - 1]);
            end = Number(pagination[param.pageIndex]);

            // 数据源信息设置
            if (param.str.indexOf('单据.') > -1) {
                data = __this.opt.data['单据'];
                // 不需要根据分页处理，只有一页
                start = 0; end = 1;
            }

            // 不是iif函数，使用原来的判断逻辑
            if (param.m.indexOf('iif') !== 0) {
                if (!/\<(.*?)\>/.test(param.str)) return param.m;
            }
            // 执行具体逻辑
            return param.execute(start, end, data, param.str);
        },

        /**
         * 解析数据源
         * @param param.text {String} 解析的表达式 - 格式：公司.名称 (TODO 森斌明确后补充)
         * @param param.data_arr {Array} 数据源
         * @param param.index {Integer} 数据源对应的下标
         * @param param.defaultValue {*} 默认返回数据，默认为空字符串
         * @param param.parseFun {Function} 自定义解析方法
         * @param param.start {Integer} 渲染数据源开始下标
         * @param param.end {Integer} 渲染数据源结束下标
         * @param param.returnValFun {*} return 出去的值
         * @return {*}
         */
        parseDataMain: function (param) {
            return String(param.text).replace(/\<(.*?)\>/g, function (a, b) {
                var res = typeof param.defaultValue === 'undefined' ? '' : param.defaultValue;  // 赋值默认数据
                var tmp = b.split('.');
                var is_masterdataRow = false;
                var data_item;
                if (Array.isArray(param.data_arr)) {      // 是数组
                    if (typeof param.index !== 'undefined') data_item = param.data_arr[param.index];  // 指定下标
                    else {
                        // 没有下标说明不是在页中，多条计算
                        is_masterdataRow = true;
                    }
                } else if (typeof param.data_arr === 'object') {  // 是对象，直接赋值
                    data_item = param.data_arr;
                }
                if (is_masterdataRow) {
                    // 没有下标说明不是在页中，多条计算
                    for (var i = param.start; i < param.end; i++) {
                        if (typeof param.parseFun === 'function') { // 自定义解析的方法
                            res = param.parseFun(res, param.data_arr[i], param.data_arr[i][tmp[1]]);
                        }
                    }
                } else {
                    if (typeof param.parseFun === 'function') { // 自定义解析的方法
                        res = param.parseFun(res, data_item, data_item[tmp[1]]);
                    }
                }

                //输出的值
                if (typeof param.returnValFun == 'function') {
                    return param.returnValFun(res);
                } else {
                    return res;
                }
            });
        },

        // 解析打印数据结果，记录是否需要再进行特殊处理
        parseResult: {
            operator: {}
        }
    };

    // 初始化参数, 并开始执行
    var init = function () {
        __opt.margin = {
            top: __opt.pageConfig.margin.top * __opt.cmToDots,
            bottom: __opt.pageConfig.margin.bottom * __opt.cmToDots,
            left: __opt.pageConfig.margin.left * __opt.cmToDots,
            right: __opt.pageConfig.margin.right * __opt.cmToDots
        };
        // 为了做向下兼容，比如老版本没有textWidth属性, 新版本有，通过此方法补全元素属性,
        for (var i = 0; i < __opt.pageStyle.length; i++) {
            var obj = __opt.pageStyle[i];
            if (obj.isParent) {
                var newChilds = [];
                for (var j = 0; j < obj.childs.length; j++) {
                    var _sub = funs.findObj('id', obj.childs[j], __opt.pageStyle);
                    if (_sub) {
                        _sub.parentID = obj.id;
                        newChilds.push(obj.childs[j]);
                    }
                }
                obj.childs = newChilds.concat();
            }
            if (obj.type === 'text') obj.textWidth = funs.getTextWidth(obj.text, obj.font_size);
            if (!obj.height && obj.width) delete obj.width;
            funs.setObjWidthHeight(obj, obj);
            funs.setObjPlace(obj, obj, __opt.pageStyle);
            obj.dataIndex = i + 13;
        }

        //设置画布宽度高度
        if (__opt['mode'] === 'edit') {
            funs.setCanvasWidth(__opt.pageConfig.width);
            funs.setCanvasHeight(__opt.pageConfig.height);
            funs.setMargin();
            funs.drawScalePoint(true);
            if (JSON.stringify(__opt.pageStyle) == "{}") {
                __this.DisplayList = __this.DisplayList.concat();
            } else {
                __this.DisplayList = __this.DisplayList.concat(__opt.pageStyle);
            }
            draw.drawAll();
            if (typeof __this.ready === 'function') __this.ready();
            __this.saveStatus();
        } else {
            var printData = []; // 打印的每张报表数据
            if (typeof __opt['data']['尺码组'] !== 'undefined') {
                __opt['data']['尺码组'].forEach(function (value, index) {
                    var obj = funs.findObj('id', 'size_title_' + (index + 1), __opt.pageStyle);
                    if (obj) {
                        obj.text = value['码数'];
                    }
                    obj = funs.findObj('id', 'size_value_' + (index + 1), __opt.pageStyle);
                    if (obj) {
                        obj.text = '[记录2.' + value['码数'] + ']';
                    }
                });
            }
            funs.setCanvasWidth(__opt.pageConfig.width);
            funs.setCanvasHeight(__opt.pageConfig.height);

            var head = funs.findObj('id', 'head', __opt.pageStyle) || { borderStyle: { height: 0 } };
            var footer = funs.findObj('id', 'footer', __opt.pageStyle) || { borderStyle: { height: 0 } };
            var data_header = funs.findObj('id', 'data_header', __opt.pageStyle) || { borderStyle: { height: 0 } };
            var data_footer = funs.findObj('id', 'data_footer', __opt.pageStyle) || { borderStyle: { height: 0 } };
            var data_main = funs.findObj('id', 'data_main', __opt.pageStyle) || { borderStyle: { height: 0 } };

            var dataMaxHeight = __opt.margin.top + __opt.margin.bottom;

            if (head.id) {
                head.borderStyle.height -= 20;
                dataMaxHeight += head.borderStyle.height
            }

            if (footer.id) {
                footer.borderStyle.height -= 20;
                dataMaxHeight += footer.borderStyle.height
            }

            if (data_header.id) {
                data_header.borderStyle.height -= 20;
                dataMaxHeight += data_header.borderStyle.height
            }

            if (data_footer.id) {
                data_footer.borderStyle.height -= 20;
                dataMaxHeight += data_footer.borderStyle.height
            }

            // 计算出能打印纸张的最大高度, 为了实现分页效果
            dataMaxHeight = __this.canvas.height - dataMaxHeight;

            var dataMainTop = __opt.margin.top + head.borderStyle.height + data_header.borderStyle.height;
            var top = dataMainTop;

            var pageData = [];  // 打印每一页数据，通过计算打印的总高度和每一页的高度得出
            var _dataMaxHeight = dataMaxHeight;
            var rowWidth = {};              // 最大列宽
            var changeTextWith = false;     // 是否更改过列宽
            var printLines = 0;             // 每页打印的主数据行数

            // __opt['data'][data_main.dataSet]  主数据集
            // data_main.childs  每行主数据绘制的单元格内容

            if (data_main.childs && __opt['data'][data_main.dataSet]) {
                // if(!Array.isArray(__opt['data'][data_main.dataSet])) __opt['data'][data_main.dataSet] = [__opt['data'][data_main.dataSet]];
                __opt['data']['主数据集'] = __opt['data'][data_main.dataSet];
                // 默认打印一页
                var data_main_len = Array.isArray(__opt['data'][data_main.dataSet]) ? __opt['data'][data_main.dataSet].length : 1;
                // 渲染主数据明细行，计算数据集里每条数据对应在主数据中元素的位置
                for (var i = 0; i < data_main_len; i++) {
                    var rowHeight = 0;
                    var childs = [];
                    var row_max_height = 0;
                    var row_min_height = 0;
                    // 渲染主数据集
                    data_main.childs.forEach(function (value, index, data) {
                        var obj = funs.extend(true, {}, funs.findObj('id', value, __opt.pageStyle), { data_main_auto_height: !!data_main.autoHeight });
                        if (!obj) return;
                        // 如果主数据集有多条，数据来源，打印明细情况
                        if (Array.isArray(__opt['data'][data_main.dataSet])) {
                            obj.text = funs.getText(obj, obj.text, i);
                        } else {
                            obj.text = funs.getText(obj, obj.text);
                        }
                        // obj.y = top;
                        var textHeight = funs.getTextHeight(obj);
                        //当文本开启自动高度并且文本高度大于行高，则增加行高
                        if (rowHeight < textHeight) {
                            if (obj.autoHeight) {
                                rowHeight = textHeight;
                            } else {
                                rowHeight = obj.borderStyle.height;
                            }
                        }
                        //当文本未开启自动高度并且文本宽度超出行宽，则增加该列行宽
                        if (!rowWidth[obj.id]) rowWidth[obj.id] = 0;
                        var textWidth = funs.getTextWidth(obj.text, obj.font_size);
                        if (!!obj.autoWidth && !obj.autoHeight && textWidth > obj.borderStyle.width) {
                            obj.borderStyle.width = textWidth;
                        }
                        // if(textWidth > rowWidth[obj.id]){
                        //     if(!obj.autoHeight && textWidth > obj.borderStyle.width){
                        //         rowWidth[obj.id] = textWidth;
                        //         changeTextWith = true;
                        //     }else{
                        //         rowWidth[obj.id] = obj.borderStyle.width;
                        //     }
                        // }

                        //计算主数据中的最高和最低位置
                        var obj_height;
                        if (!!data_main.autoHeight && !!obj.autoHeight && obj.borderStyle.height < textHeight) {
                            obj_height = textHeight;
                        } else {
                            obj_height = obj.borderStyle.height;
                        }
                        var obj_top = funs.decimalMul(obj.top < 0 ? 0 : obj.top, __opt.cmToDots);
                        var obj_bottom = funs.decimalAdd(obj_top, obj_height);
                        if (row_max_height == 0 || obj_bottom > row_max_height) {
                            row_max_height = obj_bottom;
                        }
                        if (row_max_height == 0 || obj_top < row_min_height) {
                            row_min_height = obj_top;
                        }

                        obj.y = funs.decimalAdd(top, obj_top);
                        //页中数据的下标
                        obj['data_main_index'] = i;
                        childs.push(obj);
                    });
                    //设置（新）高
                    if (!!data_main.autoHeight) {
                        childs.forEach(function (value, index, data) {
                            value.borderStyle.height = rowHeight;
                        });
                    }

                    //设置记录的高度为主数据元素最大差值
                    rowHeight = funs.decimalSub(row_max_height, row_min_height);
                    if (_dataMaxHeight <= rowHeight) { // 超出打印高度
                        if (!__opt.pageConfig.unLimitPageHeight) {    // 没有开启无限页高
                            if (pageData.length === 0 && typeof __opt.onerrorRender === 'function') {
                                __opt.onerrorRender({
                                    message: '报表设计不合法，纸张高度小于主数据集高度，请修改主数据集高度或者开启【预览打印无限页高】'
                                });
                                throw new Error('报表设计不合法，纸张高度小于主数据集高度，请修改纸张高度或者开启【预览打印无限页高】');
                            }
                            pushData(pageData, i, true);
                            top = dataMainTop;
                            printLines = 0;         // 分下一页 清空行记录

                            childs.forEach(function (value, index, data) {
                                // value.y = top
                                value.y = funs.decimalAdd(top, funs.decimalMul(value.top < 0 ? 0 : value.top, __opt.cmToDots));
                            });
                            pageData = [];
                            _dataMaxHeight = dataMaxHeight;
                        } else {
                            __this.canvas.height += rowHeight;
                        }
                    }

                    top = top + rowHeight;
                    // _dataMaxHeight -= rowHeight;
                    _dataMaxHeight = funs.decimalSub(_dataMaxHeight, rowHeight);
                    pageData = pageData.concat(childs);
                    printLines++;       // 打印记录横行数加1
                }

                // 计算填充的空白行
                // 补充空白行,以最后一条记录的高度补充
                var result = funs.calcFillBlankLine({
                    data_main: data_main,
                    dataMaxHeight: _dataMaxHeight,
                    rowHeight: rowHeight,
                    pageData: pageData,
                    top: top,
                    printLines: printLines
                });

                pageData = result.pageData;
                top = result.top;

                /**
                 * 每个图片放入渲染队列中
                 * @param pageData  每页的数据
                 * @param dataIndex 数据的长度
                 */
                function pushData(pageData, dataIndex) {
                    if (data_footer.childs) {
                        var dataFooterTop = top;
                        data_footer.childs.forEach(function (value, index, data) {
                            var obj = funs.extend(true, {}, funs.findObj('id', value, __opt.pageStyle));
                            if (!obj) return;
                            obj.text = funs.getText(obj, obj.text);
                            obj.y = dataFooterTop + obj.top * __opt.cmToDots;
                            pageData.push(obj);
                        });
                    }
                    printData.push(pageData);
                    if (!Array.isArray(__opt['data']['分页'])) __opt['data']['分页'] = [];
                    __opt['data']['分页'].push(dataIndex)
                }

                pageData.length && pushData(pageData, data_main_len);
            }

            // if(changeTextWith){
            //     //设置 新宽
            //     printData.forEach(function(value,index){
            //         value.forEach(function(value,index){
            //             if(value.id && rowWidth[value.id]){
            //                 value.borderStyle.width = rowWidth[value.id];
            //             }
            //         })
            //     })
            //     //增加列宽后，重新计算元素位置防止重叠
            //     printData.forEach(function(arr,index){
            //         arr.sort(function(value1,value2){
            //             return value1.x - value2.x;
            //         })
            //         arr.forEach(function(value,index, data){
            //             arr.forEach(function(child_value,child_index,child_data){
            //                 if(child_value.id === value.id || child_index < index) return;
            //                 if(child_value.y < value.y || child_value.y + child_value.borderStyle.height > value.y + value.borderStyle.height) return;
            //                 if(child_value.x < value.x + value.borderStyle.width){
            //                     child_value.x += value.x + value.borderStyle.width - child_value.x
            //                 }
            //             })
            //         })
            //     })
            // }

            if (__opt.isShowEmptyAlert && !__opt['data'][data_main.dataSet]) {
                __opt.onerrorRender({ message: "请设置主数据集" });
                throw new Error("请设置主数据集");
            }
            if (!printData.length) printData.push([]);

            // for (var i = 0; i < printData.length; i++) {
            asyncLoop(printData.length, function (loop) {
                var i = loop.iteration();
                __opt['data']['变量'] = __opt['data']['变量'] || {};
                __opt['data']['变量']['当前页'] = (i + 1);
                __opt['data']['变量']['总页数'] = (printData.length);

                __this.DisplayList = [];
                funs.setCanvasWidth(__opt.pageConfig.width);
                funs.setCanvasHeight(__opt.pageConfig.height);
                if (!!__opt.pageConfig.unLimitPageHeight) __this.canvas.height += Math.abs(_dataMaxHeight);

                if (head.childs) {
                    var headerTop = __opt.margin.top;
                    head.childs.forEach(function (value, index, data) {
                        var obj = funs.extend(true, {}, funs.findObj('id', value, __opt.pageStyle));
                        if (!obj) return;
                        obj.text = funs.getText(obj, obj.text, void 0, i);
                        obj.y = headerTop + obj.top * __opt.cmToDots;
                        __this.DisplayList.push(obj);
                    });
                }

                if (data_header.childs) {
                    var dataHeaderTop = __opt.margin.top + head.borderStyle.height;
                    data_header.childs.forEach(function (value, index, data) {
                        var obj = funs.extend(true, {}, funs.findObj('id', value, __opt.pageStyle));
                        if (!obj) return;
                        obj.text = funs.getText(obj, obj.text, void 0, i);
                        obj.y = dataHeaderTop + obj.top * __opt.cmToDots;
                        __this.DisplayList.push(obj);
                    });
                }

                if (footer.childs) {
                    footer.childs.forEach(function (value, index, data) {
                        var obj = funs.extend(true, {}, funs.findObj('id', value, __opt.pageStyle));
                        if (!obj) return;
                        obj.text = funs.getText(obj, obj.text, void 0, i);
                        if (!!__opt.pageConfig.unLimitPageHeight) obj.y += Math.abs(_dataMaxHeight);
                        __this.DisplayList.push(obj);
                    });
                }


                printData[i].forEach(function (value, index, data) {
                    value.text = funs.getText(value, value.text, value['data_main_index'], i);
                    __this.DisplayList.push(value);
                });

                // 加入不属于任何栏内的元素
                for (var j = 0; j < __opt.pageStyle.length; j++) {
                    if (!__opt.pageStyle[j].isParent && !__opt.pageStyle[j].parentID) {
                        var otherObj = __opt.pageStyle[j];
                        var tmp = funs.extend(true, {}, otherObj, { text: funs.getText(otherObj, otherObj['text'], void 0, i) });
                        __this.DisplayList.push(tmp);
                    }
                }

                //处理除了主数据外其他文本元素的自动宽高
                __this.DisplayList.sort(function (value1, value2) {
                    return value1.x - value2.x;
                })
                __this.DisplayList.forEach(function (value, index) {
                    if (value.parentID !== "data_main" && value.type === 'text') {
                        if (value.autoHeight) {
                            funs.autoRowHeight(value);
                        }
                        if (value.autoWidth) {
                            funs.autoRowWidth(value);
                        }
                        if (value.maxHeight) {
                            funs.maxRowHeight(value);
                        }
                        // else{
                        //     funs.overlapMove(value);
                        // }
                    }
                    //比例缩放
                    value.type == 'icon' && funs.imgScale(value);

                    //比例
                    (value.type == 'barcode' || value.type == 'miniCode') && barCode_Scale(value);
                    function barCode_Scale(obj) {
                        if (typeof obj.scaling === 'undefined') obj.scaling = 1;
                        obj.borderStyle.width = parseFloat(parseFloat(obj.borderStyle.width * obj.scaling).toFixed(0));
                        obj.borderStyle.height = parseFloat(parseFloat(obj.borderStyle.height * obj.scaling).toFixed(0));
                    }
                })

                // print  打印模式 、 preview 预览模式
                if (__opt['mode'] === 'print' || __opt['mode'] === 'preview') {
                    if (__opt['mode'] === 'print') {
                        //设置图片大小和纸张方向
                        var width = parseInt(Math.ceil(__opt.pageConfig.dpi / 25.4) * __opt.pageConfig.width * 10);
                        var height;
                        if (!__opt.pageConfig.unLimitPageHeight) {
                            height = parseInt(Math.ceil(__opt.pageConfig.dpi / 25.4) * __opt.pageConfig.height * 10);
                        } else {
                            height = parseInt(Math.ceil(__opt.pageConfig.dpi / 25.4) * (__opt.pageConfig.height * 1 + Math.abs(_dataMaxHeight) / __opt.cmToDots) * 10);
                        }
                        if (width % 8 != 0) {
                            width = width + 8 - width % 8;
                        }
                        if (height % 24 != 0) {
                            height = height + 24 - height % 24;
                        }

                        var w_rate = width / __this.canvas.width;
                        var h_rate = height / __this.canvas.height;
                        if (!!__opt.pageConfig.unLimitPageHeight) {
                            h_rate = w_rate;
                        }

                        // 数据转换
                        var Tpl_Data = __this.DisplayList;
                        for (var k = 0; k < Tpl_Data.length; k++) {
                            var tmp = funs.extend(true, {}, Tpl_Data[k]);
                            Tpl_Data[k].x = parseInt(Tpl_Data[k].x * w_rate);
                            Tpl_Data[k].y = parseInt(Tpl_Data[k].y * h_rate);

                            if (Tpl_Data[k].font_size && Tpl_Data[k].type == 'text') {
                                Tpl_Data[k].font_size = parseInt(Tpl_Data[k].font_size * w_rate);
                            }
                            if (Tpl_Data[k].borderStyle.width) {
                                Tpl_Data[k].borderStyle.width = parseInt(Tpl_Data[k].borderStyle.width * w_rate);
                            }
                            if (Tpl_Data[k].borderStyle.height) {
                                Tpl_Data[k].borderStyle.height = parseInt(Tpl_Data[k].borderStyle.height * h_rate);
                            }
                        }

                        __this.canvas.width = width;
                        __this.canvas.height = height;
                    }

                    var noList = draw.drawAll(function () {
                        var img = new Image();
                        img.src = __this.canvas.toDataURL();
                        img.onload = function () {
                            var canvas = document.createElement('canvas');
                            // canvas.setAttribute('font-family', 'mySongTi !important');
                            var ctx = canvas.getContext('2d');
                            if (isNaN(__opt.pageConfig.printMode)) __opt.pageConfig.printMode = 0;
                            var n = __opt.pageConfig.printMode * 1;
                            var degree = (n * 90 * Math.PI / 180);

                            switch (n) {
                                case 0:
                                    canvas.setAttribute('width', __opt.printConfig.width || this.width);
                                    canvas.setAttribute('height', __opt.printConfig.height || this.height);
                                    ctx.rotate(degree);
                                    ctx.drawImage(this, 0, 0);
                                    break;
                                case 1:
                                    var width = this.height;
                                    var height = this.width;

                                    // 打印模式
                                    if (__opt['mode'] === 'print') {
                                        if (width % 8 != 0) {
                                            width = width + 8 - width % 8;
                                        }
                                        if (height % 24 != 0) {
                                            height = height + 24 - height % 24;
                                        }
                                    }

                                    canvas.setAttribute('width', __opt.printConfig.width || width);
                                    canvas.setAttribute('height', __opt.printConfig.height || height);
                                    ctx.fillStyle = "#FFF";
                                    ctx.fillRect(0, 0, canvas.width, canvas.height);
                                    ctx.rotate(degree);
                                    ctx.drawImage(this, 0, -this.height);
                                    break;
                                case 2:
                                    canvas.setAttribute('width', this.width);
                                    canvas.setAttribute('height', this.height);
                                    ctx.rotate(degree);
                                    ctx.drawImage(this, -this.width, -this.height);
                                    break;
                                case 3:
                                    canvas.setAttribute('width', this.height);
                                    canvas.setAttribute('height', this.width);
                                    ctx.rotate(degree);
                                    ctx.drawImage(this, -this.width, 0);
                                    break;
                            }
                            __this.pageImg.push(canvas.toDataURL());
                            var imageData = ctx.getImageData(0, 0, canvas.width, canvas.height);
                            __this.canvasImageData.push(imageData);


                            var binaryData = funs.OTSUAlgorithm(imageData);
                            var printData = funs.imageDataToBlueToothPrint(binaryData);
                            __this.blueToothPrintData.push(printData);
                            loop.next();
                        };
                    });

                    if (!noList) {
                        loop.next();
                    }
                } else {
                    draw.drawAll(function () {
                        __this.pageImg.push(__this.canvas.toDataURL());
                        loop.next();
                    });
                }
            },
                function () {
                    __this.ready()
                })
            // }

            // 开启计时器，等待所有图片生成完毕 执行 ready 方法
            // var timer = setInterval(function () {
            //     if (__this.pageImg.length >= printData.length) {
            //         clearInterval(timer);
            //         if (typeof __this.ready === 'function') __this.ready();
            //     }
            // }, 50);

            /**
             * 迭代器
             * @param iterations 长度
             * @param func 执行方法
             * @param callback
             * @return {{next: next, break: break, iteration: (function(): number)}}
             */
            function asyncLoop(iterations, func, callback) {
                var index = 0;
                var done = false;
                var loop = {
                    next: function () {
                        if (done) {
                            return;
                        }

                        if (index < iterations) {
                            index++;
                            func(loop);

                        } else {
                            done = true;
                            callback();
                        }
                    },

                    iteration: function () {
                        return index - 1;
                    },

                    break: function () {
                        done = true;
                        callback();
                    }
                };
                loop.next();
                return loop;
            }
        }
    };

    var __this = this;
    __this.version = VERSION;
    __this.dateFormat = {
        date: 'yyyy-MM-dd',
        time: 'hh:mm:ss'
    };
    __this.pageImg = [];
    __this.canvasImageData = [];
    __this.blueToothPrintData = [];
    var __opt = funs.extend(true, {}, {
        scale_size: 5,
        mode: 'edit',  //模式,edit/print/preview
        cmToDots: 38, //一厘米多少个像素
        pageConfig: {
            margin: { top: 0.3, left: 0.3, right: 0.3, bottom: 0.3 }, // 页面边距,单位cm
            width: 21,  //页宽,单位cm
            height: 14,  //页高,单位cm
            dpi: 180,
            drawBlank: true,   // 打印空白行
            hideZero: false,    // 是否隐藏值：0
        },
        printConfig: { // 打印设置，目前只有 打印区域大小设置
            width: 0,
            height: 0
        },
        pageStyle: [], //控件样式
        data: {  //数据
            '变量': {
                '日期': funs.getDate(),
                '时间': funs.getTime(),
                '当前页': 0,
                '总页数': 0
            }
        },
        onMiniCodeLoad: null, // 加载小程序码方法, function(obj, callback)
        onClick: null,  //画布点击事件返回,点击元素属性 function(obj)
        onMoveAfter: null, //元素移动完毕, functions(obj)
        onPageClick: null, // canvas 打印区点击
        onSelectMoveAfter: null, //多选完毕, functions()
        onerrorRender: null,    // 渲染打印报表时，出错处理
        isShowEmptyAlert: false, // 是否数据集为空时弹窗
        // onSelectMultipleObj: null, // 选中多个元素回调 function()
    }, options);

    // __this.__opt = __opt;
    //方法需要保存变量值时的存放对象
    var _VariableStorage = {};
    //模块列表
    var module_list = [];
    //历史纪录
    var History = { data: [], index: 0 };
    //焦点对象
    var storage = _VariableStorage.dragHandlers = {};
    //存放所有对象
    __this.DisplayList = [];
    //画布对象
    __this.canvas = __opt['canvas'] || document.createElement('canvas');
    // __this.canvas.setAttribute('font-family', 'mySongTi !important');
    //画布上下文
    __this.ctx = __this.canvas.getContext('2d');
    //对齐线
    _VariableStorage.Snaplines = [];
    //选中的对象
    __this.dragInfo = {
        copyList: [],       // 复制的对象
        /**
         * 检查元素是否在选中的列表的中
         * @param obj 元素对象
         * @returns {boolean} true 在 | false 不存在
         */
        isInObjList: function (obj) {
            for (var i = 0; i < this.objList.length; i++) {
                if (this.objList[i].id === obj.id) {
                    return true;
                }
            }
            return false;
        },
        /**
         * 设置选中元素的属性，委托方法
         * @param opt 属性
         */
        setSelectedOpt: function (opt, controlParam) {
            var layout = ['auto_left', 'auto_right', 'auto_middle'];
            this.eachObj(0, function (obj) {
                if (typeof controlParam === 'object' && controlParam.type == 'Manycontrols') {
                    var tmp_width = obj.rect[1].x - obj.rect[0].x;
                    var tmp_height = obj.rect[2].y - obj.rect[0].y;
                    var tmp_opt = {};
                    if (layout.indexOf(obj.layout) < 0) {
                        switch (controlParam.align) {
                            case 'left'://左对齐
                                funs.setObjOpt(obj.dataIndex, opt, false, false);
                                break;
                            case 'right'://右对齐
                                tmp_opt.x = opt.x + (controlParam.maxWidth - tmp_width);
                                funs.setObjOpt(obj.dataIndex, tmp_opt, false, false);
                                break;
                            case 'center'://居中对齐
                                tmp_opt.x = parseFloat(parseFloat(controlParam.center - (obj.rect[1].x - obj.rect[0].x) / 2).toFixed(0));
                                funs.setObjOpt(obj.dataIndex, tmp_opt, false, false);
                                break;
                        }
                    }
                    switch (controlParam.align) {
                        case 'top'://顶部对齐
                            funs.setObjOpt(obj.dataIndex, opt, false, false);
                            break;
                        case 'middle'://垂直对齐
                            tmp_opt.y = parseFloat(parseFloat(controlParam.center - (obj.rect[2].y - obj.rect[0].y) / 2).toFixed(0));
                            funs.setObjOpt(obj.dataIndex, tmp_opt, false, false);
                            break;
                        case 'bottom'://底部对齐
                            tmp_opt.y = opt.y + (controlParam.maxHeight - tmp_height);
                            funs.setObjOpt(obj.dataIndex, tmp_opt, false, false);
                            break;
                    }
                    __this.saveStatus();
                } else {
                    funs.setObjOpt(obj.dataIndex, opt, false, false);
                }
            });
        },
        /**
         * 循环选中的元素
         * @param startI 开始循环的下标
         * @param callback
         */
        eachObj: function (startI, callback) {
            for (var i = startI; i < this.objList.length; i++) {
                if (typeof this.objList[i] !== 'undefined' && this.objList[i] != null) {
                    callback(this.objList[i]);
                }
            }
        }
    };
    //多选队列
    __this.dragInfo.objList = [];
    // ctrl + 鼠标左键
    __this.ctrlAndMousedown = {
        is_ctrl: false,         // 是否按住ctrl键
        is_mousedown: false,    // 是否按住鼠标左键
        is_keyup: true,         // 是否按键松开，计算选中内容
        startX: 0,
        startY: 0,
        endX: 0,
        endY: 0,
    };

    __this.canvas.addEventListener('dblclick', function (e) {
        funs.findClickObj(e.offsetX - 1, e.offsetY - 1, function (obj) {
            if (obj.isParent) {
                if (obj.id !== 'data_main') {
                    return;
                }
            }
            if (/margin_top|margin_bottom|margin_left|margin_right|margin_centen|scale_[1-8]/.test(obj.id)) return;
            if (typeof __opt.onDBLClick === 'function') __opt.onDBLClick(obj);
        });
    });

    __this.canvas.addEventListener('mousedown', function (e) {
        if (__this.ctrlAndMousedown.is_ctrl) {              // 鼠标多选事件开始
            __this.ctrlAndMousedown.is_mousedown = true;    // 鼠标左键选中
            __this.ctrlAndMousedown.startX = e.offsetX;
            __this.ctrlAndMousedown.startY = e.offsetY;
        } else {
            // 点击位置在选中区域内，设置开始拖动
            if (__this.dragInfo.objList && __this.dragInfo.objList.length && !__this.dragInfo.startDrag) {
                __this.dragInfo.scale_id = '';
                var isStartDrag = false;
                //更新当前选中的位置信息
                for (var i = 0; i < __this.DisplayList.length; i++) {
                    for (var j = 0; j < __this.dragInfo.objList.length; j++) {
                        if (__this.DisplayList[i].id == __this.dragInfo.objList[j].id) {
                            __this.dragInfo.objList[j] = __this.DisplayList[i];
                        }
                    }
                }
                for (var i = 0; i < __this.dragInfo.objList.length; i++) {
                    // 判断是否选中点在区域里面，排除固定栏的数据
                    if (funs.contains(e.offsetX - 1, e.offsetY - 1, __this.dragInfo.objList[i].rect)) {
                        if (['head', 'data_header', 'data_main', 'data_footer', 'footer'].indexOf(__this.dragInfo.objList[i].id) === -1) {
                            isStartDrag = true;
                            break;
                        }
                    }
                }
                if (isStartDrag && __this.dragInfo.objList.length > 1) {
                    __this.dragInfo.startDrag = true;
                    __this.dragInfo.upX = e.offsetX;
                    __this.dragInfo.upY = e.offsetY;
                }
            }
            if (__this.dragInfo.startDrag) return;
            funs.findClickObj(e.offsetX - 1, e.offsetY - 1, function (obj) {

                if (/margin_top|margin_bottom|margin_left|margin_right|margin_centen/.test(obj.id)) {
                    __this.selectObj = null;
                    __this.dragInfo.objList.length = 0;
                    funs.drawScalePoint(false);
                    if (typeof __opt.onPageClick === 'function') __opt.onPageClick(__this.opt);
                    return;
                }
                if (typeof __opt.onClick === 'function') __opt.onClick(obj);

                if (!/scale_[1-8]/.test(obj.id)) {
                    __this.selectObj = obj;
                    draw.Selected(obj, false);
                    __this.dragInfo.obj = obj;
                    if (e.shiftKey) {    // 按住shift选中，然后缩放处理
                        funs.drawScalePoint(false);
                        var hasSign = __this.dragInfo.isInObjList(obj);
                        if (!hasSign) __this.dragInfo.objList.push(funs.extend({}, obj));
                    } else {
                        __this.dragInfo.objList.length = 0;
                        __this.dragInfo.objList = [];
                        __this.dragInfo.objList.push(funs.extend({}, obj));
                    }
                    __this.dragInfo.scale_id = '';
                } else {
                    __this.dragInfo.scale_id = obj.id;
                }
                // storage.displayable = obj;
                // if (!obj.isDrag){
                //     storage.displayable = undefined;
                //     return;
                // }
                __this.dragInfo.startDrag = true;
                __this.dragInfo.upX = e.offsetX;
                __this.dragInfo.upY = e.offsetY;
                // 选中多个元素回调
                // if (__this.dragInfo.objList.length > 1 && typeof __opt.onSelectMultipleObj === 'function') {
                //     __opt.onSelectMultipleObj();
                // }
            });
        }
    });

    __this.canvas.addEventListener('mouseup', function (e) {
        if (__this.ctrlAndMousedown.is_ctrl) { // 鼠标多选事件结束
            // //清空上一个选中内容
            __this.dragInfo.objList = [];
            //清空缩放点
            funs.drawScalePoint(false);
            __this.ctrlAndMousedown.is_mousedown = false;
            __this.ctrlAndMousedown.is_keyup = true;
            funs.findObjsInArea(__this.ctrlAndMousedown.startX, __this.ctrlAndMousedown.startY, e.offsetX, e.offsetY);
            if (typeof __opt.onSelectMoveAfter === 'function') __opt.onSelectMoveAfter(__this.dragInfo.objList);
        } else {
            __this.dragInfo.startDrag = false;
            _VariableStorage.Snaplines.length = 0;
            var tmp_layout = ['auto_left', 'auto_right', 'auto_middle'];

            // if (__this.dragInfo.obj && typeof __opt.onMoveAfter === 'function') {
            //         __opt.onMoveAfter(funs.findObj('id', __this.dragInfo.obj.id));
            // }

            // 本次点击的元素
            var clickObj = funs.findClickObj(e.offsetX - 1, e.offsetY - 1, null);
            // console.log('mouseup', 'mouseup');
            var clickObjMoveX = 0;  // 选中元素移动距离
            var isFound = false;    // 排除重复计算
            // 设置自动对齐
            if (__this.dragInfo.objList.length) {
                for (var i = 0; i < __this.dragInfo.objList.length; i++) {
                    if (__this.dragInfo.obj && __this.dragInfo.obj.id === __this.dragInfo.objList[i].id) {
                        isFound = true;
                    }
                    // 控件自动对齐处理
                    var dataIndex = __this.dragInfo.objList[i].dataIndex;
                    var obj = __this.DisplayList[dataIndex];
                    var moveX = funs.objLayoutHandler(obj).x;
                    if (obj.id === clickObj.id || tmp_layout.indexOf(obj.layout) > -1) clickObjMoveX = moveX;
                    obj.rect = funs.getRect(obj.x, obj.y, obj.borderStyle.width, obj.borderStyle.height);

                }
            }

            if (__this.dragInfo.obj && !isFound) {
                var dataIndex = __this.dragInfo.obj.dataIndex;
                var obj = __this.DisplayList[dataIndex];
                var moveX = funs.objLayoutHandler(obj).x;
                if (obj.id === clickObj.id || tmp_layout.indexOf(obj.layout) > -1) clickObjMoveX = moveX;
                obj.rect = funs.getRect(obj.x, obj.y, obj.borderStyle.width, obj.borderStyle.height);
            }

            // 有选中内容时，才处理缩放点 移动缩放点 clickObjMoveX
            if (clickObjMoveX > 0 || obj && obj.layout == 'auto_right') {
                // if(obj.layout == 'auto_right' && __this.dragInfo.objList.length > 1){
                //     //清空缩放点
                //     funs.drawScalePoint(false);
                // }
                //获取缩放点1的位置
                var tmpScale_1;
                funs.findObj('id', 'scale_1', null, function (obj) {
                    tmpScale_1 = obj.x + 2;
                });
                for (var i = 1; i <= 8; i++) {
                    funs.findObj('id', 'scale_' + i, null, function (obj) {
                        funs.setObjOpt(obj.dataIndex, { x: (obj.x - (tmpScale_1 - clickObjMoveX)) }, false, false);
                    });
                }
            }
            if (__this.dragInfo.obj && typeof __opt.onMoveAfter === 'function') {
                // if(tmp_layout.indexOf(obj.layout) > -1){
                if (!/margin_top|margin_bottom|margin_left|margin_right|margin_centen/.test(clickObj.id)) {
                    var tmp_obj = funs.findObj('id', __this.dragInfo.obj.id);
                    funs.setObjPlace(tmp_obj, { x: tmp_obj.x });
                    for (var i = 0; i < __this.DisplayList.length; i++) {
                        if ((String(__this.DisplayList[i].rect[0].x) === String(tmp_obj.rect[0].x) ||
                            String(__this.DisplayList[i].rect[1].x) === String(tmp_obj.rect[0].x) ||
                            String(__this.DisplayList[i].rect[0].x) === String(tmp_obj.rect[1].x) ||
                            String(__this.DisplayList[i].rect[1].x) === String(tmp_obj.rect[1].x)) &&
                            tmp_obj.layout != 'auto' && tmp_obj.layout != undefined
                        ) {
                            funs.AutoLayout(tmp_obj);
                            break;
                        }
                    }
                    __opt.onMoveAfter(tmp_obj);

                }
                // } else{
                //     __opt.onMoveAfter(funs.findObj('id', __this.dragInfo.obj.id));
                // }
            }
            // 保存操作状态
            if (!/margin_top|margin_bottom|margin_left|margin_right|margin_centen/.test(clickObj.id)) {
                __this.saveStatus(true);
            }
            draw.drawAll();
        }
    });

    __this.canvas.addEventListener('mousemove', function (e) {
        // 多选处理
        if (__this.ctrlAndMousedown.is_ctrl) {
            // 记录鼠标多选的结束位置
            __this.ctrlAndMousedown.endX = e.offsetX;
            __this.ctrlAndMousedown.endY = e.offsetY;
            draw.drawAll();
            return;
        }

        var x = 0, y = 0;

        if (!__this.dragInfo.startDrag) return;

        // var x = e.offsetX - __this.dragInfo.upX;
        // var y = e.offsetY - __this.dragInfo.upY;
        var x = funs.decimalSub(e.offsetX, __this.dragInfo.upX);
        var y = funs.decimalSub(e.offsetY, __this.dragInfo.upY);

        if (!_VariableStorage.snaplines_x || Math.abs(x) > 1 || /scale_[1-8]/.test(__this.dragInfo.obj.id)) {
            __this.dragInfo.upX = e.offsetX;
        } else {
            x = 0;
        }
        if (!_VariableStorage.snaplines_y || Math.abs(y) > 1 || /scale_[1-8]/.test(__this.dragInfo.obj.id)) {
            __this.dragInfo.upY = e.offsetY;
        } else {
            y = 0;
        }

        if (isNaN(x)) x = 0;
        if (isNaN(y)) y = 0;

        // if (x==0 || y==0) return;

        if (__this.dragInfo.obj && __this.dragInfo.obj.id && __this.dragInfo.scale_id) {
            //缩放元素
            for (var i = 0; i < __this.dragInfo.objList.length; i++) {
                // var _obj = funs.findObj('id', __this.dragInfo.obj.id);
                var _obj = funs.findObj('id', __this.dragInfo.objList[i].id);
                var _x = _obj.x,
                    _y = _obj.y,
                    _w = _obj.borderStyle.width,
                    _h = _obj.borderStyle.height;
                switch (__this.dragInfo.scale_id) {
                    case 'scale_1':
                        _w = funs.decimalAdd(_w, funs.decimalMul(x, -1));
                        _h = funs.decimalAdd(_h, funs.decimalMul(y, -1));
                        _x = funs.decimalAdd(_x, x);
                        _y = funs.decimalAdd(_y, y);
                        break;
                    case 'scale_2':
                        _h = funs.decimalAdd(_h, funs.decimalMul(y, -1));
                        _y = funs.decimalAdd(_y, y);
                        break;
                    case 'scale_3':
                        _w = funs.decimalAdd(_w, x);
                        _h = funs.decimalAdd(_h, funs.decimalMul(y, -1));
                        _y = funs.decimalAdd(_y, y);
                        break;
                    case 'scale_4':
                        _w = funs.decimalAdd(_w, funs.decimalMul(x, -1));
                        _x = funs.decimalAdd(_x, x);
                        break;
                    case 'scale_5':
                        _w = funs.decimalAdd(_w, x);
                        break;
                    case 'scale_6':
                        _w = funs.decimalAdd(_w, funs.decimalMul(x, -1));
                        _h = funs.decimalAdd(_h, y);
                        _x = funs.decimalAdd(_x, x);
                        break;
                    case 'scale_7':
                        _h = funs.decimalAdd(_h, y);
                        break;
                    case 'scale_8':
                        _w = funs.decimalAdd(_w, x);
                        _h = funs.decimalAdd(_h, y);
                        break;
                }

                if (_obj.type === 'text') {
                    if (_w < _obj.borderStyle.width && _w < _obj.font_size) {
                        _w = _obj.font_size;
                        _x = _obj.x;
                    }
                    if (_h < _obj.font_size) {
                        _h = _obj.font_size;
                        _y = _obj.y;
                    }
                }
                funs.setObjOpt(_obj.dataIndex, { x: _x, y: _y, borderStyle: { height: _h, width: _w }, selected: true }, false, false);

                if (_obj.id == __this.selectObj.id) draw.Selected(funs.extend({}, __this.DisplayList[_obj.dataIndex]));
            }

        } else {
            //移动元素
            for (var z = 0; z < __this.dragInfo.objList.length; z++) {
                // var obj = funs.findObj('id', __this.dragInfo.obj.id);
                var obj = funs.findObj('id', __this.dragInfo.objList[z].id);

                if (obj.isParent) {
                    //如果选择的栏对象,则左右不能移动
                    x = 0;
                    //移动栏对象里的子元素
                    // for (var i = 0; i < obj.childs.length; i++) {
                    //     funs.findObj('id', obj.childs[i], null, function (obj) {
                    //         funs.setObjOpt(obj.dataIndex, {y: (obj.y + y)}, false);
                    //     });
                    // }
                } else {
                    // 如果选择的不是栏对象,则判断是否移动到栏的区域内，是则要嵌套到栏内
                    obj.parentID = '';
                    for (var i = 0; i < __this.DisplayList.length; i++) {
                        var sub = __this.DisplayList[i];
                        if (sub.isParent) {
                            var index = funs.inArray(obj.id, sub.childs);
                            if (index !== -1) {
                                sub.childs.splice(index, 1);
                            }

                            if (funs.contains((obj.x + x), (obj.y + y), sub.rect)) {
                                obj.parentID = sub.id;
                                funs.setObjPlace(obj, { x: obj.x, y: obj.y });
                                sub.childs.push(obj.id);
                            }
                        }
                    }

                    if (e.shiftKey && !e.altKey) x = 0;
                    if (!e.shiftKey && e.altKey) y = 0;
                }

                //移动元素时移动缩放点，缩放点跟元素是分开画的
                if (__this.selectObj && obj.isDrag && obj.id === __this.selectObj.id) {
                    for (var i = 1; i <= 8; i++) {
                        funs.findObj('id', 'scale_' + i, null, function (obj) {
                            funs.setObjOpt(obj.dataIndex, { x: (obj.x + x), y: (obj.y + y) }, false, false);
                        });
                    }
                }

                // funs.setObjOpt(obj.dataIndex, {x: (obj.x + x), y: (obj.y + y), selected: true});
                if (obj.isDrag) funs.setObjOpt(obj.dataIndex, { x: funs.decimalAdd(obj.x, x), y: funs.decimalAdd(obj.y, y), selected: true }, false, false);
            }
        }

        //绘制对齐线
        // if(__this.dragInfo.objList.length < 2) draw.Snaplines(__this.DisplayList[__this.dragInfo.obj.dataIndex]);
        if (__this.dragInfo.obj) {
            draw.Snaplines(__this.DisplayList[__this.dragInfo.obj.dataIndex]);
        } else {
            draw.drawAll();
        }
    });

    window.addEventListener('keyup', function (event) {
        // 标记ctrl松开，结束鼠标多选事件
        // console.log('keyup', event);
        if (__this.ctrlAndMousedown.is_ctrl) {
            __this.ctrlAndMousedown.is_ctrl = false;
            // console.log('keyup ctrlAndMousedown', JSON.parse(JSON.stringify(__this.ctrlAndMousedown)));
            // 鼠标多选事件结束，计算选中区域的元素
            if (__this.ctrlAndMousedown.is_mousedown) {     // 鼠标松开时，不计算
                __this.ctrlAndMousedown.is_mousedown = false;
                funs.findObjsInArea(
                    __this.ctrlAndMousedown.startX,
                    __this.ctrlAndMousedown.startY,
                    __this.ctrlAndMousedown.endX,
                    __this.ctrlAndMousedown.endY
                );
            }
        }
        //报表设计器设计页面，layer弹窗打开的情况下不用执行（ctrl+c ,ctrl+v 冲突）
        if (document.querySelectorAll('.layui-layer').length > 0) return false;

        if (event.ctrlKey && event.keyCode == 90) {
            __this.backStatus();
            return;
        } else if (event.ctrlKey && event.keyCode == 89) {
            __this.recoveryStatus();
            return;
        } else if (event.ctrlKey && event.keyCode == 67) {
            __this.copySelectObj();
            return false;
        } else if (event.ctrlKey && event.keyCode == 86) {
            __this.pasteSelectObj();
            return false;
        }
    });

    window.addEventListener('keydown', function (event) {
        var speed = 1;
        // 标记ctrl按下
        if (event.ctrlKey) {
            __this.ctrlAndMousedown.is_ctrl = true;
            // var clickObj = funs.findClickObj(event.offsetX - 1, event.offsetY - 1, null);
            // console.log('clickObj',clickObj)
            // console.log(event)
        }

        if (!__this.selectObj) return;
        if (!event.shiftKey) return;
        // if(event.shiftKey) speed = 0.1;
        for (var i = 0; i < __this.dragInfo.objList.length; i++) {
            var obj = funs.extend({}, __this.DisplayList[__this.dragInfo.objList[i].dataIndex]);
            switch (event.keyCode) {
                //方向左
                case 37:
                    funs.setObjOpt(obj.dataIndex, { x: funs.decimalSub(obj.x, speed) }, true, false);
                    if (obj.id === __this.selectObj.id) {
                        draw.Selected(funs.extend({}, __this.DisplayList[obj.dataIndex]));
                        draw.Snaplines(__this.DisplayList[obj.dataIndex]);
                    }
                    break;
                //方向上
                case 38:
                    funs.setObjOpt(obj.dataIndex, { y: funs.decimalSub(obj.y, speed) }, true, false);
                    if (obj.id === __this.selectObj.id) {
                        draw.Selected(funs.extend({}, __this.DisplayList[obj.dataIndex]));
                        draw.Snaplines(__this.DisplayList[obj.dataIndex]);
                    }
                    break;
                //方向右
                case 39:
                    funs.setObjOpt(obj.dataIndex, { x: funs.decimalAdd(obj.x, speed) }, true, false);
                    if (obj.id === __this.selectObj.id) {
                        draw.Selected(funs.extend({}, __this.DisplayList[obj.dataIndex]));
                        draw.Snaplines(__this.DisplayList[obj.dataIndex]);
                    }
                    break;
                //方向下
                case 40:
                    funs.setObjOpt(obj.dataIndex, { y: funs.decimalAdd(obj.y, speed) }, true, false);
                    if (obj.id === __this.selectObj.id) {
                        draw.Selected(funs.extend({}, __this.DisplayList[obj.dataIndex]));
                        draw.Snaplines(__this.DisplayList[obj.dataIndex]);
                    }
                    break;
            }
        }
    });

    __this.opt = __opt;

    /**
     * 导出对象配置、页面配置信息
     */
    __this.export = function () {
        var style = [];
        for (var i = 5; i < __this.DisplayList.length; i++) {
            if (typeof __this.DisplayList[i] === 'undefined' || /scale_[1-8]/.test(__this.DisplayList[i].id)) continue;
            style.push(funs.extend(true, {}, __this.DisplayList[i]))
            // tmp.push(extend({}, self.DisplayList[i]));
            //
            // var tmp2 = tmp[tmp.length - 1].childs;
            // if (id_list.includes(tmp[tmp.length - 1].id)) {
            //     tmp[tmp.length - 1].isPrint = false;
            // }
            // tmp[tmp.length - 1].childs = [];
            // delete tmp[tmp.length - 1].rect;
            // delete tmp[tmp.length - 1].borderStyle.img;
            // for (var key in tmp2) {
            //     tmp[tmp.length - 1].childs.push(tmp2[key]);
            // }
        }
        return {
            "pageStyle": style,
            "pageConfig": __opt.pageConfig
        }
    };

    //设置页面属性
    __this.setPageConfig = function (options) {
        var margin = funs.extend(true, {}, __opt.pageConfig.margin, options.margin);
        __opt.pageConfig.margin = (typeof options.margin === 'undefined') ? __opt.pageConfig.margin : options.margin;
        __opt.margin = {
            top: margin.top * __opt.cmToDots,
            bottom: margin.bottom * __opt.cmToDots,
            left: margin.left * __opt.cmToDots,
            right: margin.right * __opt.cmToDots
        };

        __opt.pageConfig.printMode = (typeof options.printMode === 'undefined') ? __opt.pageConfig.printMode : options.printMode;
        __opt.pageConfig.dpi = (typeof options.dpi === 'undefined') ? __opt.pageConfig.dpi : options.dpi;
        __opt.pageConfig.drawBlank = (typeof options.drawBlank === 'undefined') ? __opt.pageConfig.drawBlank : options.drawBlank;
        __opt.pageConfig.unLimitPageHeight = (typeof options.unLimitPageHeight === 'undefined') ? __opt.pageConfig.unLimitPageHeight : options.unLimitPageHeight;
        __opt.pageConfig.blankLines = options.blankLines !== '' ? Number(options.blankLines) : options.blankLines; // 空白行数

        // // 隐藏0
        // if (typeof __opt.pageConfig.hideZero === 'undefined') {
        //     __opt.pageConfig.hideZero = true;
        // } else {
        //     __opt.pageConfig.hideZero = options.hideZero;
        // }

        // 设计页无限高
        if (typeof __opt.pageConfig.pageMaxHeight === 'undefined') {
            __opt.pageConfig.pageMaxHeight = true;
        } else {
            __opt.pageConfig.pageMaxHeight = options.pageMaxHeight;
        }
        __opt.pageConfig.pageMaxHeightSize = options.pageMaxHeightSize; //* __opt.cmToDots;

        // 金额格式化
        __opt.pageConfig.money_style = options.money_style;
        // console.log(12,__opt)
        funs.setCanvasWidth(options.width || __opt.pageConfig.width);
        funs.setCanvasHeight(options.height || __opt.pageConfig.height, __opt.pageConfig.pageMaxHeight && options.pageMaxHeightSize);
        funs.setMargin(margin);
        funs.findObj('id', 'head', null, function (obj) {
            funs.setObjOpt(obj.dataIndex, {}, false, false)
        });
        funs.findObj('id', 'data_header', null, function (obj) {
            funs.setObjOpt(obj.dataIndex, {}, false, false)
        });
        funs.findObj('id', 'data_main', null, function (obj) {
            funs.setObjOpt(obj.dataIndex, {}, false, false)
        });
        funs.findObj('id', 'data_footer', null, function (obj) {
            funs.setObjOpt(obj.dataIndex, {}, false, false)
        });
        funs.findObj('id', 'footer', null, function (obj) {
            funs.setObjOpt(obj.dataIndex, {}, false, false)
        });
        draw.drawAll();
    };

    /**
     * 多选元素对齐操作
     * @param type string -- left(左对齐)
     *                    -- center(水平居中)
     *                    -- right(右对齐)
     *                    -- top(上对齐)
     *                    -- middle(垂直居中)
     *                    -- bottom(下对齐)
     *                    -- sameWidth(相同宽度)
     *                    -- sameHeight(相同高度)
     */
    __this.multipleSelectorAlign = function (type) {

        if (__this.dragInfo.objList && __this.dragInfo.objList.length > 0) {
            if (__this.dragInfo.objList.length < 1) return;

            //更新选中内容的坐标
            for (var i = 0; i < __this.dragInfo.objList.length; i++) {
                for (var j = 0; j < __this.DisplayList.length; j++) {
                    if (__this.DisplayList[j].id == __this.dragInfo.objList[i].id) {
                        __this.dragInfo.objList[i] = __this.DisplayList[j];
                    }
                }
            }

            var layout = ['auto_left', 'auto_right', 'auto_middle'];//是否开启靠左、靠右、水平居中
            var tmp_objList = __this.dragInfo.objList.filter(function (item) {
                return layout.indexOf(item.layout) < 0;
            })
            switch (type) {
                case 'left': // 左对齐，找到左边最小值，然后设置同个值
                    var minX = tmp_objList[0].rect[0].x; // 最大的x坐标
                    __this.dragInfo.eachObj(1, function (obj) {
                        if (layout.indexOf(obj.layout) < 0 && obj.rect[0].x < minX) {
                            minX = obj.rect[0].x;
                        }
                    });

                    // 设置x坐标
                    __this.dragInfo.setSelectedOpt({ x: minX }, { type: 'Manycontrols', align: 'left' });
                    // 重绘画布
                    draw.drawAll();
                    break;
                case 'center': // 水平居中

                    var minX = tmp_objList[0].rect[0].x;//最近的值
                    var maxX = tmp_objList[0].rect[1].x;//最远的值
                    __this.dragInfo.eachObj(1, function (obj) {
                        if (layout.indexOf(obj.layout) < 0) {
                            if (obj.rect[0].x < minX) minX = obj.rect[0].x;
                            if (obj.rect[1].x > maxX) maxX = obj.rect[1].x;
                        }
                    })

                    var centerX = parseFloat(parseFloat(maxX - (maxX - minX) / 2).toFixed(0));
                    // 设置x坐标
                    __this.dragInfo.setSelectedOpt({ x: centerX }, { type: 'Manycontrols', align: 'center', center: centerX });
                    // 重绘画布
                    draw.drawAll();

                    break;
                case 'right': // 右对齐
                    var tmp_maxX = tmp_objList[0].rect[1].x;// 最右的x坐标
                    var maxX = tmp_objList[0].rect[0].x;
                    var maxWidth = tmp_objList[0].rect[1].x - tmp_objList[0].rect[0].x; //最右的x坐标 矩形的宽度
                    __this.dragInfo.eachObj(1, function (obj) {
                        if (layout.indexOf(obj.layout) < 0 && obj.rect[1].x > tmp_maxX) {
                            maxX = obj.rect[0].x;
                            maxWidth = obj.rect[1].x - obj.rect[0].x;
                            tmp_maxX = obj.rect[1].x;
                        }
                    });

                    // 设置x坐标
                    __this.dragInfo.setSelectedOpt({ x: maxX }, { type: 'Manycontrols', align: 'right', maxWidth: maxWidth });
                    // 重绘画布
                    draw.drawAll();
                    break;
                case 'top':   // 上对齐  // FIXEM 缩放点，不移动，计算缩放点
                    var minY = __this.dragInfo.objList[0].y; // 最大的y坐标
                    __this.dragInfo.eachObj(1, function (obj) {
                        if (obj.y < minY) {
                            minY = obj.y;
                        }
                    });

                    // 设置y坐标
                    __this.dragInfo.setSelectedOpt({ y: minY }, { type: 'Manycontrols', align: 'top' });
                    // 重绘画布
                    draw.drawAll();
                    break;
                case 'middle': // 垂直居中 选取高度最高的一个元素，获取 y 的最高点跟最低点，计算垂直方向的中间位置，然后移动剩下元素的y坐标，调整到垂直方向的中间位置一样
                    var minY = __this.dragInfo.objList[0].rect[0].y;//最低的值
                    var maxY = __this.dragInfo.objList[0].rect[2].y;//最高的值
                    __this.dragInfo.eachObj(1, function (obj) {
                        if (obj.rect[0].y < minY) {
                            minY = obj.rect[0].y;
                        }
                        if (obj.rect[2].y > maxY) {
                            maxY = obj.rect[2].y;
                        }
                    })

                    var centerY = parseFloat(parseFloat(maxY - (maxY - minY) / 2).toFixed(0));
                    // 设置y坐标
                    __this.dragInfo.setSelectedOpt({ y: centerY }, { type: 'Manycontrols', align: 'middle', center: centerY });
                    // 重绘画布
                    draw.drawAll();
                    break;
                case 'bottom': // 下对齐 读取最低的元素位置坐标，设置其他元素的位置一致

                    //获取离底部最近的矩形 y坐标 和 高度
                    var maxHeight = __this.dragInfo.objList[0].rect[2].y - __this.dragInfo.objList[0].rect[0].y;//容器的高度
                    var tmp_maxY = __this.dragInfo.objList[0].rect[2].y; // 容器最高的y坐标
                    var maxY = __this.dragInfo.objList[0].y;
                    __this.dragInfo.eachObj(1, function (obj) {
                        var tmp_maxHeight = obj.rect[2].y - obj.rect[0].y;
                        if (obj.rect[2].y > tmp_maxY) {
                            maxHeight = tmp_maxHeight;
                            tmp_maxY = obj.rect[2].y;
                            maxY = obj.y;
                        }
                    });

                    // 设置y坐标
                    __this.dragInfo.setSelectedOpt({ y: maxY }, { type: 'Manycontrols', align: 'bottom', maxHeight: maxHeight });
                    // 重绘画布
                    draw.drawAll();
                    break;
                case 'sameWidth': // 相同宽度 读取选中的控件宽度最小的值，将其他被选中的控件更新为相同宽度
                    // 查找选中元素中最大宽度的元素
                    var minWidth = __this.dragInfo.objList[0].width;
                    __this.dragInfo.eachObj(1, function (obj) {
                        if (obj.width < minWidth) {
                            minWidth = obj.width;
                        }
                    });
                    // 设置高度
                    __this.dragInfo.setSelectedOpt({ width: minWidth });
                    // 重绘画布
                    draw.drawAll();
                    break;
                case 'sameHeight': // 相同高度 读取选中的控件高度最小的值，将其他被选中的控件更新为相同高度
                    var minHeight = __this.dragInfo.objList[0].height;
                    __this.dragInfo.eachObj(1, function (obj) {
                        if (obj.height < minHeight) {
                            minHeight = obj.height;
                        }
                    });

                    // 设置高度
                    __this.dragInfo.setSelectedOpt({ height: minHeight });
                    // 重绘画布
                    draw.drawAll();
                    break;
                default: break;
            }
        }
    };

    //被选中的对象
    __this.selectObj = undefined;

    /**
     * 设置选中对象的属性
     * 属性说明
     * {
     *     isForPreview: '预览时显示',
     *     isForPrint: '打印时显示',
     *     layout: 控件对齐方式
     * }
     * @param options
     */
    __this.setSelectedOpt = function (options) {
        // if (typeof __this.selectObj !== 'undefined' && __this.selectObj != null) funs.setObjOpt(__this.selectObj.dataIndex, options, {selected: true});
        // 设置选中控件的属性
        var layout = ['auto_left', 'auto_right', 'auto_middle'];
        for (var i = 0; i < __this.dragInfo.objList.length; i++) {
            for (var j = 0; j < __this.DisplayList.length; j++) {
                if (__this.DisplayList[j].id == __this.dragInfo.objList[i].id) {
                    __this.dragInfo.objList[i] = __this.DisplayList[j];
                }
            }
        }
        //自动对齐排序
        if (__this.dragInfo.objList.length > 1) {
            //排序最大对齐
            var tmp_dragInfo = __this.dragInfo.objList.sort(function (obj1, obj2) {
                if (options.layout == 'auto_right') {
                    if (obj2.x < obj1.x) return -1;
                    else if (obj2.x > obj1.x) return 1;
                    else return 0;
                } else if (options.layout == 'auto_left') {
                    if (obj2.x > obj1.x) return -1;
                    else if (obj2.x < obj1.x) return 1;
                    else return 0;
                }
            })
            for (var i = 0; i < tmp_dragInfo.length; i++) {
                if (typeof tmp_dragInfo[i] !== 'undefined' && tmp_dragInfo[i] != null) {
                    if (layout.indexOf(options.layout) > -1) {
                        funs.setObjOpt(tmp_dragInfo[i].dataIndex, options, false, true);
                        var tmp_obj = funs.findObj('id', tmp_dragInfo[i].id)
                        funs.setObjPlace(tmp_obj, { x: tmp_obj.x, y: tmp_obj.y });
                    } else {
                        funs.setObjOpt(tmp_dragInfo[i].dataIndex, options, false, false);
                    }
                }
                draw.Selected(funs.extend({}, __this.DisplayList[tmp_dragInfo[i].dataIndex]), false);                //清空缩放点
                funs.drawScalePoint(false);
            }
            draw.drawAll();
        } else {
            if (typeof __this.selectObj !== 'undefined' && __this.selectObj != null) {
                funs.setObjOpt(__this.selectObj.dataIndex, options, false, true);
                //有开启自动水平居中，靠左，靠右
                if (layout.indexOf(options.layout) > -1) {
                    var tmp_obj = funs.findObj('id', __this.selectObj.id)
                    funs.setObjPlace(tmp_obj, { x: tmp_obj.x });
                }
            }
            if (__this.selectObj) {
                draw.Selected(funs.extend({}, __this.DisplayList[__this.selectObj.dataIndex]));
            }
        }
    };

    // 删除元素
    __this.delete = function (id, multi, callback) {
        if (!multi && __this.dragInfo.objList.length > 1) {
            var selectList = __this.dragInfo.objList.concat();
            for (var x = 0; x < selectList.length; x++) {
                if (!selectList[x].id) continue;
                __this.delete(selectList[x].id, true);
            }
            funs.AutoLayout();
            draw.drawAll();
            return;
        } else {
            funs.findObj('id', id, null, function (obj) {
                if (obj.isParent) {
                    for (var i = 0; i < obj.childs.length; i++) {
                        var _sub = funs.findObj('id', obj.childs[i]);
                        _sub && (delete __this.DisplayList[_sub.dataIndex]);
                    }
                }
                delete __this.DisplayList[obj.dataIndex];

                for (var i = 0; i < __this.DisplayList.length; i++) {
                    if (!__this.DisplayList[i]) continue;
                    if (__this.DisplayList[i].isParent) {
                        for (var j = 0; j < __this.DisplayList[i].childs.length; j++) {
                            if (__this.DisplayList[i].childs[j] == obj.id) {
                                __this.DisplayList[i].childs.splice(j, 1);
                                i = __this.DisplayList.length;
                                break;
                            }
                        }
                    }
                }
                var newList = [];
                for (var i = 0, index = 0; i < __this.DisplayList.length; i++) {
                    if (!__this.DisplayList[i]) continue;
                    newList.push(funs.extend(true, {}, __this.DisplayList[i], { dataIndex: index }));
                    index++;
                }

                __this.DisplayList = newList.concat();

                __this.selectObj = null;
                for (var x = 0; x < __this.dragInfo.objList.length; x++) {
                    if (!__this.dragInfo.objList[x].id) continue;
                    if (__this.dragInfo.objList[x].id === obj.id) {
                        __this.dragInfo.objList.splice(x, 1);
                    }
                }
                if (!/margin_top|margin_bottom|margin_left|margin_right|margin_centen|scale_[1-8]/.test(obj.id)) funs.drawScalePoint(false);
                if (!/scale_[1-8]/.test(obj.id)) __this.saveStatus(true);
            });
            funs.AutoLayout();
            if (!multi) draw.drawAll();
        }
    };

    // 增加元素
    __this.add = {
        // 文本
        text: function (options) {
            var obj = funs.extend(true, {}, objDefaultOptions, {
                'id': funs.createID('text'),
                'type': 'text',
                'x': funs.get('margin_centen') ? funs.get('margin_centen').x : 0,
                'y': funs.get('margin_centen') ? funs.get('margin_centen').y : 0,
                'autoHeight': true,
                'borderStyle': {
                    'top': 0,    // 上框线
                    'bottom': 0, // 下框线
                    'left': 0,   // 左框线
                    'right': 0,  // 右框线
                    'width': 120,  // 宽度
                    'height': 28   // 高度
                },
            }, options);
            obj.rect = funs.getRect(obj.x, obj.y, obj.textWidth, obj.font_size);
            delete obj.width;
            delete obj.height;
            funs.setObjWidthHeight(obj, obj);
            funs.setObjPlace(obj, obj);
            draw.addToList(obj);
            __this.saveStatus();
            return obj;
        },
        rect: function (options) {
            var obj = funs.extend(true, {}, objDefaultOptions, {
                'id': funs.createID('rect'),
                'type': 'rect',
                'x': funs.get('margin_centen') ? funs.get('margin_centen').x : 0,
                'y': funs.get('margin_centen') ? funs.get('margin_centen').y : 0,
                'isParent': true,// 是否为父元素
                'borderStyle': {
                    'top': 1,    // 上框线
                    'bottom': 1, // 下框线
                    'left': 1,   // 左框线
                    'right': 1,  // 右框线
                    'width': 120,  // 宽度
                    'height': 120   // 高度
                }
            }, options);
            obj.rect = funs.getRect(obj.x, obj.y, obj.textWidth, obj.font_size);
            delete obj.width;
            delete obj.height;
            funs.setObjWidthHeight(obj, obj);
            funs.setObjPlace(obj, obj);
            draw.addToList(obj);
            __this.saveStatus();
            return obj;
        },
        pageHeader: function () {
            if (!funs.findObj('id', 'head')) {
                __this.add.rect({
                    'id': 'head',
                    'isDrag': false,
                    'borderStyle': { 'height': 48 },
                    'title': { 'text': '页头' },
                    zIndex: 0,
                });
                __this.saveStatus();
                return true;
            } else {
                return false;
            }
        },
        pageFooter: function () {
            if (!funs.findObj('id', 'footer')) {
                __this.add.rect({
                    'id': 'footer',
                    'isDrag': false,
                    'borderStyle': { 'height': 48 },
                    'title': { 'text': '页脚' },
                    zIndex: 0,
                });
                __this.saveStatus();
                return true;
            } else {
                return false;
            }
        },
        dataHeader: function () {
            if (!funs.findObj('id', 'data_header')) {
                __this.add.rect({
                    'id': 'data_header',
                    'isDrag': false,
                    'borderStyle': { 'height': 48 },
                    'title': { 'text': '头' },
                    zIndex: 0,
                });
                __this.saveStatus();
                return true;
            } else {
                return false;
            }
        },
        dataBar: function () {
            if (!funs.findObj('id', 'data_main')) {
                __this.add.rect({
                    'id': 'data_main',
                    'isDrag': false,
                    'borderStyle': { 'height': 96 },
                    'title': { 'text': '主数据' },
                    zIndex: 0,
                });
                __this.saveStatus();
                return true;
            } else {
                return false;
            }
        },
        dataFooter: function () {
            if (!funs.findObj('id', 'data_footer')) {
                __this.add.rect({
                    'id': 'data_footer',
                    'isDrag': false,
                    'borderStyle': { 'height': 48 },
                    'title': { 'text': '尾' },
                    zIndex: 0,
                });
                __this.saveStatus();
                return true;
            } else {
                return false;
            }
        },

        //添加图片
        image: function (param) {
            var opt = funs.extend(true, {
                'id': funs.createID('icon'),
                'type': 'icon',
                'borderStyle': { 'width': 120, 'height': 120 },
                'title': { 'text': '图片' },
                'isParent': false,
            }, param);
            __this.add.rect(opt);
        },

        //添加条形码
        barcode: function (param) {
            var opt = funs.extend(true, {
                'id': funs.createID('barcode'),
                'type': 'barcode',
                'borderStyle': { 'width': 200, 'height': 50 },
                'title': { 'text': '条形码' },
                'isParent': false,
                'barCode': 1,
            }, param);
            __this.add.rect(opt);
        },

        //添加二维码
        qrcode: function (param) {
            var opt = funs.extend(true, {
                'id': funs.createID('qrcode'),
                'type': 'qrcode',
                'borderStyle': { 'width': 150, 'height': 150 },
                'title': { 'text': '二维码' },
                'isParent': false,
            }, param);
            __this.add.rect(opt);
        },

        // 添加小程序码
        miniCode: function (param) {
            var opt = funs.extend(true, {
                'id': funs.createID('miniCode'),
                'type': 'miniCode',
                'borderStyle': { 'width': 150, 'height': 150 },
                'title': { 'text': '小程序码' },
                'isParent': false,
            }, param);
            __this.add.rect(opt);
        },

        // 复制组件，根据id判断
        copyComponents: function (arr) {
            var adujstProp = function (item) {
                delete item.id;
                delete item.rect;
                delete item.x;
                delete item.y;
            };

            for (var i = 0; i < arr.length; i++) {
                var item = arr[i];
                if (item.id.indexOf('text') > -1) {
                    adujstProp(item);
                    __this.add.text(item);
                } else if (item.id.indexOf('icon') > -1) {
                    adujstProp(item);
                    __this.add.image(item);
                } else if (item.id.indexOf('barcode') > -1) {
                    adujstProp(item);
                    __this.add.barcode(item);
                } else if (item.id.indexOf('qrcode') > -1) {
                    adujstProp(item);
                    __this.add.qrcode(item);
                }
            }
        }
    };

    __this.findObj = funs.findObj;

    //保存操作
    __this.saveStatus = function (Clearrefresh) {
        var list = __this.DisplayList.concat();
        var temp = [];
        for (var i = 0; i < list.length; i++) {
            // if(list[i] && /scale_[1-8]/.test(list[i].id)) continue;
            temp[i] = {};
            funs.extend(true, temp[i], list[i]);
        }
        History.data.length = History.index;
        History.data[History.index] = temp;
        History.index++;
        if (History.index > 1024) {
            History.data.splice(0, 512)
            History.index -= 512;
        }
        if (!Clearrefresh) draw.drawAll();
    }

    //撤销操作 Ctrl + z
    __this.backStatus = function () {
        if (History.index <= 1) return false;
        var tmp = History.data[History.index-- - 2];
        __this.DisplayList = [];
        for (var i = 0; i < tmp.length; i++) {
            // if(tmp[i] && /scale_[1-8]/.test(tmp[i].id)) continue;
            __this.DisplayList[i] = {};
            funs.extend(true, __this.DisplayList[i], tmp[i]);
        }
        __this.dragInfo.objList.length = 0;
        draw.drawAll();
    };
    //恢复操作 Ctrl + y
    __this.recoveryStatus = function () {
        if (History.data.length <= History.index) return false;
        var tmp = History.data[History.index++];
        __this.DisplayList = [];
        for (var i = 0; i < tmp.length; i++) {
            // if(tmp[i] && /scale_[1-8]/.test(tmp[i].id)) continue;
            __this.DisplayList[i] = {};
            funs.extend(true, __this.DisplayList[i], tmp[i]);
        }
        draw.drawAll();
    };

    // 复制控件  Ctrl + c
    // 存放在 __this.dragInfo.copyList
    __this.copySelectObj = function () {
        // 复制多选队列到复制队列中 __this.dragInfo.objList;
        // 复制选中对象 __this.selectObj
        if (__this.dragInfo.objList.length > 0) {   // 选中多个
            __this.dragInfo.copyList = __this.dragInfo.objList; // JSON.parse(JSON.stringify(__this.dragInfo.objList));
        } else {                                    // 选中一个
            if (__this.selectObj) {
                __this.dragInfo.copyList = [__this.selectObj]; // [JSON.parse(JSON.stringify(__this.selectObj))];
            }
        }
    };

    // 粘贴复制控件  Ctrl + v
    __this.pasteSelectObj = function () {
        // 已经复制
        if (__this.dragInfo.copyList.length > 0) {
            // 添加对应的类型
            var copyList = __this.dragInfo.copyList;
            __this.dragInfo.copyList = [];  // 清空数据，防止重复粘贴
            __this.add.copyComponents(JSON.parse(JSON.stringify(copyList)));
        }
    };

    init.call(this);
}
